# coding: utf-8
# pylint: disable=no-member

import logging

from django.core.exceptions import ObjectDoesNotExist
from django.core.urlresolvers import reverse
from django.utils.http import urlencode
from django.utils.translation import ugettext_lazy as _

from horizon import tables, messages

from openstack_dashboard import api

from tionix_dashboard.clients import TNXHypervisorsClient
from tionix_dashboard.clients.exceptions import UrlNotAvailableError
from tionix_dashboard.dashboards.admin.hypervisors.helpers import (
    ConnectedDevice)
from tionix_dashboard.helpers.actions import (
    TnxFilterAction, MultiLinkAction, TnxUpdateAction, TnxBatchActionMixin)
from tionix_dashboard.dashboards.admin.hypervisors.power_actions import (
    POWER_OFF_STATE, POWER_ON_STATE)
from tionix_dashboard.dashboards.admin.hypervisors.states import (
    STATE_UP, STATE_DOWN)

logger = logging.getLogger(__name__)


class TnxHypervisorUpdateRow(tables.Row):
    """ Action update hypervisor row in grid. """
    ajax = True

    def get_data(self, request, obj_id):
        try:
            hypervisor = TNXHypervisorsClient.get(request, obj_id)
        except UrlNotAvailableError:
            raise
        except Exception as err:
            logger.critical(err)
            raise ObjectDoesNotExist()
        return hypervisor


class TnxImageUpdateCell(tables.UpdateAction):
    """
    Action updated hypervisor's record with new value of cell "Image for PXE".
    """
    def allowed(self, request, datum, cell):
        return api.keystone.keystone_can_edit_project()

    def update_cell(self, request, datum, obj_id, cell_name, new_cell_value):
        """ Inline update of image-cell. """
        if cell_name == 'image':
            if new_cell_value.isdigit():
                new_cell_value = int(new_cell_value)
            try:
                TNXHypervisorsClient.assign_hypervisor_image(
                    request, obj_id, int(new_cell_value))
            except Exception as err:
                logger.error(unicode(err))
                msg = _('Failed to assign image '
                        'for hypervisors: "{0}"').format(new_cell_value)
                messages.error(request, msg)


class TnxNodeUpdateCell(TnxUpdateAction):
    """ Action update hypervisor title """

    def update_cell(self, request, datum, obj_id, cell_name, new_cell_value):
        """
        Update cell
        """
        TNXHypervisorsClient.update(
            request, obj_id, **{cell_name: new_cell_value})


class TnxSetReserveAction(tables.UpdateAction):
    """ Action set hypervisor as reserved. """
    def allowed(self, request, datum, cell):
        return api.keystone.keystone_can_edit_project()

    def update_cell(self, request, datum, obj_id, cell_name, new_cell_value):
        """ Inline update of is_reserved-cell. """
        if cell_name == 'is_reserved':
            TNXHypervisorsClient.set_reserved_flag_hypervisor(
                request, obj_id, new_cell_value)


class SingleLinkAction(tables.LinkAction):
    """Link action for single object"""

    def get_link_url(self, datum=None):
        datum_id = datum.id if datum else None
        params = urlencode({'id': datum_id})
        return '?'.join([reverse(self.url), params])


class HypervisorExtraAction(tables.LinkAction):
    """ Action show extra information for selected hypervisor. """
    name = 'hypervisors'
    verbose_name = _('More')
    url = "horizon:admin:hypervisors:extra"
    classes = ("ajax-modal", )
    policy_rules = (('identity', 'admin_required'), )

    def get_link_url(self, datum=None):
        hypervisor_id = datum.id if datum else None
        return reverse(self.url, args=[hypervisor_id])


class RestartHypervisorAction(SingleLinkAction):
    """ Action restart hypervisor. """
    name = 'restart'
    verbose_name = _('Restart')
    policy_rules = (('identity', 'admin_required'), )
    url = "horizon:admin:hypervisors:restart"
    classes = ("ajax-modal", )

    def allowed(self, request, datum):
        allowed = False

        if datum and datum.tnx_devices:
            if datum.tnx_power_state is None:
                allowed = datum.state == STATE_UP
            else:
                allowed = datum.tnx_power_state == POWER_ON_STATE

        return allowed


class HypervisorMultiRestartAction(MultiLinkAction):
    """ Action shut down hypervisor. """
    name = 'multi_restart'
    verbose_name = _('Restart')
    url = "horizon:admin:hypervisors:restart"
    policy_rules = (('identity', 'admin_required'), )


class PowerOnHypervisorAction(TnxBatchActionMixin, tables.BatchAction):
    """ Action power on hypervisor. """
    name = 'power_on'
    verbose_name = _('Power on')
    policy_rules = (('identity', 'admin_required'), )

    @staticmethod
    def action_present(count):
        return _("Power on")

    @staticmethod
    def action_past(count):
        return _("Power on")

    def action(self, request, datum_id):
        TNXHypervisorsClient.power_on_hypervisor(request, datum_id)

    def allowed(self, request, datum):
        allowed = True
        if datum:
            allowed = False
            if datum.tnx_devices:
                if datum.tnx_power_state is None:
                    allowed = datum.state == STATE_DOWN
                else:
                    allowed = datum.tnx_power_state == POWER_OFF_STATE

        return allowed


class PowerOffHypervisorAction(SingleLinkAction):
    """ Action power off hypervisor. """
    name = 'power_off'
    verbose_name = _('Power off')
    url = "horizon:admin:hypervisors:power_off"
    classes = ("ajax-modal", )
    policy_rules = (('identity', 'admin_required'), )

    def allowed(self, request, datum):
        allowed = False

        if datum and datum.tnx_devices:
            allowed = datum.tnx_power_state == POWER_ON_STATE

        return allowed


class HypervisorMultiPowerOffAction(MultiLinkAction):
    """ Action shut down hypervisor. """
    name = 'multi_power_off'
    verbose_name = _('Power off')
    url = "horizon:admin:hypervisors:power_off"
    policy_rules = (('identity', 'admin_required'), )


class StartReservationHypervisorAction(SingleLinkAction):
    """ Action start reservation hypervisor. """
    name = 'markreserved'
    verbose_name = _('Move to reserve')
    policy_rules = (('identity', 'admin_required'), )
    url = "horizon:admin:hypervisors:markreserved"
    classes = ("ajax-modal", )

    def allowed(self, request, datum):
        allowed = False

        if datum and datum.tnx_devices:
            # move to reserv only if device state is known
            allowed = datum.tnx_power_state == POWER_ON_STATE

        return allowed


class HypervisorMultiStartReservationAction(MultiLinkAction):
    """ Action start reservation hypervisor. """
    name = 'multi_markreserved'
    verbose_name = _('Move to reserve')
    url = "horizon:admin:hypervisors:markreserved"
    policy_rules = (('identity', 'admin_required'), )


class HypervisorMultiShutdownAction(MultiLinkAction):
    """ Action shut down hypervisor. """
    name = 'multi_shutdown'
    verbose_name = _('Shutdown')
    url = "horizon:admin:hypervisors:shutdown"
    policy_rules = (('identity', 'admin_required'), )


class HypervisorShutdownAction(SingleLinkAction):
    """ Action shut down hypervisor. """
    name = 'shutdown'
    verbose_name = _('Shutdown')
    url = "horizon:admin:hypervisors:shutdown"
    classes = ("ajax-modal", )
    policy_rules = (('identity', 'admin_required'), )

    def allowed(self, request, datum):
        allowed = False

        if datum and datum.tnx_devices:
            allowed = datum.tnx_power_state == POWER_ON_STATE
        return allowed


class HypervisorMultiAssignImageAction(MultiLinkAction):
    """ Multi action assign image for hypervisor. """
    name = 'multi_assign_image'
    verbose_name = _('Assign Image')
    url = "horizon:admin:hypervisors:assign_image"
    policy_rules = (('identity', 'admin_required'), )


class TnxAdminHypervisorsFilterAction(TnxFilterAction, tables.FilterAction):

    name = "filter_admin_hypervisors"
    filter_type = "server"

    filter_choices = (
        ('hostname', _("Hostname"), False),
        ('node_title', _("Node title"), False),
        ('hypervisor_type', _("Type"), False),
        ('image', _("Assigned image"), False),
        ('state', _("State"), False),
        ('power_state', _("Power state"), False),
        ('is_reserved', _("Reserved"), False),
    )


class HypervisorUpdateAction(SingleLinkAction):
    """Hypervisor update action"""
    name = 'update'
    verbose_name = _('Edit hypervisor')
    url = "horizon:admin:hypervisors:update"
    classes = ("ajax-modal", )
    policy_rules = (('identity', 'admin_required'), )


class HypervisorMultiUpdateAction(MultiLinkAction):
    """ Multi update action."""
    name = 'multi_update'
    verbose_name = _('Edit hypervisors')
    url = "horizon:admin:hypervisors:update"
    policy_rules = (('identity', 'admin_required'), )


class TnxConnectedDeviceUpdateRow(tables.Row):
    ajax = True

    def get_data(self, request, obj_id):
        hypervisor = self.table.kwargs['hypervisor']
        devices = getattr(hypervisor, 'tnx_devices', [])
        for name, port, default, initialized, device_id in devices:
            if obj_id == str(device_id):
                return ConnectedDevice(hypervisor.id, name, port, default,
                                       initialized, device_id)
        return None


class SetDefaultPowerDeviceUpdateCell(tables.UpdateAction):
    def allowed(self, request, datum, cell):
        return not datum.is_default

    def update_cell(self, request, datum, obj_id, cell_name, new_cell_value):
        result = True
        if new_cell_value:
            try:
                TNXHypervisorsClient.set_default_power_device_for_node(
                    request=request,
                    node_id=datum.hypervisor_id,
                    power_device_id=datum.id
                )
            except Exception as err:
                logger.error(unicode(err))
                messages.error(request, _('Failed to set default device.'))
                result = False

        return result


class SetPowerDevicePortUpdateCell(tables.UpdateAction):
    def allowed(self, request, datum, cell):
        cell.column.form_field.choices = [
            (i, i)
            for i in cell.row.table.devices_coil_map.get(datum.id, [])
        ]
        return True

    def update_cell(self, request, datum, obj_id, cell_name, new_cell_value):
        result = True
        if datum.device_port != new_cell_value:
            new_cell_value = (
                int(new_cell_value) if new_cell_value.isdigit() else
                new_cell_value
            )
            try:
                TNXHypervisorsClient.set_power_device_port_for_node(
                    request=request,
                    node_id=datum.hypervisor_id,
                    power_device_id=datum.id,
                    power_device_port=new_cell_value
                )
            except Exception as err:
                logger.error(unicode(err))
                messages.error(request, _('Failed to set coil number.'))
                result = False

        return result
