# coding: utf-8

import logging

from django.forms.fields import BooleanField, ChoiceField
from django.forms.widgets import RadioSelect
from django.http import HttpResponseRedirect
from django.template import defaultfilters as filters
from django.utils.functional import cached_property

from django.utils.translation import ugettext_lazy as _

from horizon import tables

from openstack_dashboard.dashboards.admin.hypervisors import (
    tables as admin_tables
)
from tionix_dashboard.clients import TNXPowerDevicesClient

from tionix_dashboard.clients.exceptions import UrlNotAvailableError
from tionix_dashboard.clients.pxe_images import TNXPXEImagesClient
from tionix_dashboard.dashboards.tionix.scheduler.actions import (
    HypervisorSchedulerMultiAction, HypervisorSchedulerAction)
from tionix_dashboard.helpers.fields import EmptyCharField
from tionix_dashboard.helpers.tables import (
    DTTnxPaginationMixin, ColoredRowMixin, DataTableFirstLastPagination,
    InstancesFilterPaginationMixin)
from tionix_dashboard.dashboards.admin.hypervisors import actions
from tionix_dashboard.helpers.views import RefererRedirectMixin

logger = logging.getLogger(__name__)


POWER_STATE_DISPLAY_CHOICES = (
    ('None', '-'),
    ('power_on', _('On')),
    ('power_off', _('Off'))
)

STATE_DISPLAY_CHOICES = (
    ('up', _('Up')),
    ('down', _('Down'))
)

STATUS_DISPLAY_CHOICES = (
    ('enabled', _('enabled')),
    ('disabled', _('disabled'))
)


class TnxAdminHypervisorsTable(DTTnxPaginationMixin,
                               ColoredRowMixin,
                               DataTableFirstLastPagination,
                               InstancesFilterPaginationMixin,
                               tables.DataTable):
    """
    Hypervisors table extension.
    """

    POWER_STATE_CHOICES = (
        (None, None),
        ('power_on', True),
        ('power_off', False)
    )

    hostname = tables.Column(
        "hypervisor_hostname",
        link="horizon:admin:hypervisors:detail",
        attrs={'data-type': 'naturalSort'},
        verbose_name=_("Hostname"))

    node_title = tables.Column(
        lambda x: unicode(x.tnx_node_title or ''),
        form_field=EmptyCharField(max_length=100),
        verbose_name=_("Node title"),
        update_action=actions.TnxNodeUpdateCell)

    hypervisor_type = tables.Column(
        "hypervisor_type",
        verbose_name=_("Type"))

    image = tables.Column(
        lambda x: unicode(x.tnx_image_name or ''),
        verbose_name=_("Assigned image"),
        form_field=ChoiceField(),
        update_action=actions.TnxImageUpdateCell)

    state = tables.Column(
        "state",
        verbose_name=_('State'),
        classes=('hypervisor_state', ),
        display_choices=STATE_DISPLAY_CHOICES,
    )

    power_state = tables.Column(
        lambda x: unicode(getattr(x, 'tnx_power_state', '')),
        verbose_name=_('Power state'),
        status_choices=POWER_STATE_CHOICES,
        display_choices=POWER_STATE_DISPLAY_CHOICES,
        classes=('hypervisor_power_state', ))

    is_reserved = tables.Column(
        "tnx_is_reserved",
        verbose_name=_('Reserved'),
        empty_value=False,
        status=True,
        filters=(filters.yesno, filters.capfirst),
        form_field=BooleanField(label=_('Reserved'), required=False),
    )

    running_vms = tables.Column("running_vms",
                                verbose_name=_("Instances"))

    def __init__(self, request, data=None, needs_form_wrapper=None, **kwargs):
        super(TnxAdminHypervisorsTable, self).__init__(
            request, data, needs_form_wrapper, **kwargs)

        self._populate_images(request)

    def _populate_images(self, request):
        """
        populate images for assigned image column
        """
        try:
            self.columns['image'].form_field.choices = [
                (image.id, image.image_name)
                for image in TNXPXEImagesClient.list(request)]
        except UrlNotAvailableError:
            raise
        except Exception as err:
            logger.debug(err)
            self.columns['image'].form_field.choices = [('default', 'default')]

    def get_object_display(self, datum):
        return getattr(datum, datum.NAME_ATTR)

    @property
    def needs_form_wrapper(self):
        """
        csrf-token into template and so on in request.Meta
        """
        return True

    def calculate_row_status(self, statuses):
        return statuses

    def get_row_status_class(self, status):
        _statuses = []
        if all((status.get('state', None) is False,
                status.get('power_state', None) is True)):
            _statuses.append('red_color')
        else:
            if status.get('state', None) is False:
                _statuses.append('tnx_state_cell_orange')
            else:
                _statuses.append('tnx_state_cell_green')

            if status.get('power_state', None) is False:
                _statuses.append('tnx_power_state_cell_orange')
            else:
                _statuses.append('tnx_power_state_cell_green')

        return u' '.join(_statuses)

    class Meta(object):
        name = 'tnx_hypervisors'
        verbose_name = _('Hypervisors')
        template = '_data_table.html'
        row_class = actions.TnxHypervisorUpdateRow
        multi_select = True
        status_columns = ['power_state', 'state']

        table_actions = (actions.TnxAdminHypervisorsFilterAction,)

        table_actions_menu = (
            actions.PowerOnHypervisorAction,
            actions.HypervisorMultiShutdownAction,
            actions.HypervisorMultiPowerOffAction,
            actions.HypervisorMultiRestartAction,
            actions.HypervisorMultiStartReservationAction,
            actions.HypervisorMultiAssignImageAction,
            HypervisorSchedulerMultiAction,
            actions.HypervisorMultiUpdateAction)

        row_actions = (
            actions.HypervisorExtraAction,
            actions.HypervisorShutdownAction,
            HypervisorSchedulerAction,
            actions.RestartHypervisorAction,
            actions.PowerOnHypervisorAction,
            actions.PowerOffHypervisorAction,
            actions.StartReservationHypervisorAction,
            actions.HypervisorUpdateAction)


class AdminHypervisorsTable(admin_tables.AdminHypervisorsTable):

    class Meta(object):
        name = "hypervisors"
        verbose_name = _("Hypervisors")
        multi_select = True
        table_actions_menu = (HypervisorSchedulerMultiAction,)

        row_actions = (HypervisorSchedulerAction,)


class ConnectedDevicesTable(ColoredRowMixin,
                            RefererRedirectMixin,
                            tables.DataTable):

    STATUS_CLASSES = {
        True: 'green_color'
    }

    name = tables.Column("device_name", verbose_name=_("Device title"))
    number = tables.Column(
        "device_port",
        verbose_name=_("Coil number"),
        form_field=ChoiceField(),
        update_action=actions.SetPowerDevicePortUpdateCell
    )
    is_default = tables.Column(
        "is_default",
        verbose_name=_("Default"),
        filters=(filters.yesno, filters.capfirst),
        form_field=BooleanField(initial=True,
                                widget=RadioSelect(choices=((True, '',),))),
        update_action=actions.SetDefaultPowerDeviceUpdateCell
    )
    is_initialized = tables.Column(
        'is_initialized',
        verbose_name=_('Initialized'),
        hidden=True
    )

    # pylint: disable=unused-argument
    def __init__(self, request, data=None, needs_form_wrapper=None, **kwargs):
        super(ConnectedDevicesTable, self).__init__(
            request, data=data, needs_form_wrapper=True, **kwargs
        )

    def inline_update_action(self, request, datum, cell, obj_id, cell_name):
        """Handling update by POST of the cell.
        """
        response = super(ConnectedDevicesTable, self).inline_update_action(
            request, datum, cell, obj_id, cell_name
        )
        if response.status_code == 200:
            response = HttpResponseRedirect(self.get_redirect_url())
        return response

    @cached_property
    def devices_coil_map(self):

        try:
            power_devices = TNXPowerDevicesClient.list(self.request)
        except Exception as err:
            logger.error(unicode(err))
            power_devices = []

        return {power_device.id: power_device.coil_map
                for power_device in power_devices}

    class Meta(object):
        name = "connected_devices"
        verbose_name = _("Connected devices")
        row_class = actions.TnxConnectedDeviceUpdateRow
        status_columns = ['is_initialized']
