# coding: utf-8

import logging
import operator

from django.core.context_processors import csrf
from django.utils.translation import ugettext_lazy as _

from horizon import tabs, exceptions

from openstack_dashboard import api
from openstack_dashboard.dashboards.admin.hypervisors import (
    tabs as admin_tabs
)
from openstack_dashboard.dashboards.admin.hypervisors import (
    tables as admin_tables)
from openstack_dashboard.dashboards.admin.hypervisors.compute import (
    tabs as admin_cmp_tabs
)

from tionix_scheduler_client.enums import ObjectTypeEnum, ObjectGroupEnum

from tionix_dashboard.clients import (
    TNXHypervisorsClient, TNXServersClient, TNXNodeJournalClient)
from tionix_dashboard.dashboards.admin.hypervisors import tables
from tionix_dashboard.dashboards.admin.hypervisors.compute import (
    tabs as cmp_tabs)
from tionix_dashboard.dashboards.admin.hypervisors.helpers import (
    ConnectedDevice)
from tionix_dashboard.dashboards.admin.hypervisors.images import (
    tabs as images_tabs)
from tionix_dashboard.dashboards.admin.hypervisors.tables import (
    ConnectedDevicesTable)
from tionix_dashboard.dashboards.admin.instances import (
    tables as instances_tables)
from tionix_dashboard.dashboards.tionix.metrics.tabs import MetricsTab
from tionix_dashboard.dashboards.tionix.scheduler.tabs import ActiveTasksTab
from tionix_dashboard.dashboards.utils import update_instances_list
from tionix_dashboard.dashboards.license import LicenseController
from tionix_dashboard.helpers.views import (
    ServerFilterTabMixin, DTVTnxPaginationMixin)

from tionix_dashboard.dashboards.admin.hypervisors.audit_tables import (
    AuditTable)

logger = logging.getLogger(__name__)


class TnxHypervisorTab(ServerFilterTabMixin, DTVTnxPaginationMixin,
                       admin_tabs.HypervisorTab):
    """Tionix Hypervisors Tab."""
    table_classes = (tables.TnxAdminHypervisorsTable, )

    def get_tnx_hypervisors_data(self):
        hypervisors = []
        offset, limit = self.get_offset_limit()
        try:
            self.table.total_count = TNXHypervisorsClient.count(self.request)
            hypervisors = TNXHypervisorsClient.list(
                self.request, offset=offset, limit=limit + 1)
        except Exception as err:
            logger.error(unicode(err))
            exceptions.handle(self.request,
                              _('Unable to retrieve hypervisor information.'))
        else:
            total = len(hypervisors)
            if offset == 0:
                self._prev = False
            if total <= limit:
                self._more = False
            else:
                hypervisors.pop()

            self.table.first_marker, self.table.last_marker = (
                0, self.calc_last_marker(self.table.total_count, limit)
            )

        return hypervisors


class TnxHypervisorHostTabs(admin_tabs.HypervisorHostTabs):
    tabs = (TnxHypervisorTab,
            cmp_tabs.TnxComputeHostTab,
            images_tabs.HypervisorImagesTab)


class OverviewTab(tabs.TableTab):
    name = _("Overview")
    slug = "overview"
    template_name = "hypervisors/_detail_overview.html"
    table_classes = (ConnectedDevicesTable,)
    preload = False

    def get_context_data(self, request, **kwargs):
        context = super(OverviewTab, self).get_context_data(request, **kwargs)

        context.update({
            'hypervisor': self.tab_group.kwargs['hypervisor']
        })

        context.update(csrf(request))

        return context

    def get_connected_devices_data(self):
        hypervisor = self.tab_group.kwargs['hypervisor']
        return [ConnectedDevice(hypervisor.id, *device)
                for device in getattr(hypervisor, 'tnx_devices', [])]


class AuditTab(tabs.TableTab):
    name = _("Action Log")
    slug = "audit"
    table_classes = (AuditTable,)
    template_name = "hypervisors/_detail_audit.html"
    preload = False

    def get_audit_data(self):
        hypervisor = self.tab_group.kwargs['hypervisor']
        journal = []

        try:
            journal = TNXNodeJournalClient.list(self.request,
                                                node_id=hypervisor.id)
        except Exception as err:
            logger.error(unicode(err))
            exceptions.handle(
                self.request, _('Unable to retrieve hypervisor journal.'))
        else:
            journal.sort(key=operator.attrgetter('created_at'), reverse=True)

        return journal


class SLATab(tabs.Tab):
    name = _("SLA")
    slug = "sla"
    template_name = "hypervisors/_detail_sla.html"


class TnxInstancesTab(DTVTnxPaginationMixin,
                      tabs.TableTab):
    table_classes = (instances_tables.AdminHypervisorInstancesTable,)
    name = _("Instances list")
    slug = "tnx_servers"
    template_name = "hypervisors/_instances_tab.html"
    preload = False

    @property
    def table(self):
        return self._tables['hypervisor_instances']

    # pylint: disable=protected-access
    def get_hypervisor_instances_data(self):
        instances = []

        prev_marker = self.request.GET.get(
            self.table._meta.prev_pagination_param, None)

        total_count = first_marker = last_marker = 0

        sort_dir, marker, is_last_page = 'desc', None, False
        if prev_marker is not None:
            if prev_marker != str(last_marker):
                marker = prev_marker
                sort_dir = 'asc'
        else:
            marker = self.request.GET.get(
                self.table._meta.pagination_param, None)
            if marker == str(last_marker):
                marker = None
                sort_dir = 'asc'
                is_last_page = True

        search_opts = {'marker': marker,
                       'paginate': True}

        try:
            hypervisor = self.tab_group.kwargs['hypervisor']
            search_opts.update(
                {'host': getattr(hypervisor, hypervisor.NAME_ATTR)}
            )
            instances, self._more, self._prev, total_count = (
                TNXServersClient.get_paginator(
                    self.request,
                    search_opts=search_opts,
                    all_tenants=True,
                    sort_dir=sort_dir,
                    last_page=is_last_page)
            )
            if sort_dir == 'asc':
                instances = instances[::-1]
        except Exception as err:
            logger.error(err.message)
            msg = _('Unable to retrieve hypervisor details.')
            exceptions.handle(self.request, msg)

        instances = update_instances_list(self.request, instances)

        self.table.first_marker = first_marker
        self.table.last_marker = last_marker
        self.table.total_count = total_count

        return instances

    def allowed(self, request):
        return LicenseController.is_valid(
            request, LicenseController.CLOUD_CONTROL)

    def get_context_data(self, request, **kwargs):
        context = super(TnxInstancesTab, self).get_context_data(
            request, **kwargs
        )
        context['disable_actions_menu'] = len(context['table'].data) == 0

        return context


class InstancesTab(tabs.TableTab):
    table_classes = (admin_tables.AdminHypervisorInstancesTable,)
    name = _("Instances list")
    slug = "servers"
    template_name = "horizon/common/_detail_table.html"

    def allowed(self, request):
        return not LicenseController.is_valid(
            request, LicenseController.CLOUD_CONTROL)

    def get_hypervisor_instances_data(self):
        instances = []
        try:
            hypervisor = self.tab_group.kwargs['hypervisor']
            result = api.nova.hypervisor_search(
                self.request,
                getattr(hypervisor, hypervisor.NAME_ATTR, '')
            )
            for hyp in result:
                if hyp.id == hypervisor.id:
                    try:
                        instances += hyp.servers
                    except AttributeError:
                        pass
        except Exception as err:
            logger.error(unicode(err))
            exceptions.handle(
                self.request,
                _('Unable to retrieve hypervisor instances list.'))
        return instances


class TnxHypervisorDetailTabs(tabs.TabGroup):
    slug = "hypervisor_details"
    tabs = (OverviewTab,
            TnxInstancesTab,
            InstancesTab,
            AuditTab,
            ActiveTasksTab,
            MetricsTab,
            SLATab)
    sticky = True
    obj_type = ObjectTypeEnum.hypervisor
    obj_group = ObjectGroupEnum.admin


class HypervisorDetailTabs(tabs.TabGroup):
    slug = "hypervisor_details"
    tabs = (InstancesTab,
            TnxInstancesTab,
            MetricsTab)
    sticky = True
    obj_type = ObjectTypeEnum.hypervisor
    obj_group = ObjectGroupEnum.admin


class HypervisorTab(admin_tabs.HypervisorTab):
    table_classes = (tables.AdminHypervisorsTable,)


class HypervisorHostTabs(admin_tabs.HypervisorHostTabs):
    tabs = (HypervisorTab, admin_cmp_tabs.ComputeHostTab)
