# coding: utf-8

import mock

from openstack_dashboard import api

from tionix_tests.helpers import get_fake_request, Empty

from tionix_dashboard.dashboards.admin.hypervisors.tables import (
    TnxAdminHypervisorsTable)
from tionix_dashboard.dashboards.admin.hypervisors.tabs import (
    TnxHypervisorTab, AuditTab, OverviewTab, TnxInstancesTab, InstancesTab)
from tionix_dashboard.dashboards.admin.instances.tables import (
    AdminHypervisorInstancesTable)
from tionix_dashboard.tests.helpers import BaseClientTestCase, FakeUser
from tionix_dashboard.clients import TNXNodeJournalClient
from tionix_dashboard.tests.fake_cloud_state import SERVERS_COUNT


class TnxHypervisorTabTestCase(BaseClientTestCase):

    def test1(self):

        request = get_fake_request(
            user=FakeUser(),
            session={},
            COOKIES={}
        )

        hypervisors = self.tnx_hypervisors.list()

        table = TnxAdminHypervisorsTable(request=request)
        tab = TnxHypervisorTab(tab_group=table, request=request)

        self.assertEqual(unicode(tab.table), unicode(table.Meta.verbose_name))

        self.assertEqual(len(tab.get_tnx_hypervisors_data()), len(hypervisors))

    def test2(self):
        horizon_pagesize = 3

        request = get_fake_request(
            user=FakeUser(),
            session={
                'horizon_pagesize': horizon_pagesize
            },
            COOKIES={})

        table = TnxAdminHypervisorsTable(request=request)
        tab = TnxHypervisorTab(tab_group=table, request=request)

        self.assertEqual(len(tab.get_tnx_hypervisors_data()), horizon_pagesize)


class OverviewTabTestCase(BaseClientTestCase):

    def test(self):

        request = get_fake_request(
            user=FakeUser())
        for hypervisor in self.tnx_hypervisors.list():
            table = TnxAdminHypervisorsTable(
                request=request,
                hypervisor=hypervisor)
            tab = OverviewTab(tab_group=table, request=request)

            result = tab.get_context_data(request)

            self.assertListEqual(sorted(result.keys()),
                                 sorted(['csrf_token', 'hypervisor', 'table',
                                         'connected_devices_table']))
            self.assertEqual(result['hypervisor'].id, hypervisor.id)


class TnxInstancesTabTestCase(BaseClientTestCase):

    def test1(self):

        request = get_fake_request(
            is_ajax=True,
            horizon={
                'async_messages': []
            },
            user=FakeUser(
                tenant_id='1'
            ),
            get_params={
                'marker': '0'
            }
        )

        table = AdminHypervisorInstancesTable(request=request)
        tab = TnxInstancesTab(tab_group=table, request=request)

        self.assertEqual(unicode(tab.table), unicode(table.Meta.verbose_name))

        servers = tab.get_hypervisor_instances_data()
        self.assertIsInstance(servers, list)
        self.assertEqual(len(servers), 0)

    def test2(self):

        request = get_fake_request(
            is_ajax=True,
            horizon={
                'async_messages': []
            },
            session={},
            COOKIES={},
            user=FakeUser(
                tenant_id='1'
            ),
            get_params={
                'prev_marker': '1'
            }
        )

        for hypervisor in self.tnx_hypervisors.list():
            table = AdminHypervisorInstancesTable(
                request=request,
                hypervisor=hypervisor)
            tab = TnxInstancesTab(tab_group=table, request=request)

            self.assertEqual(unicode(tab.table),
                             unicode(table.Meta.verbose_name))

            servers = tab.get_hypervisor_instances_data()
            self.assertIsInstance(servers, list)
            self.assertEqual(len(servers),
                             SERVERS_COUNT[hypervisor.hypervisor_hostname])


class InstancesTabTestCase(BaseClientTestCase):

    @mock.patch('tionix_dashboard.dashboards.license.LicenseController.'
                'is_valid', new=mock.Mock(return_value=False))
    def test(self):

        request = get_fake_request(
            is_ajax=True,
            user=FakeUser(),
            horizon={
                'async_messages': []
            }
        )
        table = AdminHypervisorInstancesTable(request=request)

        tab = InstancesTab(tab_group=table, request=request)
        self.assertTrue(tab.allowed(request=request))

        servers = tab.get_hypervisor_instances_data()
        self.assertIsInstance(servers, list)
        self.assertEqual(len(servers), 0)

    @mock.patch.object(api.nova, 'hypervisor_search')
    @mock.patch('tionix_dashboard.dashboards.license.LicenseController.'
                'is_valid', new=mock.Mock(return_value=False))
    def test2(self, hypervisor_search):
        hypervisors = self.hypervisors.list()[:2]  # pylint: disable=no-member

        hypervisor_search.return_value = hypervisors

        request = get_fake_request(
            is_ajax=True,
            user=FakeUser(
                username='admin',
                token='admin'
            ),
            horizon={
                'async_messages': []
            }
        )

        table = AdminHypervisorInstancesTable(
            request=request,
            hypervisor=hypervisors[0])

        tab = InstancesTab(tab_group=table, request=request)
        self.assertTrue(tab.allowed(request=request))

        servers = tab.get_hypervisor_instances_data()
        self.assertIsInstance(servers, list)
        self.assertEqual(len(servers), len(hypervisors[0].servers))


class AuditTabTestCase(BaseClientTestCase):

    request = None
    test_node_actions = None

    def setUp(self):
        super(AuditTabTestCase, self).setUp()

        self.request = get_fake_request(user=FakeUser())
        self.test_node_actions = [
            Empty(id=1,
                  action="power-off",
                  created_at="2016-05-11T10:24:25.000000",
                  user_name=None),
            Empty(id=2,
                  action="power-on",
                  created_at="2016-05-05T12:08:26.000000",
                  user_name=None)
        ]

    @mock.patch.object(TNXNodeJournalClient, 'list')
    def test(self, journal_list):
        journal_list.return_value = self.test_node_actions

        for hypervisor in self.tnx_hypervisors.list():
            table = TnxAdminHypervisorsTable(
                request=self.request,
                hypervisor=hypervisor)
            tab = AuditTab(tab_group=table, request=self.request)

            result = tab.get_audit_data()

            self.assertEqual(len(result),
                             len(self.test_node_actions))
