# coding: utf-8

import logging

from django.utils.translation import ugettext_lazy as _

from horizon import forms, workflows, messages

from tionix_dashboard.clients import TNXHypervisorsClient
from tionix_dashboard.clients.exceptions import UrlNotAvailableError
from tionix_dashboard.clients.pxe_images import TNXPXEImagesClient

logger = logging.getLogger(__name__)


class AssignImageFirstAction(workflows.Action):
    table_caption = _('Hypervisors list')

    columns = [
        _('Hostname'),
        _('Node title'),
        _('Type'),
        _('Assigned image'),
        _('State'),
        _('Power state'),
        _('Reserved'),
        _('Instances')
    ]

    image = forms.ChoiceField(label=_("Image Name"), required=True)

    restart = forms.BooleanField(
        label=_('Restart Hypervisors'),
        required=False, widget=forms.CheckboxInput()
    )

    def populate_image_choices(self, request, context):
        try:
            images = [
                (image.id, image.image_name)
                for image in
                TNXPXEImagesClient.list(request)]
        except UrlNotAvailableError:
            raise
        except Exception as err:
            logger.debug(err)
            images = [('default', 'default')]
        return images


class AssignNodeFirstStep(workflows.Step):
    """Assign image first step"""

    action_class = AssignImageFirstAction
    template_name = 'hypervisors/_assign_image_first.html'
    contributes = ('image', 'restart')
    depends_on = ('id', 'hypervisors')

    def prepare_action_context(self, request, context):
        context['rows'] = context['hypervisors']
        return context


class AssignImageSecondAction(workflows.Action):
    table_caption = _('Instances list')

    columns = [
        _('Host name'),
        _('Project'),
        _('Name'),
        _('State'),
        _('Internal IP'),
        _('Type')
    ]


class AssignNodeSecondStep(workflows.Step):
    """Assign image first step"""

    action_class = AssignImageSecondAction
    template_name = 'hypervisors/_assign_image_second.html'
    depends_on = ('id', 'instances', 'restart',)

    def prepare_action_context(self, request, context):
        context['rows'] = context['instances']
        return context


class AssignImageWorkflow(workflows.Workflow):
    slug = 'assign_image'
    name = _('Assigning image')
    finalize_button_name = _('Assign Image')
    success_url = failure_url = 'horizon:admin:hypervisors:index'
    default_steps = (AssignNodeFirstStep, AssignNodeSecondStep)
    wizard = True
    template_name = 'hypervisors/_assign_image.html'
    success_hyps = None
    failed_hyps = None

    def get_absolute_url(self):
        return self.request.get_full_path()

    def handle(self, request, data):
        success_hyps, failed_hyps = [], []

        hypervisors = data['hypervisors']

        image_id = data['image']
        if image_id.isdigit():
            image_id = int(image_id)

        for hypervisor in hypervisors:
            try:
                TNXHypervisorsClient.assign_hypervisor_image(
                    request, hypervisor[0], image_id)
            except UrlNotAvailableError:
                raise
            except Exception as err:
                logger.error(unicode(err))
                failed_hyps.append(hypervisor[1])
            else:
                success_hyps.append(hypervisor[1])

        if data['restart'] is True:
            could_not_restart = []

            for hypervisor in hypervisors:
                # skipping if image could be assigned
                if hypervisor[1] in failed_hyps:
                    continue
                # TODO fix tionix_api.reset_node for handling statuses
                if '-' == hypervisor[6]:
                    could_not_restart.append(hypervisor[1])
                    continue

                try:
                    TNXHypervisorsClient.reset_hypervisor(
                        request, hypervisor[0])
                except UrlNotAvailableError:
                    raise
                except Exception as err:
                    logger.error(unicode(err))
                    could_not_restart.append(hypervisor[1])

            if could_not_restart:
                msg = _("Failed to restart hypervisors: {0}").format(
                    ', '.join(could_not_restart))
                messages.error(request, msg)

        self.success_hyps = set(success_hyps)
        self.failed_hyps = set(failed_hyps)

        return not failed_hyps

    def format_status_message(self, message):
        msg = super(AssignImageWorkflow, self).format_status_message(
            message)

        if self.failed_hyps:
            msg = _(
                'Failed to assign image for hypervisors: "{0}"').format(
                '", "'.join(self.failed_hyps))

        elif self.success_hyps:
            msg = _(
                'Successfully assigned image for hypervisors: "{0}"').format(
                '", "'.join(self.success_hyps))

        return msg
