# coding: utf-8

import logging

from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _, ungettext_lazy
from django.utils.http import urlencode

from horizon import messages
from horizon import tables

from openstack_dashboard import policy
from openstack_dashboard.dashboards.admin.instances import (
    tables as admin_tables)
from openstack_dashboard.dashboards.project.instances import (
    tables as project_tables)

from tionix_dashboard.clients import TNXServersClient
from tionix_dashboard.dashboards.license import LicenseController
from tionix_dashboard.dashboards.project.project_instances import (
    actions as project_actions)
from tionix_dashboard.helpers.actions import (
    TnxFilterAction, TnxUpdateAction, MultiLinkAction, TnxBatchActionMixin)
from tionix_dashboard.helpers.utils import get_filter_value_by_dict
from tionix_dashboard.helpers.vm_constants import ACTIVE, VMTypes

logger = logging.getLogger(__name__)


class TnxAdminInstanceFilterAction(TnxFilterAction, tables.FilterAction):
    """
    .. attribute:: filter_string_modifiers

        Modifies filter field value to make api filtering.
    """
    name = "filter_admin_instances"
    filter_type = "server"

    filter_choices = (('project', _("Project ="), True),
                      ('name', _("Name"), True),
                      ('host', _("Host ="), True),
                      ('tnx_node_title', _("Node Title"), False),
                      ('tnx_type', _("Type ="), True),
                      ('ip', _("IPv4 Address ="), True),
                      ('ip6', _("IPv6 Address ="), True),
                      ('status', _("Status ="), True),
                      ('power_state', _("Power state"), True),
                      ('uuid', _("Instance ID ="), True),
                      ('image', _("Image ID ="), True),
                      ('flavor', _("Flavor ID ="), True),
                      ('OS-EXT-AZ:availability_zone',
                       _("Availability zone"),
                       False),)
    filter_string_modifiers = {
        'power_state': project_actions.get_power_states_by_name,
        'tnx_type': get_filter_value_by_dict(VMTypes.TYPES,
                                             not_found_none=False),
        'status': project_actions.get_status_by_name
    }


class CreateImageBatchAction(TnxBatchActionMixin, tables.BatchAction):
    """
    Action power on selected hypervisor.
    """
    name = 'create_image'
    verbose_name = _(u'Create image')
    policy_rules = (('identity', 'admin_required'), )

    @staticmethod
    def action_present(count):
        return CreateImageBatchAction.verbose_name

    @staticmethod
    def action_past(count):
        return CreateImageBatchAction.verbose_name

    def action(self, request, datum_id):
        TNXServersClient.create_image_for_instance(request, datum_id)


class InstanceExtraAction(tables.LinkAction):
    """
    Action show extra information for selected instance.
    """
    name = 'instances'
    verbose_name = _('More')
    url = "horizon:admin:instances:extra"
    classes = ("ajax-modal", )
    policy_rules = (('identity', 'admin_required'), )

    def get_link_url(self, datum=None):
        instance_id = datum.id if datum else None
        return reverse(self.url, args=[instance_id])


class TnxMigrateInstance(TnxBatchActionMixin, admin_tables.MigrateInstance):

    @staticmethod
    def action_present(count):
        return ungettext_lazy(
            u"Migrate Instance",
            u"Migrate Instances",
            count
        )

    def allowed(self, request, instance):
        return (instance is None or
                ((instance.status in project_tables.ACTIVE_STATES or
                  instance.status == 'SHUTOFF') and
                 not project_tables.is_deleting(instance)))


class InstanceUpdateCell(TnxUpdateAction):
    """
    Action update instance name
    """

    def update_cell(self, request, datum, obj_id, cell_name, new_cell_value):
        """
        Update cell
        """
        result = True
        try:
            TNXServersClient.update(request, obj_id, name=new_cell_value)
        except Exception as err:
            logger.error(unicode(err))
            messages.error(request, _('Unable to update instance.'))
            result = False
        return result


class TnxStopInstance(TnxBatchActionMixin, project_tables.StopInstance):
    def allowed(self, request, instance):
        if instance is None:
            return True
        allow = super(TnxStopInstance, self).allowed(request, instance)
        return allow and instance.status.lower() == ACTIVE


class ResetStateAction(project_actions.ResetStateAction):
    """ Action reset vm's state."""
    url = "horizon:admin:instances:reset_state"


class MultiResetStateAction(project_actions.MultiResetStateAction):
    """ Action reset vm's state."""
    url = "horizon:admin:instances:reset_state"


class TnxLiveMigrateInstance(policy.PolicyTargetMixin,
                             tables.LinkAction):
    name = "live_migrate"
    verbose_name = _("Live Migrate Instance")
    url = "horizon:admin:instances:live_migrate"
    classes = ("ajax-modal", "btn-migrate", "btn-danger")
    policy_rules = (
        ("compute", "compute_extension:admin_actions:migrateLive"),)

    def allowed(self, request, instance):
        return (
            LicenseController.is_valid(
                request, LicenseController.CLOUD_CONTROL) and
            (instance.status in project_tables.ACTIVE_STATES) and
            not project_tables.is_deleting(instance)
        )

    def get_link_url(self, datum=None):
        params = urlencode({'id': datum.id if datum else ''})
        return '?'.join([reverse(self.url), params])


class MultiLiveMigrateInstance(policy.PolicyTargetMixin,
                               MultiLinkAction):
    """ Action shut down hypervisor. """
    name = 'multi_live_migrate'
    verbose_name = _('Live Migrate Instance')
    url = 'horizon:admin:instances:live_migrate'
    classes = ("btn-migrate", "btn-danger")
    policy_rules = (
        ("compute", "compute_extension:admin_actions:migrateLive"),)

    def allowed(self, request, instance):
        allowed = LicenseController.is_valid(
            request, LicenseController.CLOUD_CONTROL
        )
        if (instance and
                (instance.status not in project_tables.ACTIVE_STATES or
                 project_tables.is_deleting(instance))):
            allowed = False
        return allowed

    @property
    def attr_url(self):
        params = urlencode(
            {'hypervisor_id': self.table.kwargs['hypervisor_id']}
        )
        return '?'.join([reverse(self.url), params])


class TnxAdminConsoleLink(project_actions.TnxConsoleLink):
    url = 'horizon:admin:instances:console_tab'


class TnxAdminLogLink(project_actions.TnxLogLink):
    url = 'horizon:admin:instances:log_tab'
