# coding: utf-8

import logging
from copy import copy

from django.core.urlresolvers import reverse
from django.utils.http import urlencode
from django.utils.translation import pgettext_lazy, ugettext_lazy as _

from horizon import tables

from openstack_dashboard import api, policy
from openstack_dashboard.dashboards.identity.projects import (
    workflows as proj_workflows)

from tionix_dashboard.dashboards.utils import is_domain_supported
from tionix_dashboard.helpers.actions import MultiLinkAction, TnxFilterAction

logger = logging.getLogger(__name__)


class TenantDeleteAllowedMixin(policy.PolicyTargetMixin):
    """Tenant delete policy mixin"""

    def allowed(self, request, project):
        """Checks if action is allowed"""
        if (api.keystone.is_multi_domain_enabled() and
                not api.keystone.is_domain_admin(request)):
            return False
        return api.keystone.keystone_can_edit_project()


class TnxDeleteTenant(TenantDeleteAllowedMixin, tables.LinkAction):
    """ Action to delete project """
    name = "single_delete"
    verbose_name = _("Delete Project")
    classes = ("ajax-modal", "btn-danger")
    url = "horizon:identity:projects:delete"
    policy_rules = (("identity", "identity:delete_project"),)
    policy_target_attrs = (("target.project.domain_id", "domain_id"),)

    # pylint: disable=arguments-differ
    def get_link_url(self, tenant=None):
        """Retrieve action url"""
        params = urlencode({'id': tenant.id})
        return '?'.join([reverse(self.url), params])


class TnxMultiDeleteTenant(TenantDeleteAllowedMixin, MultiLinkAction):
    """ Group action to delete projects """
    name = 'delete'
    verbose_name = _('Delete Projects')
    classes = ("btn-danger",)
    url = "horizon:identity:projects:delete"


class ManageHostAggregatesAction(tables.LinkAction):
    """Manage host aggregates action"""
    name = "manage_host_aggregate"
    verbose_name = _("Manage Host Aggregates")
    url = "horizon:identity:projects:manage_host_aggregates"
    classes = ("ajax-modal",)
    policy_rules = (("identity", "admin_and_matching_domain_id"),)


class TenantFilterAction(TnxFilterAction):
    """Tenants filter"""
    filter_type = "server"

    filter_choices = [
        ('name', _("Project Name"), False),
        ('description', _("Description"), False),
        ('id', _("Project ID"), False),
        ('domain_name', _("Domain Name"), False),
        ('enabled',
         pgettext_lazy("Enabled field for project", u"Enabled"),
         False)
    ]
    if not is_domain_supported():
        filter_choices.remove(('domain_name', _("Domain Name"), False))


class TnxTenantFilterAction(TenantFilterAction):
    """Tenants filter with vdi"""
    filter_choices = copy(TenantFilterAction.filter_choices)

    filter_choices.insert(1, ('tnx_project_type', _("Type"), False))


# pylint: disable=super-on-old-class
class TnxUpdateProjectMembersAction(proj_workflows.UpdateProjectMembersAction):

    def __init__(self, request, *args, **kwargs):
        super(TnxUpdateProjectMembersAction, self).__init__(
            request, *args, **kwargs)

        #  pylint:disable=no-member
        for field_name in self.fields:
            if field_name.startswith(self.slug):
                # sort by name
                self.fields[field_name].choices.sort(
                    key=lambda x: x[1].lower())

    class Meta(object):
        name = _("Project Members")
        slug = proj_workflows.PROJECT_USER_MEMBER_SLUG
