# coding: utf-8

from django.core.urlresolvers import reverse
from django.template import defaultfilters as filters
from django.utils.translation import ugettext_lazy as _, pgettext_lazy

from horizon import forms, tables

from openstack_dashboard import api, policy
from openstack_dashboard.dashboards.identity.projects import (
    tables as project_tables)

from tionix_dashboard.templatetags.tionix_tags import format_date
from tionix_dashboard.dashboards.identity.projects.actions import (
    TnxDeleteTenant, TnxMultiDeleteTenant, ManageHostAggregatesAction,
    TenantFilterAction, TnxTenantFilterAction)
from tionix_dashboard.dashboards.identity.projects.enum import TenantTypeEnum
from tionix_dashboard.dashboards.project.project_instances.tables import (
    TnxAuditLog)
from tionix_dashboard.dashboards.tionix.scheduler.actions import (
    ProjectSchedulerAction, ProjectSchedulerMultiAction
)
from tionix_dashboard.dashboards.utils import is_domain_supported
from tionix_dashboard.helpers.utils import add_kwargs
from tionix_dashboard.url_resolve_map import IDENTITY_PROJECT_DETAIL


class TnxUpdateCell(project_tables.UpdateCell):
    def allowed(self, request, project, cell):
        policy_rule = (("identity", "identity:update_project"),)
        policy_target = {'target.project.domain_id': project.domain_id}
        return (
            (cell.column.name != 'enabled' or
             request.user.project_id != cell.datum.id) and
            api.keystone.keystone_can_edit_project() and
            policy.check(policy_rule, request, target=policy_target))


class ModifyQuotas(project_tables.ModifyQuotas):
    def allowed(self, request, datum):
        if api.keystone.VERSIONS.active < 3:
            result = True
        else:
            if api.keystone.is_multi_domain_enabled():
                result = api.keystone.is_domain_admin(request)
            else:
                result = api.keystone.keystone_can_edit_project()
        return result


class TnxTenantsTable(tables.DataTable):
    """Tenants table with vdi enabled"""
    name = tables.Column(
        'name',
        verbose_name=_('Name'),
        link=IDENTITY_PROJECT_DETAIL,
        form_field=forms.CharField(max_length=64),
        update_action=TnxUpdateCell)
    tnx_project_type = tables.Column(
        lambda x: getattr(x, "tnx_project_type", '') or '',
        verbose_name=_("Type"),
        display_choices=TenantTypeEnum.values.items())
    description = tables.Column(
        lambda obj: getattr(obj, 'description', None),
        verbose_name=_('Description'),
        form_field=forms.CharField(
            widget=forms.Textarea(attrs={'rows': 4}),
            required=False),
        update_action=TnxUpdateCell)
    id = tables.Column('id', verbose_name=_('Project ID'))
    if is_domain_supported():
        domain_name = tables.Column(
            'domain_name', verbose_name=_('Domain Name'))
    enabled = tables.Column(
        'enabled',
        verbose_name=pgettext_lazy("Enabled field for project", u"Enabled"),
        status=True,
        filters=(filters.yesno, filters.capfirst),
        form_field=forms.BooleanField(
            label=_('Enabled'),
            required=False),
        update_action=TnxUpdateCell)

    def get_project_detail_link(self, project):
        """return link to project"""
        # this method is an ugly monkey patch, needed because
        # the column link method does not provide access to the request
        policy_rule = (("identity", "identity:get_project"),)
        policy_target = {
            'project': project,
            'target.project.domain_id': project.domain_id,
        }
        if policy.check(policy_rule, self.request, target=policy_target):
            return reverse(IDENTITY_PROJECT_DETAIL,
                           args=(project.id,))
        return None

    def __init__(self, request, data=None, needs_form_wrapper=None, **kwargs):
        super(TnxTenantsTable, self).__init__(
            request,
            data=data,
            needs_form_wrapper=needs_form_wrapper,
            **kwargs)
        # see the comment above about ugly monkey patches
        self.columns['name'].get_link_url = self.get_project_detail_link

    class Meta(project_tables.TenantsTable.Meta):

        row_actions = (
            project_tables.UpdateMembersLink, project_tables.UpdateGroupsLink,
            project_tables.UpdateProject, ModifyQuotas,
            project_tables.RescopeTokenToProject, TnxDeleteTenant,
            ProjectSchedulerAction, ManageHostAggregatesAction
        )
        table_actions = (
            TnxTenantFilterAction, project_tables.CreateProject,
            TnxMultiDeleteTenant, ProjectSchedulerMultiAction
        )


class TenantsTable(project_tables.TenantsTable):
    """Tenants table"""
    class Meta(project_tables.TenantsTable.Meta):

        row_actions = (
            project_tables.UpdateMembersLink,
            project_tables.UpdateGroupsLink,
            project_tables.UpdateProject,
            project_tables.UsageLink,
            ModifyQuotas,
            project_tables.RescopeTokenToProject,
            TnxDeleteTenant,
            ProjectSchedulerAction,
            ManageHostAggregatesAction
        )
        table_actions = (
            TenantFilterAction,
            project_tables.CreateProject,
            TnxMultiDeleteTenant,
            ProjectSchedulerMultiAction
        )


def get_url_for_instance(datum, is_admin=False):
    link = None
    if is_admin:
        obj_id = getattr(datum, 'instance_id', None)
        link = reverse("horizon:admin:instances:detail", args=(obj_id,))
    return link


class BaseInstanceAuditLog(tables.DataTable):
    request_id = tables.Column('request_id',
                               verbose_name=_('Request ID'))
    instance_name = tables.Column('instance_name',
                                  verbose_name=_('Instance'),
                                  link=get_url_for_instance)
    action = tables.Column('action', verbose_name=_('Action'),
                           display_choices=TnxAuditLog.ACTION_DISPLAY_CHOICES)
    details = tables.Column('details', verbose_name=_('More'))
    start_time = tables.Column('start_time', verbose_name=_('Start time'),
                               filters=(format_date, ))
    user = tables.Column('user', verbose_name=_('User'))
    message = tables.Column('message', verbose_name=_('Message'))


class TnxInstancesAuditLog(BaseInstanceAuditLog):

    def __init__(self, request, data=None, needs_form_wrapper=None, **kwargs):
        super(TnxInstancesAuditLog, self).__init__(
            request,
            data=data,
            needs_form_wrapper=needs_form_wrapper,
            **kwargs)

        is_admin = api.keystone.is_domain_admin(request)
        self.columns['instance_name'].get_link_url = add_kwargs(
            get_url_for_instance, is_admin=is_admin
        )

    class Meta(object):
        name = 'audit'
        verbose_name = _('Instances Action List')
        multi_select = False
        table_actions = (tables.FilterAction,)
