# coding: utf-8
"""Identity projects tabs"""

import logging

from django.utils.translation import ugettext_lazy as _

from horizon import exceptions, tabs

from openstack_dashboard import api, usage as os_usage
from openstack_dashboard.dashboards.project.instances import (
    tabs as project_tabs)

from tionix_scheduler_client.enums import ObjectTypeEnum, ObjectGroupEnum

from tionix_dashboard.clients import TNXServersClient
from tionix_dashboard.dashboards.project.project_instances.helpers import (
    get_instances_audit_data)
from tionix_dashboard.dashboards.tionix.scheduler.tabs import ActiveTasksTab
from tionix_dashboard.dashboards.identity.projects import tables, utils
from tionix_dashboard.dashboards.utils import is_domain_supported

logger = logging.getLogger(__name__)


class OverviewTab(tabs.Tab):
    """Project overview tab"""
    name = _("Overview")
    slug = "overview"
    template_name = 'projects/_detail_overview.html'

    def get_context_data(self, request, **kwargs):
        """project context"""

        project = self.tab_group.kwargs['project']
        domain_enabled = is_domain_supported()
        if domain_enabled:
            domains_dict = utils.domain_lookup(self.request)
            project.domain_name = domains_dict.get(project.domain_id, None)
        return {"project": project,
                "domain_enabled": domain_enabled}


class UsageTab(tabs.TableTab):
    """Project usage tab"""
    name = _("Usage")
    slug = "usage"
    template_name = 'projects/_detail_usage.html'
    table_classes = (os_usage.ProjectUsageTable,)
    usage_class = os_usage.ProjectUsage
    preload = False
    usage = None

    def get_context_data(self, request, **kwargs):
        context = super(UsageTab, self).get_context_data(request, **kwargs)
        context['table'].kwargs['usage'] = self.usage
        context['form'] = self.usage.form
        context['usage'] = self.usage

        try:
            context['simple_tenant_usage_enabled'] = (
                api.nova.extension_supported('SimpleTenantUsage', self.request)
            )
        except Exception as err:
            logger.error(unicode(err))
            context['simple_tenant_usage_enabled'] = True

        return context

    def get_project_usage_data(self):
        instances = []
        project_id = self.tab_group.kwargs.get('project_id',
                                               self.request.user.tenant_id)
        try:
            self.usage = self.usage_class(self.request, project_id)
            self.usage.summarize(*self.usage.get_date_range())
            instances = self.usage.get_instances()
        except Exception as err:
            logger.error(err)
            exceptions.handle(self.request,
                              _('Unable to retrieve usage information.'))
        return instances


class InstancesAuditTab(project_tabs.AuditTab):
    """Tionix audit of instances tab"""

    table_classes = (tables.TnxInstancesAuditLog, )

    def get_audit_data(self):
        """Project instances audit data"""
        data = []
        project_id = self.tab_group.kwargs['tenant_id']
        search_opts = {'project_id': project_id}
        if api.keystone.is_domain_admin(self.request):
            search_opts.update({'all_tenants': True})
        try:
            instances = TNXServersClient.list(self.request,
                                              search_opts=search_opts)
        except Exception as err:
            logger.error(unicode(err))
            exceptions.handle(self.request,
                              _('Unable to get project instances list.'))
        else:
            if instances:
                instances_dict = {server.id: server.name
                                  for server in instances}

                data = get_instances_audit_data(self.request,
                                                instances_dict.keys())
                for obj in data:
                    instance_id = obj.instance_id
                    obj.instance_name = instances_dict.get(instance_id,
                                                           instance_id)
                    obj.project_id = project_id
        return data


class SLATab(tabs.Tab):
    """Sla tab"""
    name = _("SLA")
    slug = "sla"
    template_name = "project/project_instances/_detail_sla.html"

    def get_context_data(self, request, **kwargs):
        """sla context data"""
        return {
            'up_time': '99,999', 'recovery_time': '8'
        }


class ProjectDetailTabs(tabs.TabGroup):
    """Tionix project details tab group"""
    slug = "tnx_project_details"
    tabs = (OverviewTab,
            UsageTab,
            InstancesAuditTab,
            ActiveTasksTab,
            SLATab)
    sticky = True
    obj_type = ObjectTypeEnum.project
    obj_group = ObjectGroupEnum.identity
