# coding: utf-8
# pylint: disable=no-member
# pylint: disable=unused-variable

"""Identity projects tests"""
import json

import mock

from django import http
from django.conf import settings
from django.core.urlresolvers import reverse
from django.test.utils import override_settings
from django.utils.http import urlencode
from tionix_scheduler_client.enums import ObjectGroupEnum

from openstack_dashboard import api
from openstack_auth import policy as policy_backend

from tionix_tests.helpers import get_fake_request, Empty

from tionix_dashboard.clients import TNXServersClient
from tionix_dashboard.clients.journal import TNXJournalClient
from tionix_dashboard.clients.tenants import TNXTenantClient
from tionix_dashboard.dashboards.identity.projects import (
    actions, tables, views, utils, tabs)
from tionix_dashboard.dashboards.identity.projects import forms
from tionix_dashboard.dashboards.tionix.scheduler.helpers import (
    SchedulingPreChecker)
from tionix_dashboard.tests.helpers import (
    BaseClientTestCase, BaseAdminClientTestCase, FakeUser)


INDEX_URL = "horizon:identity:projects:index"
DETAIL_URL = "horizon:identity:projects:detail"
DELETE_URL = "horizon:identity:projects:delete"


@mock.patch('tionix_dashboard.dashboards.utils.is_domain_supported',
            mock.Mock(return_value=True))
@override_settings(POLICY_CHECK_FUNCTION=policy_backend.check)
class TnxTenantsTablesTestCase(BaseClientTestCase):

    def test_tnx_tenants_table(self):

        table = tables.TnxTenantsTable(self.request)

        self.assertListEqual(
            sorted(table.columns.keys()),
            sorted(['multi_select', 'actions', 'name', 'tnx_project_type',
                    'description', 'id', 'enabled', 'domain_name'])
        )

    @mock.patch.object(SchedulingPreChecker, 'is_actions_not_empty',
                       mock.Mock(return_value=True))
    def test_tnx_tenants_table_actions(self):
        tenants = self.tenants.list()

        table = tables.TnxTenantsTable(self.request, data=tenants)

        self.assertListEqual(
            sorted([action.name for action in table.get_table_actions()]),
            sorted(['filter'])
        )

    @mock.patch.object(SchedulingPreChecker, 'is_actions_not_empty',
                       mock.Mock(return_value=True))
    def test_tnx_tenants_row_actions(self):
        table = tables.TnxTenantsTable(self.request)

        tenant = self.tnx_tenants.list()[0]
        self.assertListEqual(
            [action.name for action in table.get_row_actions(tenant)], []
        )


@mock.patch('tionix_dashboard.dashboards.utils.is_domain_supported',
            mock.Mock(return_value=True))
@override_settings(POLICY_CHECK_FUNCTION=policy_backend.check)
class TnxAdminTenantsTablesTestCase(BaseAdminClientTestCase):

    @mock.patch.object(SchedulingPreChecker, 'is_actions_not_empty',
                       mock.Mock(return_value=True))
    def test_tnx_tenants_table_actions(self):
        tenants = self.tenants.list()

        table = tables.TnxTenantsTable(self.request, data=tenants)

        self.assertListEqual(
            sorted([action.name for action in table.get_table_actions()]),
            sorted(['create', 'delete', 'filter', 'project_multi_schedule'])
        )

    @mock.patch.object(SchedulingPreChecker, 'is_actions_not_empty',
                       mock.Mock(return_value=True))
    def test_tnx_tenants_row_actions(self):
        table = tables.TnxTenantsTable(self.request)

        tenant = self.tnx_tenants.list()[0]
        self.assertListEqual(
            sorted([action.name for action in table.get_row_actions(tenant)]),
            sorted(['single_delete', 'groups', 'project_schedule', 'quotas',
                    'update', 'users', 'manage_host_aggregate'])
        )


class TnxTenantsViewsAdminTest(BaseClientTestCase):

    @mock.patch.object(TNXTenantClient, 'list')
    @mock.patch.object(utils, 'domain_lookup')
    @mock.patch.object(api.keystone, 'get_effective_domain_id')
    @mock.patch.object(api.keystone, 'tenant_list')
    def test_index_get_data(self, tenant_list, get_effective_domain_id,
                            domain_lookup, tnx_tenant_list):
        tenants = self.tenants.list()
        domain = self.domains.get(id="1")

        tenant_list.return_value = (tenants, None)
        get_effective_domain_id.return_value = domain.id
        domain_lookup.return_value = {domain.id: domain.name}
        tnx_tenant_list.return_value = tenants

        view = views.TnxIndexView()
        view.request = get_fake_request(**{
            'is_ajax': True,
            'session': {},
            'COOKIES': {},
            'horizon': {
                'async_messages': []
            }
        })

        data = view.get_data()

        self.assertEqual(len(data), len(tenants))

    @mock.patch.object(views.TnxDeleteTenantView,
                       'get_tasks_rows',
                       mock.Mock(return_value=[]))
    @mock.patch.object(api.keystone,
                       'tenant_delete',
                       mock.Mock(return_value=True))
    @mock.patch.object(TNXTenantClient, 'get')
    def test_delete_from_index(self, tenant_get):
        tenant = self.tnx_tenants.list()[0]

        tenant_get.return_value = tenant

        url = '?'.join([reverse(DELETE_URL), urlencode({'id': tenant.id})])

        req = self.factory.post(url)

        index_url = reverse(INDEX_URL)
        req.META['HTTP_REFERER'] = ''.join([settings.TESTSERVER,
                                            settings.WEBROOT,
                                            index_url])
        req.META['HTTP_HOST'] = settings.TESTSERVER

        response = views.TnxDeleteTenantView.as_view()(req)

        self.assertIn(index_url, response.url)

    @mock.patch.object(views.TnxDeleteTenantView,
                       'get_tasks_rows',
                       mock.Mock(return_value=[]))
    @mock.patch.object(api.keystone,
                       'tenant_delete',
                       mock.Mock(return_value=True))
    @mock.patch.object(TNXTenantClient, 'get')
    def test_delete_from_detail(self, tenant_get):
        tenant = self.tnx_tenants.list()[0]

        tenant_get.return_value = tenant

        url = '?'.join([reverse(DELETE_URL), urlencode({'id': tenant.id})])

        req = self.factory.post(url)

        index_url = reverse(INDEX_URL)
        req.META['HTTP_REFERER'] = ''.join([
            settings.TESTSERVER,
            settings.WEBROOT,
            reverse(DETAIL_URL, args=(tenant.id,))])
        req.META['HTTP_HOST'] = settings.TESTSERVER

        response = views.TnxDeleteTenantView.as_view()(req)

        self.assertIn(index_url, response.url)


@mock.patch('tionix_dashboard.dashboards.utils.is_domain_supported',
            mock.Mock(return_value=True))
class TnxTenantsActionsTestCase(BaseClientTestCase):

    def test_filter_action(self):

        fields, names, api_flags = zip(
            *actions.TenantFilterAction.filter_choices
        )

        self.assertListEqual(
            sorted(fields),
            sorted(['name', 'description', 'id', 'enabled', 'domain_name'])
        )

    def test_tnx_filter_action1(self):

        fields, names, api_flags = zip(
            *actions.TnxTenantFilterAction.filter_choices
        )

        self.assertListEqual(
            sorted(fields),
            sorted(['name', 'description', 'id', 'enabled', 'domain_name',
                    'tnx_project_type'])
        )

    def test_tnx_filter_action2(self):
        tenants = self.tnx_tenants.list()
        tenant = tenants[0]

        action = actions.TnxTenantFilterAction()

        fields, names, api_flags = zip(*action.filter_choices)

        for field in fields:
            table = tables.TenantsTable(
                request=get_fake_request(
                    session={
                        'tenants__filter__q_field': field
                    }
                )
            )

            filtered_data = action.filter(table,
                                          tenants,
                                          getattr(tenant, field, ''))

            self.assertEqual(len(filtered_data), 1)


class TnxTenantsTabsTestCase(BaseClientTestCase):
    @mock.patch.object(TNXServersClient, 'list', mock.Mock(return_value=[]))
    def test_audit_data_empty_instances(self):
        tenant = self.tnx_tenants.list()[0]

        tab_group = tabs.ProjectDetailTabs(
            self.request, tenant_id=tenant.id
        )

        tab = tabs.InstancesAuditTab(tab_group=tab_group, request=self.request)

        self.assertEqual(len(tab.get_audit_data()), 0)

    @mock.patch('openstack_dashboard.policy.check', mock.Mock())
    @mock.patch('tionix_dashboard.dashboards.utils.user_list')
    @mock.patch.object(TNXJournalClient, 'list')
    def test_audit_data(self, journal_list, user_list):
        tenant = self.tnx_tenants.list()[0]
        instance = self.tnx_servers.list()[0]
        users = self.tnx_users.list()
        user = users[0]

        user_list.return_value = users

        journal_list.return_value = [
            Empty(instance_id=instance.id,
                  user_id=user.id,
                  created_at=None,
                  request_user_id=user.id)
        ]

        tab_group = tabs.ProjectDetailTabs(
            self.request, tenant_id=tenant.id
        )

        tab = tabs.InstancesAuditTab(tab_group=tab_group, request=self.request)

        data = tab.get_audit_data()

        self.assertEqual(len(data), 1)
        self.assertEqual(data[0].instance_name, instance.name)
        self.assertEqual(data[0].user, user.name)


@mock.patch('tionix_dashboard.dashboards.identity.projects.workflows.'
            'remove_project_cache', mock.Mock(return_value=True))
class TnxTenantsFormsTestCase(BaseClientTestCase):

    def test_manage_host_aggregate_valid(self):
        tenant = self.tnx_tenants.list()[0]

        form = forms.ManageHostAggregatesForm(
            self.request, data={
                'new_allocated': json.dumps([{'id': 1,
                                              'name': 'test aggregate'}]),
                'old_allocated': '[]',
                'project_id': tenant.id
            }
        )

        self.assertTrue(form.is_valid())

    def test_manage_host_aggregate_not_valid(self):
        tenant = self.tnx_tenants.list()[0]

        form = forms.ManageHostAggregatesForm(self.request,
                                              data={'new_allocated': '[]',
                                                    'old_allocated': '[]',
                                                    'project_id': tenant.id})

        self.assertFalse(form.is_valid())

    @mock.patch.object(api.nova, 'aggregate_set_metadata', mock.Mock())
    def test_manage_host_aggregate_handle(self):
        tenant = self.tnx_tenants.list()[0]

        data = {
            'new_allocated': [{'id': 2, 'name': 'aggregate 2'}],
            'old_allocated': [{'id': 1, 'name': 'aggregate'},
                              {'id': 2, 'name': 'aggregate 2'}],
            'project_id': tenant.id
        }
        request = get_fake_request(
            user=FakeUser(),
            is_ajax=True,
            horizon={
                'async_messages': []
            }
        )
        form = forms.ManageHostAggregatesForm(request)
        form.initial = {'project_id': tenant.id}
        form.handle(request, data)

        self.assertTrue(request.horizon['async_messages'][0][1],
                        u'Successfully updated host aggregates: '
                        u'"test aggregate".')

    @mock.patch.object(api.nova,
                       'aggregate_set_metadata',
                       mock.Mock(side_effect=Exception))
    def test_manage_host_aggregate_handle_error(self):
        tenant = self.tnx_tenants.list()[0]

        data = {
            'new_allocated': [{'id': 2, 'name': 'aggregate 2'}],
            'old_allocated': [{'id': 1, 'name': 'aggregate'},
                              {'id': 2, 'name': 'aggregate 2'}],
            'project_id': tenant.id
        }
        request = get_fake_request(
            user=FakeUser(),
            is_ajax=True,
            horizon={
                'async_messages': []
            }
        )
        form = forms.ManageHostAggregatesForm(request)
        form.initial = {'project_id': tenant.id}
        form.handle(request, data)

        self.assertTrue(request.horizon['async_messages'][0][1],
                        u'Unable to update host aggregates: "test aggregate".')

    @mock.patch.object(TNXTenantClient, 'delete')
    @mock.patch.object(api.keystone, 'tenant_delete')
    def test_tnx_delete_vdi_project(self, tenant_delete, delete):
        tenant = self.tnx_tenants.list()[0]

        form = forms.TnxDeleteTenantForm(self.request)
        form.initial = {
            'tenants_with_instances': [],
            'projects_groups': {tenant.id: (tenant.id,
                                            tenant.name,
                                            ObjectGroupEnum.vdi)},
            'redirect_url': '',
            'rows': []
        }

        request = get_fake_request(
            user=FakeUser(unscoped_token=None),
            is_ajax=True,
            horizon={
                'async_messages': []
            }
        )

        form.handle(request, {})

        self.assertTrue(delete.called)
        self.assertFalse(tenant_delete.called)

    @mock.patch.object(TNXTenantClient, 'delete')
    @mock.patch.object(api.keystone, 'tenant_delete')
    def test_tnx_delete_identity_project(self, tenant_delete, delete):
        tenant = self.tnx_tenants.list()[0]

        form = forms.TnxDeleteTenantForm(self.request)
        form.initial = {
            'tenants_with_instances': [],
            'projects_groups': {tenant.id: (tenant.id,
                                            tenant.name,
                                            ObjectGroupEnum.identity)},
            'redirect_url': '',
            'rows': []
        }

        request = get_fake_request(
            user=FakeUser(unscoped_token=None),
            is_ajax=True,
            horizon={
                'async_messages': []
            }
        )

        form.handle(request, {})

        self.assertFalse(delete.called)
        self.assertTrue(tenant_delete.called)

    @mock.patch('tionix_dashboard.dashboards.identity.projects.forms.'
                'scheduler_api.delete_scheduled_tasks_list')
    @mock.patch.object(TNXTenantClient, 'delete')
    def test_tnx_delete_with_tasks(self, delete, delete_scheduled_tasks_list):
        tenant = self.tnx_tenants.list()[0]

        form = forms.TnxDeleteTenantForm(self.request)
        form.initial = {
            'tenants_with_instances': [],
            'projects_groups': {tenant.id: (tenant.id,
                                            tenant.name,
                                            ObjectGroupEnum.vdi)},
            'redirect_url': '',
            'rows': [None]
        }

        request = get_fake_request(
            user=FakeUser(token=Empty(id=None), unscoped_token=None),
            is_ajax=True,
            horizon={
                'async_messages': []
            }
        )

        form.handle(request, {})

        self.assertTrue(delete.called)
        self.assertTrue(delete_scheduled_tasks_list.called)

    @mock.patch.object(TNXTenantClient, 'delete', mock.Mock())
    def test_tnx_delete_with_instances(self):
        tenant = self.tnx_tenants.list()[0]

        form = forms.TnxDeleteTenantForm(self.request)
        form.initial = {
            'tenants_with_instances': [tenant.id],
            'projects_groups': {tenant.id: (tenant.id,
                                            tenant.name,
                                            ObjectGroupEnum.vdi)},
            'redirect_url': '',
            'rows': []
        }

        request = get_fake_request(
            user=FakeUser(token=Empty(id=None), unscoped_token=None),
            is_ajax=True,
            horizon={
                'async_messages': []
            }
        )

        form.handle(request, {})

        self.assertTrue(request.horizon['async_messages'][0][1],
                        u'Projects "{0}" contain instances, removing '
                        u'is prohibited.'.format(tenant.id))
        self.assertTrue(request.horizon['async_messages'][0][0], 'info')

    @mock.patch.object(TNXTenantClient,
                       'delete',
                       mock.Mock(side_effect=Exception))
    def test_tnx_delete_redirect(self):
        tenant = self.tnx_tenants.list()[0]

        form = forms.TnxDeleteTenantForm(self.request)
        form.initial = {
            'tenants_with_instances': [],
            'projects_groups': {tenant.id: (tenant.id,
                                            tenant.name,
                                            ObjectGroupEnum.vdi)},
            'redirect_url': '',
            'rows': []
        }

        request = get_fake_request(
            user=FakeUser(token=Empty(id=None), unscoped_token=None),
            is_ajax=True,
            horizon={
                'async_messages': []
            }
        )

        result = form.handle(request, {})

        self.assertIsInstance(result, http.HttpResponseRedirect)
