# coding: utf-8

import functools

from django.utils.translation import ugettext_lazy as _

from horizon import forms

from openstack_dashboard import api
from openstack_dashboard.dashboards.identity.projects import (
    workflows as proj_workflows)

from tionix_dashboard.dashboards.identity.projects import actions
from tionix_dashboard.dashboards.identity.projects.utils import (
    remove_project_cache
)
from tionix_dashboard.helpers.fields import AutoGenNameCharField


def post_remove_project_cache(f):

    @functools.wraps(f)
    def wrapped(self, *args, **kwargs):
        result = f(self, *args, **kwargs)
        if hasattr(self, 'request'):
            remove_project_cache(self.request.user.unscoped_token)
        return result

    return wrapped


class CreateProjectInfoAction(proj_workflows.CreateProjectInfoAction):
    name = AutoGenNameCharField(label=_("Name"), max_length=64)

    class Meta(object):
        name = _("Project Information")
        help_text = _("Create a project to organize users.")


class CreateProjectInfo(proj_workflows.CreateProjectInfo):
    action_class = CreateProjectInfoAction


class TnxUpdateProjectMembers(proj_workflows.UpdateProjectMembers):
    action_class = actions.TnxUpdateProjectMembersAction


class ClearUserProjectCacheMixin(object):

    @post_remove_project_cache
    def handle(self, *args, **kwargs):
        return super(ClearUserProjectCacheMixin, self).handle(*args, **kwargs)


class ProjectQuotaActionMixin(object):

    def displayed_fields(self):
        return [self[name] for name, field in self.fields.items()
                if not isinstance(field.widget, forms.HiddenInput)]


class CreateProjectQuotaAction(ProjectQuotaActionMixin,
                               proj_workflows.CreateProjectQuotaAction):
    class Meta(object):
        name = _("Quota")
        slug = 'create_quotas'
        help_text = _("Set maximum quotas for the project.")
        permissions = ('openstack.roles.admin', 'openstack.services.compute')


class CreateProjectQuota(proj_workflows.UpdateProjectQuota):
    action_class = CreateProjectQuotaAction
    template_name = "projects/_quota_fields.html"


#  pylint:disable=non-parent-init-called,super-init-not-called
class CreateProject(proj_workflows.CreateProject):
    default_steps = (CreateProjectInfo,
                     proj_workflows.UpdateProjectMembers,
                     proj_workflows.CreateProjectQuota)

    def __init__(self, request=None, context_seed=None, entry_point=None,
                 *args, **kwargs):
        if proj_workflows.PROJECT_GROUP_ENABLED:
            self.default_steps = (CreateProjectInfo,
                                  proj_workflows.UpdateProjectMembers,
                                  proj_workflows.UpdateProjectGroups,
                                  proj_workflows.CreateProjectQuota)

        proj_workflows.CommonQuotaWorkflow.__init__(
            self, request=request, context_seed=context_seed,
            entry_point=entry_point, *args, **kwargs)


class TnxCreateProject(ClearUserProjectCacheMixin, CreateProject):
    default_steps = (CreateProjectInfo,
                     TnxUpdateProjectMembers,
                     CreateProjectQuota)

    def __init__(self, request=None, context_seed=None, entry_point=None,
                 *args, **kwargs):
        if proj_workflows.PROJECT_GROUP_ENABLED:
            self.default_steps = (CreateProjectInfo,
                                  TnxUpdateProjectMembers,
                                  proj_workflows.UpdateProjectGroups,
                                  CreateProjectQuota)
        super(TnxCreateProject, self).__init__(request=request,
                                               context_seed=context_seed,
                                               entry_point=entry_point,
                                               *args,
                                               **kwargs)

    def handle(self, request, data):  # pylint:disable=arguments-differ
        project = self._create_project(request, data)
        if not project:
            return False
        project_id = project.id
        self._update_project_members(request, data, project_id)
        if proj_workflows.PROJECT_GROUP_ENABLED:
            self._update_project_groups(request, data, project_id)
        if api.keystone.is_domain_admin(request):
            self._update_project_quota(request, data, project_id)
        return True


class UpdateProjectQuotaAction(ProjectQuotaActionMixin,
                               proj_workflows.UpdateProjectQuotaAction):
    class Meta(object):
        name = _("Quota")
        slug = 'update_quotas'
        help_text = _("Set maximum quotas for the project.")
        permissions = ('openstack.roles.admin', 'openstack.services.compute')


class UpdateProjectQuota(proj_workflows.UpdateProjectQuota):
    action_class = UpdateProjectQuotaAction
    template_name = "projects/_quota_fields.html"


class UpdateProject(ClearUserProjectCacheMixin,
                    proj_workflows.UpdateProject):
    default_steps = (proj_workflows.UpdateProjectInfo,
                     TnxUpdateProjectMembers,
                     UpdateProjectQuota)

    def __init__(self, request=None, context_seed=None, entry_point=None,
                 *args, **kwargs):
        if proj_workflows.PROJECT_GROUP_ENABLED:
            self.default_steps = (proj_workflows.UpdateProjectInfo,
                                  TnxUpdateProjectMembers,
                                  proj_workflows.UpdateProjectGroups,
                                  UpdateProjectQuota)

        proj_workflows.CommonQuotaWorkflow.__init__(
            self, request=request, context_seed=context_seed,
            entry_point=entry_point, *args, **kwargs)

    def handle(self, request, data):  # pylint:disable=arguments-differ
        project = self._update_project(request, data)
        if not project:
            return False

        project_id = data['project_id']
        # Use the domain_id from the project if available
        domain_id = getattr(project, "domain_id", '')

        ret = self._update_project_members(request, data, project_id)
        if not ret:
            return False

        if proj_workflows.PROJECT_GROUP_ENABLED:
            ret = self._update_project_groups(request, data,
                                              project_id, domain_id)
            if not ret:
                return False

        if api.keystone.is_domain_admin(request):
            ret = self._update_project_quota(request, data, project_id)
            if not ret:
                return False

        return True
