# coding: utf-8
# pylint:disable=arguments-differ

import logging

from django.conf import settings
from django.core.urlresolvers import reverse
from django.http import HttpResponse
from django.utils.http import urlencode
from django.utils.translation import (
    ugettext_lazy as _, ungettext_lazy, string_concat,
    npgettext_lazy)

from horizon import messages, exceptions, tables

from openstack_dashboard import api
from openstack_dashboard import policy
from openstack_dashboard.dashboards.project.instances import (
    tables as project_tables)

from tionix_dashboard.clients import TNXServersClient, TNXHypervisorsClient
from tionix_dashboard.helpers.actions import (
    TnxFilterAction, TnxUpdateAction, MultiLinkAction, TnxBatchActionMixin)
from tionix_dashboard.helpers.utils import get_filter_value_by_dict
from tionix_dashboard.helpers.vm_constants import (
    PAUSED, SUSPENDED, SHELVED_OFFLOADED, ERROR,
    VMTypes)
from tionix_dashboard.dashboards.license import LicenseController


logger = logging.getLogger(__name__)


RUNNING = "RUNNING"


def get_status_by_name(value):
    """Return found status from display value"""
    if value is not None:
        value = unicode(value).lower()
        for actual_value, translation in project_tables.STATUS_DISPLAY_CHOICES:
            if unicode(translation).lower() == value:
                value = actual_value
                break
        return value


def get_power_states_by_name(value):
    """Return list of power states"""
    reversed_power_states = {
        v: k for k, v in project_tables.POWER_STATES.items()
    }
    power_states = [
        reversed_power_states.get(state)
        for state, state_name in project_tables.POWER_DISPLAY_CHOICES
        if value.lower() in unicode(state_name).lower()]
    return power_states or [None]


def is_vdi_instance(instance):
    """
    Checks if instance is vdi type
    :param instance:
    :return:
    """
    return unicode(getattr(instance, "tnx_type", '')) == VMTypes.VDI


class TnxInstancesFilterAction(TnxFilterAction):
    """Filter action for instances table"""
    filter_type = "server"
    filter_choices = (('name', _("Instance Name"), True),
                      ('image', _("Image ID ="), True),
                      ('ip', _("IPv4 Address"), True),
                      ('ip6', _("IPv6 Address"), True),
                      ('flavor', _("Flavor ID ="), True),
                      ('tnx_type', _("Type ="), True),
                      ('status', _("Status ="), True),
                      ('OS-EXT-AZ:availability_zone',
                       _("Availability zone"),
                       False),
                      ('power_state', _("Power state"), True),
                      ('uuid', _("Instance ID ="), True),)

    filter_string_modifiers = {
        'status': get_status_by_name,
        'tnx_type': get_filter_value_by_dict(VMTypes.TYPES,
                                             not_found_none=False),
        'power_state': get_power_states_by_name
    }


class TnxToggleMixin(object):
    policy = None
    toggle_type = None
    states = None
    current_present_action = None

    def allowed(self, request, instance=None):
        allowed = True

        if not api.nova.extension_supported('AdminActions', request):
            allowed &= False

        if instance and self.states:
            allowed &= instance.status in self.states

        if self.toggle_type is not None:
            self.current_present_action = self.toggle_type

        policy_check = getattr(settings, "POLICY_CHECK_FUNCTION", None)
        if policy_check:
            allowed &= policy_check(
                self.policy, request,
                target={'project_id': getattr(instance, 'tenant_id', None)})

        return allowed


class TnxMultiTogglePause(TnxBatchActionMixin, TnxToggleMixin,
                          project_tables.TogglePause):
    name = 'tnx_pause'
    classes = ("btn-danger",)
    icon = None
    policy = (("compute", "compute_extension:admin_actions:pause"),)
    toggle_type = project_tables.PAUSE
    paused = False
    states = project_tables.ACTIVE_STATES


class TnxMultiToggleUnPause(TnxBatchActionMixin, TnxToggleMixin,
                            project_tables.TogglePause):
    name = "tnx_unpause"
    icon = None
    policy = (("compute", "compute_extension:admin_actions:unpause"),)
    paused = True
    states = (PAUSED.upper(), )

    @staticmethod
    def action_present(count):
        return ungettext_lazy(
            u"Unpause Instance",
            u"Unpause Instances",
            count
        )

    @staticmethod
    def action_past(count):
        return ungettext_lazy(
            u"Unpaused Instance",
            u"Unpaused Instances",
            count
        )


class TnxMultiToggleSuspend(TnxBatchActionMixin, TnxToggleMixin,
                            project_tables.ToggleSuspend):
    name = 'tnx_suspend'
    classes = ("btn-danger",)
    icon = None
    policy = (("compute", "compute_extension:admin_actions:suspend"),)
    toggle_type = project_tables.PAUSE
    suspended = False
    states = project_tables.ACTIVE_STATES

    @staticmethod
    def action_present(count):
        return (
            ungettext_lazy(
                u"Suspend Instance",
                u"Suspend Instances",
                count
            )
        )

    @staticmethod
    def action_past(count):
        return (
            ungettext_lazy(
                u"Suspended Instance",
                u"Suspended Instances",
                count
            )
        )


class TnxMultiToggleResume(TnxBatchActionMixin, TnxToggleMixin,
                           project_tables.ToggleSuspend):
    name = "tnx_resume"
    icon = None
    policy = (("compute", "compute_extension:admin_actions:resume"),)
    toggle_type = project_tables.UNPAUSE
    suspended = True
    states = (SUSPENDED.upper(), )

    @staticmethod
    def action_present(count):
        return ungettext_lazy(
            u"Resume Instance",
            u"Resume Instances",
            count
        )

    @staticmethod
    def action_past(count):
        return ungettext_lazy(
            u"Resumed Instance",
            u"Resumed Instances",
            count
        )


class TnxToggleShelve(TnxBatchActionMixin, project_tables.ToggleShelve):

    @staticmethod
    def action_present(count):
        return (
            ungettext_lazy(
                u"Shelve Instance",
                u"Shelve Instances",
                count
            ),
            ungettext_lazy(
                u"Unshelve Instance",
                u"Unshelve Instances",
                count
            ),
        )

    @staticmethod
    def action_past(count):
        return (
            ungettext_lazy(
                u"Shelved Instance",
                u"Shelved Instances",
                count
            ),
            ungettext_lazy(
                u"Unshelved Instance",
                u"Unshelved Instances",
                count
            ),
        )

    def allowed(self, request, instance=None):
        allowed = super(TnxToggleShelve, self).allowed(request, instance)

        self.set_danger_btn_class(not self.shelved)

        if instance is not None and not self.shelved:
            allowed &= project_tables.get_power_state(instance) == RUNNING
        return allowed


class TnxMultiToggleShelve(TnxBatchActionMixin, TnxToggleMixin,
                           project_tables.ToggleShelve):
    name = 'tnx_shelve'
    classes = ("btn-danger",)
    icon = None
    policy = (("compute", "compute_extension:admin_actions:shelve"),)
    toggle_type = project_tables.SHELVE
    shelved = False
    states = project_tables.ACTIVE_STATES

    @staticmethod
    def action_present(count):
        return (
            ungettext_lazy(
                u"Shelve Instance",
                u"Shelve Instances",
                count
            )
        )

    @staticmethod
    def action_past(count):
        return (
            ungettext_lazy(
                u"Shelved Instance",
                u"Shelved Instances",
                count
            )
        )

    def allowed(self, request, instance=None):
        allowed = super(TnxMultiToggleShelve, self).allowed(request, instance)
        if instance is not None:
            allowed &= project_tables.get_power_state(instance) == RUNNING
        return allowed


class TnxMultiToggleUnShelve(TnxBatchActionMixin, TnxToggleMixin,
                             project_tables.ToggleShelve):
    name = 'tnx_unshelve'
    icon = None
    policy = (("compute", "compute_extension:admin_actions:unshelve"),)
    toggle_type = project_tables.UNSHELVE
    shelved = True
    states = (SHELVED_OFFLOADED.upper(), )

    @staticmethod
    def action_present(count):
        return ungettext_lazy(
            u"Unshelve Instance",
            u"Unshelve Instances",
            count
        )

    @staticmethod
    def action_past(count):
        return ungettext_lazy(
            u"Unshelved Instance",
            u"Unshelved Instances",
            count
        )


class TnxTogglePause(TnxBatchActionMixin, project_tables.TogglePause):

    classes = ("btn-danger",)

    @staticmethod
    def action_present(count):
        return (
            ungettext_lazy(
                u"Pause Instance",
                u"Pause Instances",
                count
            ),
            ungettext_lazy(
                u"Unpause Instance",
                u"Unpause Instances",
                count
            )
        )

    @staticmethod
    def action_past(count):
        return (
            ungettext_lazy(
                u"Paused Instance",
                u"Paused Instances",
                count
            ),
            ungettext_lazy(
                u"Unpaused Instance",
                u"Unpaused Instances",
                count
            )
        )

    def allowed(self, request, instance=None):
        allow = super(TnxTogglePause, self).allowed(request, instance)
        self.set_danger_btn_class(not self.paused)
        return allow


class TnxStopInstance(TnxBatchActionMixin, project_tables.StopInstance):

    @staticmethod
    def action_present(count):
        return npgettext_lazy(
            "Action to perform (the instance is currently running)",
            u"Shut Off Instance",
            u"Shut Off Instances",
            count
        )

    @staticmethod
    def action_past(count):
        return npgettext_lazy(
            "Past action (the instance is currently already Shut Off)",
            u"Shut Off Instance",
            u"Shut Off Instances",
            count
        )

    def allowed(self, request, instance):
        if instance is None:
            return True
        allow = super(TnxStopInstance, self).allowed(request, instance)
        return allow and instance.status.lower() != ERROR


class InstanceUpdateCell(TnxUpdateAction):
    """
    Action update instance name
    """

    def update_cell(self, request, datum, obj_id, cell_name, new_cell_value):
        """
        Update cell
        """
        result = True
        try:
            TNXServersClient.update(request, obj_id, name=new_cell_value)
        except Exception as err:
            logger.error(unicode(err))
            messages.error(request, _('Unable to update instance.'))
            result = False
        return result


class UpdateRow(tables.Row):
    ajax = True

    def get_data(self, request, instance_id):

        instance = TNXServersClient.get(request, instance_id)
        try:
            instance.full_flavor = api.nova.flavor_get(
                request,
                instance.flavor["id"])
        except Exception as err:
            logger.error(unicode(err))
            exceptions.handle(
                request,
                _('Unable to retrieve flavor '
                  'information for instance "%s".') % instance_id,
                ignore=True)

        try:
            api.network.servers_update_addresses(request, [instance])
        except Exception as err:
            logger.error(unicode(err))
            exceptions.handle(request,
                              _('Unable to retrieve Network information '
                                'for instance "%s".') % instance_id,
                              ignore=True)

        tenant_name = ''
        try:
            tenant = api.keystone.tenant_get(request,
                                             instance.tenant_id,
                                             admin=True)
        except Exception as err:
            logger.error(unicode(err))
        else:
            tenant_name = getattr(tenant, "name", None)
        instance.tenant_name = tenant_name

        try:
            host = TNXHypervisorsClient.search(
                request, getattr(instance, 'OS-EXT-SRV-ATTR:host')
            )[0]
            instance.host_id = host.id
            instance.node_title = getattr(host, 'tnx_node_title', '')
        except Exception as err:
            logger.error(unicode(err))

        return instance


class ResetStateAction(tables.LinkAction):
    """ Action reset vm's state."""
    name = 'reset_state'
    verbose_name = _('Reset state')
    policy_rules = (('identity', 'admin_required'),)
    url = "horizon:project:instances:reset_state"
    classes = ("ajax-modal", "btn-danger")
    states = (ERROR.upper(),)

    def get_link_url(self, instance=None):
        instance_id = instance.id if instance else None
        params = urlencode({'id': instance_id})
        return '?'.join([reverse(self.url), params])

    def allowed(self, request, instance):
        allowed = True

        if instance and self.states:
            allowed = instance.status in self.states

        if not LicenseController.is_valid(
                request, LicenseController.CLOUD_CONTROL):
            allowed = False
        return allowed


class MultiResetStateAction(MultiLinkAction):
    """ Action reset vm's state."""
    name = 'multi_reset_state'
    verbose_name = _('Reset state')
    url = "horizon:project:instances:reset_state"
    policy_rules = (('identity', 'admin_required'),)
    classes = ("btn-danger",)


class LaunchInstanceAllowedMixin(object):

    verbose_name = None
    classes = None

    def allowed(self, request, datum):
        """Check quotas"""
        try:
            limits = api.nova.tenant_absolute_limits(request, reserved=True)

            instances_available = (
                limits['maxTotalInstances'] - limits['totalInstancesUsed']
            )
            cores_available = (
                limits['maxTotalCores'] - limits['totalCoresUsed']
            )
            ram_available = limits['maxTotalRAMSize'] - limits['totalRAMUsed']

            if (instances_available <= 0 or
                    cores_available <= 0 or ram_available <= 0):
                if "disabled" not in self.classes:
                    self.classes = list(self.classes) + ['disabled']
                    self.verbose_name = string_concat(self.verbose_name, ' ',
                                                      _("(Quota exceeded)"))
            else:
                self.verbose_name = self.verbose_name_base
                self.classes = [c for c in self.classes if c != "disabled"]
        except Exception as err:
            logger.error(unicode(err))
            # If we can't get the quota information, leave it to the
            # API to check when launching
        return True  # The action should always be displayed


class LaunchBaseLink(LaunchInstanceAllowedMixin, tables.LinkAction):
    """Action to launch base instance"""
    name = "launch_base"
    verbose_name = verbose_name_base = _("Launch Base Instance")
    url = 'horizon:project:instances:launch_base'
    classes = ("ajax-modal", "btn-launch")
    icon = "cloud-upload"
    policy_rules = (("compute", "compute:create"),)
    ajax = True

    # pylint: disable=unused-argument
    def single(self, table, request, object_id=None):
        self.allowed(request, None)
        return HttpResponse(self.render(is_table_action=True))


class TnxDeleteInstance(policy.PolicyTargetMixin, TnxBatchActionMixin,
                        tables.LinkAction):
    """ Action to delete instance """
    name = 'delete'
    verbose_name = _("Delete Instance")
    classes = ("ajax-modal", "btn-danger")
    url = "horizon:project:instances:delete"
    policy_rules = (("compute", "compute:delete"),)
    help_text = _("Deleted instances are not recoverable.")

    def allowed(self, request, instance=None):
        """Allow delete action if instance not currently being deleted."""
        return not project_tables.is_deleting(instance)

    def get_link_url(self, instance=None):
        instance_id = instance.id if instance else None
        params = urlencode({'id': instance_id})
        return '?'.join([reverse(self.url), params])


class TnxMultiDeleteInstance(MultiLinkAction):
    """ Group action to delete instances """
    name = 'multi_delete'
    verbose_name = _('Delete Instances')
    classes = ("btn-danger",)
    url = "horizon:project:instances:delete"
    policy_rules = (("compute", "compute:delete"),)


class AuditFilterAction(TnxFilterAction):
    """Audit filter"""
    filter_type = "server"

    filter_choices = [('request_id', _("Request ID"), False),
                      ('action', _("Action"), False),
                      ('user_id', _("User ID"), False),
                      ('message', _("Message"), False)]


class TnxAuditFilterAction(TnxFilterAction):
    """Tionix Audit filter"""
    filter_type = "server"

    filter_choices = [('request_id', _("Request ID"), False),
                      ('action', _("Action"), False),
                      ('details', _("More"), False),
                      ('user', _("User"), False),
                      ('message', _("Message"), False)]


class ConfirmResize(project_tables.ConfirmResize):
    """Confirm resize"""
    on_the_top = True


class CloneNGLink(LaunchInstanceAllowedMixin, tables.LinkAction):
    """Clone instance action (LAUNCH_INSTANCE_NG_ENABLED)"""
    name = "clone-ng"
    verbose_name = verbose_name_base = _("Clone Instance")
    url = "horizon:project:instances:index"
    classes = ("btn-launch",)
    policy_rules = (("compute", "compute:create"),)
    ajax = False
    vdi_url = "horizon:tionix:vdi:instances:clone"

    def get_default_attrs(self):
        if self.datum and not is_vdi_instance(self.datum):
            url = reverse(self.url)
            ngclick = ("modal.openLaunchInstanceWizard({ successUrl: '%s', "
                       "event: $event, isClone: true })" % url)
            self.attrs.update({
                'ng-controller': 'LaunchInstanceModalController as modal',
                'ng-click': ngclick
            })
        return super(CloneNGLink, self).get_default_attrs()

    def get_link_url(self, instance=None):
        if is_vdi_instance(instance):
            return reverse(self.vdi_url, args=[instance.tenant_id,
                                               instance.id])
        self.attrs.update({'data-object-id': instance.id})
        return "javascript:void(0);"

    def allowed(self, request, instance):
        if LicenseController.is_valid(request,
                                      LicenseController.CLOUD_CONTROL):
            allowed = super(CloneNGLink, self).allowed(request, instance)
        else:
            allowed = False
        return allowed and instance.status.lower() != ERROR


class CloneLink(LaunchInstanceAllowedMixin, tables.LinkAction):
    """Clone instance action (LAUNCH_INSTANCE_LEGACY_ENABLED)"""
    name = "clone"
    verbose_name = verbose_name_base = _("Clone Instance")
    url = "horizon:project:instances:clone"
    vdi_url = "horizon:tionix:vdi:instances:clone"
    classes = ("ajax-modal", "btn-launch")
    policy_rules = (("compute", "compute:create"),)
    ajax = True

    def allowed(self, request, instance):
        if LicenseController.is_valid(request,
                                      LicenseController.CLOUD_CONTROL):
            allowed = super(CloneLink, self).allowed(request, instance)
        else:
            allowed = False
        return allowed and instance.status.lower() != ERROR

    def get_link_url(self, instance=None):
        if is_vdi_instance(instance):
            url = reverse(self.vdi_url, args=[instance.tenant_id, instance.id])
        else:
            url = reverse(self.url, args=[instance.id])

        return url

    # pylint: disable=unused-argument
    def single(self, table, request, object_id=None):
        return HttpResponse(self.render())


class ToggleSuspend(TnxBatchActionMixin, project_tables.ToggleSuspend):

    @staticmethod
    def action_present(count):
        return (
            ungettext_lazy(
                u"Suspend Instance",
                u"Suspend Instances",
                count
            ),
            ungettext_lazy(
                u"Resume Instance",
                u"Resume Instances",
                count
            ),
        )

    @staticmethod
    def action_past(count):
        return (
            ungettext_lazy(
                u"Suspended Instance",
                u"Suspended Instances",
                count
            ),
            ungettext_lazy(
                u"Resumed Instance",
                u"Resumed Instances",
                count
            ),
        )

    def allowed(self, request, instance=None):
        allowed = super(ToggleSuspend, self).allowed(request, instance)
        self.set_danger_btn_class(not self.suspended)
        return allowed


class TnxConsoleLink(project_tables.ConsoleLink):
    url = 'horizon:project:instances:console_tab'

    # pylint: disable=bad-super-call
    def get_link_url(self, datum=None):
        return super(project_tables.ConsoleLink, self).get_link_url(datum)


class TnxLogLink(project_tables.LogLink):
    url = 'horizon:project:instances:log_tab'

    # pylint: disable=bad-super-call
    def get_link_url(self, datum=None):
        return super(project_tables.LogLink, self).get_link_url(datum)
