# coding: utf-8

import logging

from django.utils.translation import ugettext_lazy as _

from horizon import exceptions
from horizon import messages
from openstack_dashboard import api

from openstack_dashboard import policy

from tionix_dashboard.clients.journal import TNXJournalClient
from tionix_dashboard.dashboards import utils


logger = logging.getLogger(__name__)


def get_instances_audit_data(request, instances):
    """
    Retrieve instances audit data extended with vdi journal
    :param request: django request
    :param instances: instances list
    :return:
    """

    journals = []
    users_dict = {}

    if isinstance(instances, basestring):
        instances = [instances]

    if policy.check((("identity", "identity:list_users"),), request):
        try:
            domain = getattr(request.user, 'user_domain_id', None)
            users_dict = {obj.id: obj.name for obj in utils.user_list(
                request, domain=domain)}
        except Exception as err:
            logger.error(unicode(err))
            exceptions.handle(request, _('Unable to retrieve user list.'))

    try:
        journals = TNXJournalClient.list(request, instance_id=instances,
                                         search_opts={'include_system': True})
    except Exception as err:
        logger.error(unicode(err))
        exceptions.handle(request, _('Unable to retrieve VDI journal.'))
    else:
        for journal in journals:
            journal.start_time = journal.created_at
            user_id = journal.user_id
            if user_id:
                journal.details = users_dict.get(user_id, user_id)
            request_user_id = journal.request_user_id
            journal.user = users_dict.get(request_user_id, request_user_id)

    return sorted(journals, reverse=True, key=lambda y: y.start_time)


def get_instance_security_groups(request, instance_id):
    """
    Retrieve security groups ids by instance_id
    :param request:
    :param instance_id:
    :return:
    """
    security_groups = []
    try:
        security_groups = api.network.server_security_groups(request,
                                                             instance_id)
    except Exception as err:
        logger.error(err)
        messages.error(request, _('Unable to get security groups.'))
    else:
        security_groups = [group.id for group in security_groups]

    return security_groups


def get_instance_networks_and_ports(request, instance_id):
    """
    Retrieve networks and ports ids by instance_id
    :param request:
    :param instance_id:
    :return:
    """
    networks, ports = [], []
    try:
        _ports = api.neutron.list_resources_with_long_filters(
            api.neutron.port_list,
            'device_id',
            [instance_id],
            request=request
        )
    except Exception as err:
        logger.error(err)
        messages.error(request, _('Unable to connect to Neutron.'))
    else:
        ports = [port.id for port in _ports]
        networks = set([port.network_id for port in _ports])

    return networks, ports
