# coding: utf-8

from django.conf import settings
from django.utils.translation import (
    ugettext_lazy as _, pgettext_lazy, ungettext_lazy)

from horizon import tables
from horizon.utils import filters

from openstack_dashboard.dashboards.project.instances import (
    tables as project_tables, audit_tables as project_audit_tables)
from tionix_dashboard.dashboards.tionix.metrics.instances.actions import (
    MetricsInstanceShowAction)

from tionix_dashboard.dashboards.tionix.scheduler.actions import (
    ProjectInstanceSchedulerMultiAction, ProjectInstanceSchedulerAction)
from tionix_dashboard.dashboards.project.project_instances import actions

from tionix_dashboard.helpers.actions import TnxBatchActionMixin
from tionix_dashboard.helpers.fields import EmptyCharField
from tionix_dashboard.helpers.tables import (
    DataTableFirstLastPagination, InstancesFilterPaginationMixin, TnxColumn,
    ServerFilterTabTableMixin)
from tionix_dashboard.helpers.vm_constants import VMTypes

from tionix_dashboard.templatetags.tionix_tags import format_date
from tionix_dashboard.templatetags.tionix_tags import timesince_sortable


class StartInstance(TnxBatchActionMixin, project_tables.StartInstance):
    @staticmethod
    def action_present(count):
        return ungettext_lazy(
            u"Start Instance",
            u"Start Instances",
            count
        )

    @staticmethod
    def action_past(count):
        return ungettext_lazy(
            u"Started Instance",
            u"Started Instances",
            count
        )


class SoftRebootInstance(TnxBatchActionMixin,
                         project_tables.SoftRebootInstance):
    @staticmethod
    def action_present(count):
        return ungettext_lazy(
            u"Soft Reboot Instance",
            u"Soft Reboot Instances",
            count
        )

    @staticmethod
    def action_past(count):
        return ungettext_lazy(
            u"Soft Rebooted Instance",
            u"Soft Rebooted Instances",
            count
        )


class LockInstance(TnxBatchActionMixin, project_tables.LockInstance):
    @staticmethod
    def action_present(count):
        return ungettext_lazy(
            u"Lock Instance",
            u"Lock Instances",
            count
        )

    @staticmethod
    def action_past(count):
        return ungettext_lazy(
            u"Locked Instance",
            u"Locked Instances",
            count
        )


class UnlockInstance(TnxBatchActionMixin, project_tables.UnlockInstance):
    @staticmethod
    def action_present(count):
        return ungettext_lazy(
            u"Unlock Instance",
            u"Unlock Instances",
            count
        )

    @staticmethod
    def action_past(count):
        return ungettext_lazy(
            u"Unlocked Instance",
            u"Unlocked Instances",
            count
        )


class RebootInstance(TnxBatchActionMixin, project_tables.RebootInstance):
    @staticmethod
    def action_present(count):
        return ungettext_lazy(
            u"Hard Reboot Instance",
            u"Hard Reboot Instances",
            count
        )

    @staticmethod
    def action_past(count):
        return ungettext_lazy(
            u"Hard Rebooted Instance",
            u"Hard Rebooted Instances",
            count
        )


class TnxInstancesTable(DataTableFirstLastPagination,
                        InstancesFilterPaginationMixin,
                        tables.DataTable):

    name = tables.Column(
        "name",
        link="horizon:project:instances:detail",
        verbose_name=_("Name"),
        form_field=EmptyCharField(max_length=255),
        update_action=actions.InstanceUpdateCell)
    image_name = tables.Column(
        "image_name",
        verbose_name=_("Image Name"))
    ip = tables.Column(
        project_tables.get_ips,
        verbose_name=_("IP Address"),
        attrs={'data-type': "ip"})
    size = tables.Column(
        project_tables.get_size,
        sortable=False,
        verbose_name=_("Size"))
    type = tables.Column(
        lambda x: unicode(getattr(x, "tnx_type", '') or ''),
        verbose_name=_("Type"),
        display_choices=VMTypes.TYPES.items())
    status = tables.Column(
        "status",
        filters=(project_tables.title, filters.replace_underscores),
        verbose_name=_("Status"),
        status=True,
        status_choices=project_tables.InstancesTable.STATUS_CHOICES,
        display_choices=project_tables.STATUS_DISPLAY_CHOICES)
    az = tables.Column(
        "availability_zone",
        verbose_name=_("Zone"))
    task = tables.Column(
        "OS-EXT-STS:task_state",
        verbose_name=_("Task"),
        empty_value=project_tables.TASK_DISPLAY_NONE,
        status=True,
        status_choices=project_tables.InstancesTable.TASK_STATUS_CHOICES,
        display_choices=project_tables.TASK_DISPLAY_CHOICES)
    state = tables.Column(
        project_tables.get_power_state,
        filters=(project_tables.title, filters.replace_underscores),
        verbose_name=_("Power State"),
        display_choices=project_tables.POWER_DISPLAY_CHOICES)
    created = tables.Column(
        "created",
        verbose_name=_("Time since created"),
        filters=(filters.parse_isotime,
                 timesince_sortable),
        attrs={'data-type': 'timesince'})

    class Meta(object):
        name = "instances"
        template = '_data_table.html'
        verbose_name = _("Instances")
        status_columns = ["status", "task"]
        row_class = actions.UpdateRow
        launch_actions = (actions.LaunchBaseLink, )
        if getattr(settings, 'LAUNCH_INSTANCE_LEGACY_ENABLED', False):
            launch_actions = launch_actions + (project_tables.LaunchLink,)
        if getattr(settings, 'LAUNCH_INSTANCE_NG_ENABLED', True):
            launch_actions = launch_actions + (project_tables.LaunchLinkNG,)
        table_actions = launch_actions + (actions.TnxInstancesFilterAction, )

        table_actions_menu = (
            ProjectInstanceSchedulerMultiAction,
            actions.TnxMultiDeleteInstance,
            StartInstance,
            actions.TnxStopInstance,
            SoftRebootInstance,
            actions.MultiResetStateAction,
            actions.TnxMultiTogglePause,
            actions.TnxMultiToggleUnPause,
            actions.TnxMultiToggleSuspend,
            actions.TnxMultiToggleResume,
            actions.TnxMultiToggleShelve,
            actions.TnxMultiToggleUnShelve)

        row_actions = (
            ProjectInstanceSchedulerAction,
            StartInstance,
            actions.ConfirmResize,
            project_tables.RevertResize,
            project_tables.CreateSnapshot,
            project_tables.SimpleAssociateIP,
            project_tables.AssociateIP,
            project_tables.SimpleDisassociateIP,
            project_tables.AttachInterface,
            project_tables.DetachInterface,
            project_tables.EditInstance,
            project_tables.UpdateMetadata,
            project_tables.DecryptInstancePassword,
            project_tables.EditInstanceSecurityGroups,
            actions.TnxConsoleLink,
            actions.TnxLogLink,
            actions.TnxTogglePause,
            actions.ToggleSuspend,
            project_tables.ResizeLink,
            LockInstance,
            UnlockInstance,
            SoftRebootInstance,
            RebootInstance,
            actions.TnxStopInstance,
            project_tables.RebuildInstance,
            actions.TnxDeleteInstance,
            actions.ResetStateAction,
            actions.TnxToggleShelve,
            MetricsInstanceShowAction)
        if getattr(settings, 'LAUNCH_INSTANCE_LEGACY_ENABLED', False):
            row_actions = row_actions + (actions.CloneLink,)
        if getattr(settings, 'LAUNCH_INSTANCE_NG_ENABLED', True):
            row_actions = row_actions + (actions.CloneNGLink,)


class InstancesTable(project_tables.InstancesTable):
    class Meta(project_tables.InstancesTable.Meta):

        table_actions_menu = (
            project_tables.InstancesTable.Meta.table_actions_menu +
            (ProjectInstanceSchedulerMultiAction,)
        )

        row_actions = (
            project_tables.InstancesTable.Meta.row_actions +
            (ProjectInstanceSchedulerAction, MetricsInstanceShowAction)
        )


class TnxAuditLog(ServerFilterTabTableMixin, tables.DataTable):

    ACTION_DISPLAY_CHOICES = (
        project_audit_tables.AuditTable.ACTION_DISPLAY_CHOICES +
        (("assign", pgettext_lazy("Action log of an instance", 'Assign user')),
         ("unassign", pgettext_lazy("Action log of an instance",
                                    'Unassign user')),
         ("shelve", pgettext_lazy("Action log of an instance", 'Shelve')),
         ("unshelve", pgettext_lazy("Action log of an instance", 'Unshelve')),
         ("addFloatingIp", pgettext_lazy("Action log of an instance",
                                         'Add floating ip')),
         ("addSecurityGroup", pgettext_lazy("Action log of an instance",
                                            'Add security group')),
         ("changePassword", pgettext_lazy("Action log of an instance",
                                          'Change password')),
         ("confirmResize", pgettext_lazy("Action log of an instance",
                                         'Confirm resize')),
         ("createImage", pgettext_lazy("Action log of an instance",
                                       'Create image')),
         ("lock", pgettext_lazy("Action log of an instance", 'Lock')),
         ("removeFloatingIp", pgettext_lazy("Action log of an instance",
                                            'Remove floating ip')),
         ("removeSecurityGroup", pgettext_lazy("Action log of an instance",
                                               'Remove security group')),
         ("rescue", pgettext_lazy("Action log of an instance", 'Rescue')),
         ("revertResize", pgettext_lazy("Action log of an instance",
                                        'Revert resize')),
         ("os-start", pgettext_lazy("Action log of an instance", 'Start')),
         ("os-stop", pgettext_lazy("Action log of an instance", 'Stop')),
         ("unrescue", pgettext_lazy("Action log of an instance", 'Unrescue')),
         ("addFixedIp", pgettext_lazy("Action log of an instance",
                                      'Add fixed ip')),
         ("removeFixedIp", pgettext_lazy("Action log of an instance",
                                         'Remove fixed ip')),
         ("evacuate", pgettext_lazy("Action log of an instance", 'Evacuate')),
         ("forceDelete", pgettext_lazy("Action log of an instance",
                                       'Force delete')),
         ("restore", pgettext_lazy("Action log of an instance", 'Restore')),

         ("os-getConsoleOutput", pgettext_lazy("Action log of an instance",
                                               'Get output console')),
         ("os-getRDPConsole", pgettext_lazy("Action log of an instance",
                                            'Get RDP console')),
         ("os-getSerialConsole", pgettext_lazy("Action log of an instance",
                                               'Get serial console')),
         ("os-getSPICEConsole", pgettext_lazy("Action log of an instance",
                                              'Get SPICE console')),
         ("os-getVNCConsole", pgettext_lazy("Action log of an instance",
                                            'Get VNC console')),
         ("shelveOffload", pgettext_lazy("Action log of an instance",
                                         'Shelve offload')),
         ("createBackup", pgettext_lazy("Action log of an instance",
                                        'Create backup')),
         ("injectNetworkInfo", pgettext_lazy("Action log of an instance",
                                             'Inject network info')),
         ("migrate", pgettext_lazy("Action log of an instance", 'Migrate')),
         ("os-migrateLive", pgettext_lazy("Action log of an instance",
                                          'Live migrate')),
         ("os-resetState", pgettext_lazy("Action log of an instance",
                                         'Reset state')))
    )
    request_id = tables.Column('request_id',
                               verbose_name=_('Request ID'))
    action = TnxColumn('action', verbose_name=_('Action'),
                       display_choices=ACTION_DISPLAY_CHOICES)
    details = tables.Column('details', verbose_name=_('More'))
    start_time = tables.Column('start_time', verbose_name=_('Start time'),
                               filters=(format_date, ))
    user = tables.Column('user', verbose_name=_('User'))
    message = tables.Column('message', verbose_name=_('Message'))

    class Meta(object):
        name = 'audit'
        verbose_name = _('Instance Action List')
        multi_select = False
        table_actions = (actions.TnxAuditFilterAction,)


class AuditTable(ServerFilterTabTableMixin, project_audit_tables.AuditTable):

    class Meta(object):
        name = 'audit'
        verbose_name = _('Instance Action List')
        table_actions = (actions.AuditFilterAction,)
