# coding: utf-8

"""Tnx project volumes helpers"""
import logging

from django.utils.functional import cached_property
from django.utils.translation import ugettext_lazy as _

from horizon import exceptions
from horizon.utils import functions as utils
from horizon.utils import memoized

from openstack_dashboard.api.cinder import (
    cinderclient, transfer_list, VERSIONS, Volume,
    VolumeSnapshot, VolumeBackup, update_pagination)
from tionix_dashboard.helpers.utils import get_table_filter_action

logger = logging.getLogger(__name__)


def get_cinder_backups(request, search_opts=None):
    """Retrieve backups by volume"""
    backups = []
    cinder_client = cinderclient(request)
    if cinder_client:
        try:
            backups = cinder_client.backups.list(search_opts=search_opts)
        except Exception as err:
            logger.error(err)
            exceptions.handle(request,
                              _("Unable to retrieve volume backups."))
    return backups


def get_object_list_paged(request, object_manager, object_wrapper,
                          search_opts=None, marker=None, paginate=False,
                          sort_dir="desc"):
    page_size = utils.get_page_size(request)
    has_more_data = False
    has_prev_data = False
    volumes = []

    c_client = cinderclient(request)
    manager = getattr(c_client, object_manager, None)
    if c_client is None or manager is None:
        return volumes, has_more_data, has_prev_data

    # build a dictionary of volume_id -> transfer
    transfers = {t.volume_id: t
                 for t in transfer_list(request, search_opts=search_opts)}

    kwargs = {'search_opts': search_opts}

    if VERSIONS.active > 1:
        # sort_key and sort_dir deprecated in kilo, use sort
        # if pagination is true, we use a single sort parameter
        # by default, it is "created_at"
        sort = 'created_at:' + sort_dir
        kwargs.update({'marker': marker, 'sort': sort})
        if paginate:
            kwargs.update({'limit': page_size + 1})

    for v in manager.list(**kwargs):
        v.transfer = transfers.get(v.id)
        volumes.append(object_wrapper(v))

    if paginate:
        volumes, has_more_data, has_prev_data = update_pagination(
            volumes, page_size, marker, sort_dir)

    return volumes, has_more_data, has_prev_data


def volume_list_paged(request, search_opts=None, marker=None,
                      paginate=False, sort_dir="desc"):
    """To see all volumes in the cloud as an admin you can pass in a special
    search option: {'all_tenants': 1}
    """
    return get_object_list_paged(request, 'volumes', Volume, search_opts,
                                 marker, paginate, sort_dir)


def volume_snapshot_list_paged(request, search_opts=None, marker=None,
                               paginate=False, sort_dir="desc"):
    return get_object_list_paged(request, 'volume_snapshots', VolumeSnapshot,
                                 search_opts, marker, paginate, sort_dir,)


def volume_backup_list_paged(request, search_opts=None, marker=None,
                             paginate=False, sort_dir="desc"):
    return get_object_list_paged(request, 'backups', VolumeBackup, search_opts,
                                 marker, paginate, sort_dir)


class PagedTableFilterMixin(object):

    data = None

    @memoized.memoized_method
    def page_size(self):
        return utils.get_page_size(self.request)

    @cached_property
    def filtered_data(self):
        filtered_data = self.data
        filter_action = get_table_filter_action(self)
        if self._meta.filter and filter_action:
            filter_string = self.get_filter_string()
            filter_field = self.get_filter_field()
            request_method = self.request.method
            needs_preloading = (not filter_string
                                and request_method == 'GET'
                                and filter_action.needs_preloading)
            valid_method = (request_method == filter_action.method)
            not_api_filter = (
                filter_string
                and not filter_action.is_api_filter(filter_field)
            )

            if valid_method or needs_preloading or not_api_filter:
                if self._meta.mixed_data_type:
                    filtered_data = filter_action.data_type_filter(
                        self, self.data, filter_string)
                else:
                    filtered_data = filter_action.filter(
                        self, self.data, filter_string)

            if self.kwargs.get('has_not_api_filters'):
                page_size = self.page_size()
                filtered_data, has_more_data, has_prev_data = (
                    update_pagination(
                        filtered_data[:page_size + 1], page_size,
                        self.kwargs.get('marker'),
                        self.kwargs.get('sort_dir', 'desc')
                    )
                )
                self._meta.has_more_data = has_more_data
                self._meta.has_prev_data = has_prev_data
                self.data = filtered_data
        return filtered_data
