# coding: utf-8

import logging

from django.utils.translation import ugettext_lazy as _

from horizon import exceptions

from openstack_dashboard import api
from openstack_dashboard.dashboards.project.volumes import (
    tabs as volumes_tabs)

from tionix_dashboard.dashboards.project.project_volumes.helpers import (
    volume_list_paged, volume_snapshot_list_paged, volume_backup_list_paged)
from tionix_dashboard.dashboards.project.project_volumes.volumes import (
    tables as tnx_volumes_tables)
from tionix_dashboard.dashboards.project.project_volumes.snapshots import (
    tables as tnx_snapshots_tables)
from tionix_dashboard.dashboards.project.project_volumes.backups import (
    tables as tnx_backups_tables)
from tionix_dashboard.helpers.views import ServerFilterTabMixin


logger = logging.getLogger(__name__)


class TnxVolumeTab(ServerFilterTabMixin, volumes_tabs.VolumeTab):
    table_classes = (tnx_volumes_tables.TnxVolumesTableBase, )

    def _get_volumes(self, search_opts=None):
        volumes = []
        marker, sort_dir = self._get_marker()
        filters = self.get_filters(include_not_api=True)
        marker = filters.pop('marker', marker)
        has_not_api_filters = bool(filters)
        try:
            volumes, self._has_more_data, self._has_prev_data = (
                volume_list_paged(self.request, marker=marker,
                                  sort_dir=sort_dir, search_opts=search_opts,
                                  paginate=not has_not_api_filters)
            )
            if sort_dir == "asc":
                volumes.reverse()
        except Exception as err:
            logger.error(unicode(err))
            exceptions.handle(self.request,
                              _('Unable to retrieve volume list.'))

        self.table.kwargs.update(has_not_api_filters=bool(filters),
                                 marker=marker, sort_dir=sort_dir)
        return volumes


class SnapshotTab(ServerFilterTabMixin, volumes_tabs.SnapshotTab):
    table_classes = (tnx_snapshots_tables.VolumeSnapshotsTable,)
    _has_more_data = False
    _has_prev_data = False

    def _get_snapshots(self, search_opts=None):
        snapshots = []
        marker, sort_dir = self._get_marker()
        filters = self.get_filters(include_not_api=True)
        marker = filters.pop('marker', marker)
        has_not_api_filters = bool(filters)
        try:
            snapshots, self._has_more_data, self._has_prev_data = (
                volume_snapshot_list_paged(
                    self.request, paginate=not has_not_api_filters,
                    search_opts=search_opts,
                    marker=marker, sort_dir=sort_dir)
            )
        except Exception as err:
            logger.error(unicode(err))
            exceptions.handle(self.request, _("Unable to retrieve "
                                              "volume snapshots."))
        self.table.kwargs.update(has_not_api_filters=bool(filters),
                                 marker=marker, sort_dir=sort_dir)

        return snapshots

    def get_volume_snapshots_data(self):
        snapshots = []
        if api.base.is_service_enabled(self.request, 'volumev2'):
            snapshots = self._get_snapshots()
            if snapshots:
                try:
                    volumes = {
                        volume.id: volume
                        for volume in api.cinder.volume_list(self.request)}
                except Exception as err:
                    logger.error(unicode(err))
                    exceptions.handle(self.request,
                                      _("Unable to retrieve "
                                        "snapshot volume information."))
                else:
                    for snapshot in snapshots:
                        volume = volumes.get(snapshot.volume_id)
                        setattr(snapshot, '_volume', volume)

        return snapshots


class BackupsTab(ServerFilterTabMixin, volumes_tabs.BackupsTab):
    table_classes = (tnx_backups_tables.BackupsTable,)

    def get_volume_backups_data(self):
        backups = []
        marker, sort_dir = self._get_marker()
        filters = self.get_filters(include_not_api=True)
        marker = filters.pop('marker', marker)
        has_not_api_filters = bool(filters)
        try:
            backups, self._has_more_data, self._has_prev_data = (
                volume_backup_list_paged(
                    self.request, marker=marker, sort_dir=sort_dir,
                    paginate=not has_not_api_filters)
            )
            volumes = {volume.id: volume
                       for volume in api.cinder.volume_list(self.request)}
        except Exception as err:
            logger.error(unicode(err))
            exceptions.handle(self.request, _("Unable to retrieve "
                                              "volume backups."))
        else:
            for backup in backups:
                backup.volume = volumes.get(backup.volume_id)

        self.table.kwargs.update(has_not_api_filters=bool(filters),
                                 marker=marker, sort_dir=sort_dir)
        return backups


class TnxVolumeAndSnapshotTabs(volumes_tabs.VolumeAndSnapshotTabs):
    tabs = (TnxVolumeTab, SnapshotTab, BackupsTab)
