# coding: utf-8

import logging

from django.core.urlresolvers import reverse, reverse_lazy
from django.utils.http import urlencode
from django.utils.translation import (
    ugettext_lazy as _, string_concat, ungettext_lazy
)

from horizon import exceptions
from horizon import messages
from horizon import tables

from openstack_dashboard import api
from openstack_dashboard.api import cinder
from openstack_dashboard.dashboards.project.volumes.volumes import (
    tables as volumes_tables)

from tionix_dashboard.clients import TNXVolumesClient
from tionix_dashboard.dashboards.project.project_volumes.actions import (
    BaseVolumeFilterAction)
from tionix_dashboard.helpers.actions import (
    MultiLinkAction, TnxUpdateAction, TnxBatchActionMixin)
from tionix_dashboard.helpers.utils import generate_timestamp_name

logger = logging.getLogger(__name__)


class TnxCreateSnapshot(volumes_tables.CreateSnapshot):
    def get_link_url(self, datum=None):
        params = urlencode({'id': datum.id if datum else ''})
        return '?'.join([reverse(self.url), params])


class TnxMultiCreateSnapshot(volumes_tables.VolumePolicyTargetMixin,
                             MultiLinkAction):
    name = 'multi_snapshots'
    verbose_name = _('Create Snapshot')
    url = 'horizon:project:volumes:volumes:create_snapshot'
    policy_rules = (('volume', 'volume:create_snapshot'),)

    def allowed(self, request, datum):
        try:
            limits = cinder.tenant_absolute_limits(request)
        except Exception as err:
            logger.error(unicode(err))
            exceptions.handle(request,
                              _('Unable to retrieve tenant limits.'))
            limits = {}

        snapshots_available = (
            limits.get('maxTotalSnapshots', float('inf')) -
            limits.get('totalSnapshotsUsed', 0)
        )

        if snapshots_available <= 0 and 'disabled' not in self.classes:
            self.classes = [c for c in self.classes] + ['disabled']
            self.verbose_name = string_concat(self.verbose_name, ' ',
                                              _('(Quota exceeded)'))
        return super(TnxMultiCreateSnapshot, self).allowed(request,
                                                           datum)


class TnxCreateBackup(volumes_tables.VolumePolicyTargetMixin,
                      TnxBatchActionMixin, tables.BatchAction):
    name = "backup"
    classes = ('btn-danger',)
    policy_rules = (("volume", "backup:create"),)
    success_url = reverse_lazy("horizon:project:volumes:backups_tab")

    @staticmethod
    def action_present(count):
        return ungettext_lazy(
            u"Create Volume backup",
            u"Create Volumes backups",
            count
        )

    @staticmethod
    def action_past(count):
        return ungettext_lazy(
            u"Created Volume backup",
            u"Created Volumes backups",
            count
        )

    def allowed(self, request, volume=None):
        return (cinder.volume_backup_supported(request) and (
            volume is None or volume.status == "available"))

    def action(self, request, obj_id):

        try:
            volume = api.cinder.volume_get(request, obj_id)
        except Exception as err:
            logger.error(unicode(err))
        else:
            name = generate_timestamp_name(volume.name)
            api.cinder.volume_backup_create(request, obj_id, None, name, None)


class VolumeUpdateCell(TnxUpdateAction):
    """
    Action update volume cell
    """

    def update_cell(self, request, datum, obj_id, cell_name,
                    new_cell_value):
        """
        Update cell
        """
        result = True
        try:
            TNXVolumesClient.update(
                request, obj_id, {cell_name: new_cell_value})
        except Exception as err:
            logger.error(unicode(err))
            messages.error(request, _('Unable to update volume.'))
            result = False
        return result


class VolumeSetBootableAction(tables.UpdateAction):
    """ Action set volume as bootable. """

    def allowed(self, request, project, cell):
        """
        check permission
        """
        return api.keystone.keystone_can_edit_project()

    def update_cell(self, request, datum, obj_id, cell_name,
                    new_cell_value):
        """ Inline update of bootable-cell. """
        if cell_name == 'bootable' and datum.is_bootable != new_cell_value:
            try:
                cinder.volume_set_bootable(request, obj_id,
                                           new_cell_value)
            except Exception as err:
                logger.error(unicode(err))
                exceptions.handle(
                    request,
                    _('Unable to set bootable flag on volume.'),
                    redirect=reverse("horizon:project:volumes:index")
                )


class TnxDeleteVolume(tables.LinkAction, TnxBatchActionMixin,
                      volumes_tables.DeleteVolume):
    """ Action to delete instance """
    name = "delete"
    verbose_name = _("Delete Volume")
    classes = ("ajax-modal", "btn-danger")
    url = "horizon:project:volumes:volumes:delete"

    def get_link_url(self, volume=None):
        params = urlencode({'id': volume.id if volume else None})
        return '?'.join([reverse(self.url), params])


class TnxMultiDeleteVolume(MultiLinkAction):
    """ Group action to delete instances """
    name = 'multi_delete'
    verbose_name = _('Delete Volumes')
    classes = ("btn-danger",)
    url = "horizon:project:volumes:volumes:delete"
    policy_rules = (("volume", "volume:delete"),)


class VolumeFilterAction(BaseVolumeFilterAction):
    """Volume snapshots filter"""
    filter_type = "server"
    filter_choices = (BaseVolumeFilterAction.filter_choices +
                      (('volume_tape', _("Type"), False),
                       ('attachments', _("Attached To"), False),
                       ('availability_zone', _("Availability Zone"), False),
                       ('bootable', _("Bootable"), False),
                       ('encryption', _("Encrypted"), False)))
    escape_enabled = False
