# coding: utf-8

import logging

from django.core.urlresolvers import reverse, reverse_lazy
from django.utils.translation import ugettext_lazy as _

from horizon import exceptions, forms, messages, tabs as horizon_tabs
from horizon.utils import memoized

from openstack_dashboard.api import cinder, nova
from openstack_dashboard.usage import quotas
from openstack_dashboard.utils import filters
from openstack_dashboard.dashboards.project.volumes.volumes import (
    views as volume_views)

from tionix_scheduler_client.enums import ObjectTypeEnum

from tionix_dashboard.dashboards.project.project_volumes.volumes import (
    tables, tabs)
from tionix_dashboard.dashboards.project.project_volumes.volumes.forms import (
    TnxCreateSnapshotForm, DeleteVolumeForm, CreateForm, CreateTransferForm)
from tionix_dashboard.dashboards.tionix.scheduler.actions import (
    ObjectTasksViewMixin)
from tionix_dashboard.helpers.views import (
    ServerFilterViewMixin, ServerSortMultiSelect)

logger = logging.getLogger(__name__)


class TnxCreateSnapshotView(forms.ModalFormView):
    form_class = TnxCreateSnapshotForm
    modal_header = _("Create Volume Snapshot")
    template_name = 'project/volumes/volumes/create_snapshot.html'
    submit_url = "horizon:project:volumes:volumes:create_snapshot"
    success_url = reverse_lazy('horizon:project:volumes:snapshots_tab')
    page_title = _("Create Volume Snapshot")

    def get_context_data(self, **kwargs):
        context = super(TnxCreateSnapshotView, self).get_context_data(**kwargs)
        ids = context_ids = self.request.GET.getlist('id', [])
        attached_volumes, failed_volumes = [], []

        for _id in ids:
            try:
                volume = cinder.volume_get(self.request, _id)
                if volume.status in ("available", "in-use"):
                    if volume.status == 'in-use':
                        attached_volumes.append(volume.name)
                else:
                    failed_volumes.append(volume.name)
                    context_ids.remove(_id)
            except Exception as err:
                logger.error(unicode(err))
                exceptions.handle(self.request,
                                  _('Unable to retrieve volume information.'))

        try:
            context['usages'] = quotas.tenant_limit_usages(self.request)
        except Exception as err:
            logger.error(unicode(err))
            exceptions.handle(self.request,
                              _('Unable to retrieve tenant usages.'))

        if attached_volumes:
            context['attached'] = True
            context['form'].set_warning(
                _('Volumes %s is currently attached to an instance. '
                  'In some cases, creating a snapshot from an attached '
                  'volume can result in a corrupted snapshot.') %
                ','.join(attached_volumes)
            )

        if failed_volumes:
            msg = _('"Create Volume Snapshot" is '
                    'not available for volumes: "%s"')
            messages.error(self.request, msg % '", "'.join(failed_volumes))

        context['submit_url'] = '?id='.join([reverse(self.submit_url),
                                             '&id='.join(context_ids)])

        return context

    def get_initial(self):
        return {'volumes_ids': self.request.GET.getlist('id', [])}


class DetailView(ServerSortMultiSelect,
                 ServerFilterViewMixin,
                 horizon_tabs.TabbedTableView):

    tab_group_class = tabs.VolumeDetailTabs
    template_name = 'horizon/common/_detail.html'
    page_title = "{{ volume.name|default:volume.id }}"

    def get_context_data(self, **kwargs):
        context = super(DetailView, self).get_context_data(**kwargs)
        volume = self.get_data()
        table = tables.TnxVolumesTableBase(self.request)
        context["volume"] = volume
        context["url"] = self.get_redirect_url()
        context["actions"] = table.render_row_actions(volume)
        choices = table.STATUS_DISPLAY_CHOICES
        volume.status_label = filters.get_display_label(choices, volume.status)
        return context

    @memoized.memoized_method
    def get_data(self):
        volume = None
        volume_id = self.kwargs['volume_id']
        try:
            volume = cinder.volume_get(self.request, volume_id)
            snapshots = cinder.volume_snapshot_list(
                self.request, search_opts={'volume_id': volume.id})
            if snapshots:
                setattr(volume, 'has_snapshot', True)
            for att in volume.attachments:
                att['instance'] = nova.server_get(self.request,
                                                  att['server_id'])
        except Exception as err:
            logger.error(err)
            redirect = self.get_redirect_url()
            exceptions.handle(self.request,
                              _('Unable to retrieve volume details.'),
                              redirect=redirect)
        return volume

    def get_redirect_url(self):
        return reverse('horizon:project:volumes:index')

    def get_tabs(self, request, **kwargs):
        volume = self.get_data()
        return self.tab_group_class(request, volume=volume, **kwargs)


class DeleteVolumeView(ObjectTasksViewMixin, forms.ModalFormView):
    """ Delete volumes view """

    form_class = DeleteVolumeForm
    modal_header = _('Delete Confirmation')
    template_name = "tionix_dashboard/delete_object.html"
    modal_id = "delete_volume"
    submit_label = _('Delete')
    context_object_name = 'volume'
    success_url = reverse_lazy("horizon:project:volumes:index")
    confirm_url = reverse_lazy("horizon:project:volumes:volumes:delete")
    hidden_title = False
    failure_msg = _('"Delete" is not available for volume')
    msg = _("Deleted volumes are not recoverable. "
            "All data stored in the volume will be removed.")

    @property
    def redirect_url(self):
        return self.request.META['HTTP_REFERER']

    def get_context_data(self, **kwargs):
        context = super(DeleteVolumeView, self).get_context_data(**kwargs)
        context["id"] = self.request.GET.getlist('id')
        return context

    def get_initial(self):
        # FIXME many requests by id
        ids = self.request.GET.getlist('id')
        names = {}
        for volume_id in ids:
            try:
                volume = cinder.volume_get(self.request, volume_id)
            except Exception as err:
                logger.error(unicode(err))
                redirect = reverse(self.redirect_url)
                exceptions.handle(self.request,
                                  _('Unable to retrieve details for '
                                    'volume "%s".') % volume_id,
                                  redirect=redirect)
                # Not all exception types handled above will result in a
                # redirect. Need to raise here just in case.
                raise exceptions.Http302(redirect)
            else:
                names[volume_id] = volume.name

        query_params = {
            'subject_id': ids,
            'subject_type': ObjectTypeEnum.volume,
            'status': True,
        }
        rows = self.get_tasks_rows(self.request, query_params)
        return {
            'rows': rows,
            'object_names_dict': names,
            'msg': self.msg,
            'confirm_url': self.confirm_url,
            'redirect_url': self.redirect_url,
        }


class CreateView(volume_views.CreateView):
    form_class = CreateForm


class CreateTransferView(volume_views.CreateTransferView):
    form_class = CreateTransferForm
