# coding: utf-8
# pylint: disable=no-member

import logging

from django.core.exceptions import ObjectDoesNotExist
from django.core.urlresolvers import reverse_lazy, reverse
from django.http.response import JsonResponse
from django.utils.translation import ugettext_lazy as _

from horizon import exceptions, views
from horizon import messages
from horizon import workflows
from horizon.utils import memoized

from tionix_dashboard.clients import TNXServersClient
from tionix_dashboard.clients.exceptions import UrlNotAvailableError
from tionix_dashboard.dashboards.tionix.metrics import constants
from tionix_dashboard.dashboards.tionix.metrics.api import (
    get_instance_metric_by_id)
from tionix_dashboard.dashboards.tionix.metrics.constants import METRICS_INDEX
from tionix_dashboard.dashboards.tionix.metrics.enums import MetricsTypes
from tionix_dashboard.dashboards.tionix.metrics.models import UserMetrics
from tionix_dashboard.dashboards.tionix.metrics.workflows import (
    MetricsSettingsWorkflow)

logger = logging.getLogger(__name__)


def get_metrics_data(request, instance_id):
    """Return metrics for instance"""
    metrics = get_instance_metric_by_id(request, instance_id)
    return JsonResponse(data={'data': metrics})


class InstanceMetricsView(views.HorizonTemplateView):
    template_name = 'metrics/instances/metrics.html'
    page_title = "{{ instance.name }}"

    def get_context_data(self, **kwargs):
        context = super(InstanceMetricsView, self).get_context_data(**kwargs)
        instance = self.get_data()
        context['instance_metrics_url'] = reverse_lazy(
            'horizon:tionix:metrics:instances:data', args=[instance.id]
        )
        context["instance"] = instance
        return context

    @memoized.memoized_method
    def get_data(self):
        instance = None
        try:
            instance = TNXServersClient.get(self.request,
                                            self.kwargs['instance_id'])
        except UrlNotAvailableError:
            raise
        except Exception as err:
            logger.error(err)
            exceptions.handle(self.request,
                              _('Unable to retrieve instance details.'),
                              redirect=reverse(METRICS_INDEX))
        return instance


class MetricsInstanceSettingsView(workflows.WorkflowView):
    workflow_class = MetricsSettingsWorkflow
    success_url = reverse_lazy(constants.METRICS_INDEX)
    page_title = _("Settings")

    def get_initial(self):
        initial = super(MetricsInstanceSettingsView, self).get_initial()

        try:
            metrics_names = TNXServersClient.get_metrics_names(self.request)
        except UrlNotAvailableError:
            raise
        except Exception as err:
            logger.error(unicode(err))
            exceptions.handle(self.request,
                              _('Unable to get available metrics list.'))
            raise exceptions.Http302(self.success_url)
        else:
            if not metrics_names:
                messages.error(self.request, _('No metrics found.'))
                raise exceptions.Http302(self.success_url)

        try:
            user_metrics_names = UserMetrics.objects.get(
                user_id=self.request.user.id,
                metric_type=MetricsTypes.INSTANCE
            ).metrics_names
        except ObjectDoesNotExist:
            user_metrics_names = []

        initial.update(metric_type=MetricsTypes.INSTANCE,
                       metrics_names=metrics_names,
                       user_metrics_names=user_metrics_names)
        return initial
