# coding: utf-8
import logging

from django.core.urlresolvers import reverse
from django.utils.http import urlencode
from django.utils.translation import ugettext_lazy as _

from horizon import tables
from horizon import exceptions

from tionix_dashboard.clients import TNXPowerDevicesClient
from tionix_dashboard.clients.exceptions import UrlNotAvailableError
from tionix_dashboard.helpers.actions import MultiLinkAction, TnxUpdateAction
from tionix_dashboard.url_resolve_map import (
    POWER_DEVICES_INDEX, POWER_DEVICES_DELETE, POWER_DEVICES_CREATE,
    POWER_DEVICES_CLONE)


logger = logging.getLogger(__name__)


class DeletePowerDeviceAction(tables.LinkAction):

    verbose_name = _('Delete power device')
    name = 'delete_power_device'
    classes = ('ajax-modal', 'btn-danger')
    url = POWER_DEVICES_DELETE

    def get_link_url(self, datum=None):
        params = urlencode({'id': datum.id if datum else ''})
        return '?'.join([reverse(self.url), params])


class DeletePowerDeviceMultiAction(MultiLinkAction):
    """Power device delete action."""

    name = 'multi_device_delete'
    verbose_name = _('Delete power devices')
    url = POWER_DEVICES_DELETE
    classes = ('btn-danger', )


class CreatePowerDevice(tables.LinkAction):
    """Power device create action."""
    name = 'create'
    verbose_name = _('Create power device')
    url = POWER_DEVICES_CREATE
    classes = ('ajax-modal',)
    icon = 'plus'


class PowerDeviceFilterAction(tables.FilterAction):
    """Power device filter table action."""
    def filter(self, table, data, filter_string):
        """Really naive case-insensitive search."""
        q = filter_string.lower()

        def comp(power_device):
            return q in power_device.name.lower()

        return filter(comp, data)


class TnxInstancesUpdateRow(tables.Row):
    """
    Action updated power device rows in table.
    """
    ajax = True

    def get_data(self, request, obj_id):
        try:
            device = TNXPowerDevicesClient.get(request, obj_id)
        except UrlNotAvailableError:
            raise
        except Exception as err:
            logger.error(unicode(err))
            exceptions.handle(
                request,
                _('Unable to get power device.'),
                redirect=reverse(POWER_DEVICES_INDEX))
            device = None

        return device


class DeviceTitleUpdateCell(TnxUpdateAction):
    """
    Action update device title
    """

    def update_cell(self, request, datum, obj_id, cell_name, new_cell_value):
        """
        Update cell
        """
        try:
            TNXPowerDevicesClient.update(
                request, obj_id, **{cell_name: new_cell_value})
        except Exception as err:
            exceptions.handle(
                request,
                _(unicode(err.message)))


class ClonePowerDevice(tables.LinkAction):
    """Power device clone action."""
    name = 'clone'
    verbose_name = _('Clone power device')
    url = POWER_DEVICES_CLONE
    classes = ('ajax-modal',)
