# coding: utf-8

import logging

from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _

from horizon import exceptions
from horizon import forms
from horizon import messages
from horizon.utils import validators


from tionix_dashboard.clients import TNXPowerDevicesClient
from tionix_dashboard.clients.exceptions import UrlNotAvailableError
from tionix_dashboard.url_resolve_map import POWER_DEVICES_INDEX

logger = logging.getLogger(__name__)


class CreatePowerDeviceForm(forms.SelfHandlingForm):
    """Power device create form."""
    name = _('Create power device')
    device_name = forms.ChoiceField(label=_('Device type'),
                                    help_text=_('Choose a device type.'))
    protocol_type = forms.ChoiceField(label=_('Protocol type'),
                                      help_text=_('Choose a protocol type.'))
    title = forms.CharField(
        label=_('Device title'),
        help_text=_('Input device title. Maximum 50 symbols'),
        max_length=50
    )
    host = forms.CharField(
        label=_('Host or ip'),
        help_text=_('Input host or ip of device. '
                    'Maximum 255 symbols, cyrillic characters not allowed'),
        validators=[validators.validators.URLValidator],
        max_length=255,
        widget=forms.TextInput(attrs={
            'onkeypress': 'return onPressKeyHost(event);'
        })
    )
    port_number = forms.IntegerField(
        label=_('Port'),
        help_text=_('Input port of device. Must be in the range '
                    'from 1 to 65535'),
        validators=[validators.validate_port_range],
        min_value=1,
        max_value=65535
    )
    login = forms.CharField(
        label=_('Device login'),
        required=False,
        max_length=255
    )
    password = forms.RegexField(
        label=_('Device password'), required=False,
        widget=forms.PasswordInput(render_value=True),
        regex=validators.password_validator(),
        error_messages={'invalid': validators.password_validator_msg()})

    def __init__(self, request, *args, **kwargs):
        super(CreatePowerDeviceForm, self).__init__(request, *args, **kwargs)

        self.fields['device_name'].choices = [
            (device_name, device_name)
            for device_name in TNXPowerDevicesClient.types_list(request)]

        self.fields['protocol_type'].choices = [
            (protocol_type, protocol_type)
            for protocol_type in TNXPowerDevicesClient.protocols_list(request)]

    def handle(self, request, data):
        try:
            device = TNXPowerDevicesClient.create(request, **data)
            if device:
                msg = _('Device successfully created.')
                messages.success(request, msg)
                return True
        except UrlNotAvailableError:
            raise
        except Exception as err:
            msg = _('Device could not be added! {0}.').format(err)
            redirect = reverse(POWER_DEVICES_INDEX)
            exceptions.handle(request, msg, redirect=redirect)


class DeletePowerDeviceForm(forms.SelfHandlingForm):
    """Power device delete form"""

    def handle(self, request, data):
        power_devices = self.initial['power_devices']

        success, failed = [], []
        for power_device in power_devices:
            try:
                TNXPowerDevicesClient.power_device_delete(
                    request, power_device.id)
            except UrlNotAvailableError:
                raise
            except Exception as err:
                logger.debug(unicode(err))
                failed.append(power_device.name)
                continue
            success.append(power_device.name)

        if failed:
            msg = _('Failed to delete power devices:')
            messages.error(
                request, '%s %s' % (unicode(msg), ', '.join(failed)))
        if success:
            msg = _('Successfully deleted power devices:')
            messages.success(
                request, '%s %s' % (unicode(msg), ', '.join(success)))

        return True
