# coding: utf-8

import mock

from django.core.urlresolvers import reverse

from horizon.exceptions import Http302

from tionix_tests.helpers import get_fake_request

from tionix_dashboard.clients import TNXPowerDevicesClient
from tionix_dashboard.dashboards.tionix.power_devices import (
    actions, forms, tables, views
)
from tionix_dashboard.tests.fake_cloud_state import POWER_DEVICE_COUNT
from tionix_dashboard.tests.helpers import (
    FakeUser, BaseAdminClientTestCase
)

POWER_DEVICES_INDEX_URL = reverse(
    'horizon:tionix:power_devices:index')


# pylint: disable=no-member
class TionixPowerDeviceActionsTest(BaseAdminClientTestCase):

    def test_delete_power_device(self):
        power_device = self.power_devices.get(1)

        action = actions.DeletePowerDeviceAction()

        self.assertEqual(
            action.get_link_url(power_device),
            u'/tionix/power_devices/delete/?id=1'
        )

    def test_power_device_filter(self):
        power_devices = self.power_devices.list()

        action = actions.PowerDeviceFilterAction()

        filtered_data = action.filter(
            tables.PowerDevicesTable(self.request),
            power_devices,
            'Power device')

        self.assertEqual(
            sorted(filtered_data),
            sorted([power_devices[0], power_devices[1]])
        )

    def test_tnx_instances_update_row(self):
        power_device = self.power_devices.get(1)

        action = actions.TnxInstancesUpdateRow(
            table=tables.PowerDevicesTable(self.request)
        )

        data = action.get_data(self.request, power_device.id)

        self.assertEqual(data.id, power_device.id)

    @mock.patch.object(TNXPowerDevicesClient,
                       'get',
                       mock.Mock(side_effect=Exception))
    def test_tnx_instances_update_row_error(self):
        power_device = self.power_devices.get(1)

        request = get_fake_request(
            user=FakeUser(),
            is_ajax=True,
            horizon={
                'async_messages': []
            }
        )

        action = actions.TnxInstancesUpdateRow(
            table=tables.PowerDevicesTable(request)
        )

        with self.assertRaises(Http302):
            action.get_data(request, power_device.id)

    def test_device_title_update_cell(self):
        power_device = self.power_devices.get(1)

        action = actions.DeviceTitleUpdateCell()

        data = action.update_cell(
            self.request,
            power_device,
            power_device.id,
            'name',
            'new name'
        )

        self.assertIsNone(data)


class TionixPowerDeviceFormsTest(BaseAdminClientTestCase):

    def test_create_power_device_form(self):

        form = forms.CreatePowerDeviceForm(self.request)

        self.assertEqual(
            len(form.fields['device_name'].choices),
            2
        )

        self.assertEqual(
            len(form.fields['protocol_type'].choices),
            3
        )

        self.assertListEqual(
            sorted(form.fields.keys()),
            sorted(['device_name', 'protocol_type', 'title', 'host',
                    'port_number', 'login', 'password'])
        )

    @mock.patch.object(TNXPowerDevicesClient,
                       'create',
                       mock.Mock(return_value=True))
    def test_test_create_power_device_form_handle(self):

        request = get_fake_request(
            user=FakeUser(),
            is_ajax=True,
            horizon={
                'async_messages': []
            }
        )

        form = forms.CreatePowerDeviceForm(request=request)

        self.assertTrue(form.handle(request, {}))

    @mock.patch.object(TNXPowerDevicesClient,
                       'create',
                       mock.Mock(side_effect=Exception))
    def test_test_create_power_device_form_handle_error(self):

        request = get_fake_request(
            user=FakeUser(),
            is_ajax=True,
            horizon={
                'async_messages': []
            }
        )

        form = forms.CreatePowerDeviceForm(request=request)

        with self.assertRaises(Http302):
            form.handle(request, {})

    @mock.patch.object(TNXPowerDevicesClient,
                       'power_device_delete',
                       mock.Mock(return_value=True))
    def test_delete_power_device_form(self):
        power_devices = self.power_devices.list()

        request = get_fake_request(
            user=FakeUser(),
            is_ajax=True,
            horizon={
                'async_messages': []
            }
        )

        form = forms.DeletePowerDeviceForm(request)
        form.initial = {
            'power_devices': power_devices
        }

        result = form.handle(request, None)

        self.assertTrue(result)

        self.assertEqual(request.horizon['async_messages'][0][0], 'success')

        self.assertEqual(
            request.horizon['async_messages'][0][1],
            u'Successfully deleted power devices: {0}'.format(
                u', '.join([i.name for i in power_devices]))
        )

    @mock.patch.object(TNXPowerDevicesClient,
                       'power_device_delete',
                       mock.Mock(side_effect=Exception))
    def test_delete_power_device_form_fail(self):
        power_devices = self.power_devices.list()

        request = get_fake_request(
            user=FakeUser(),
            is_ajax=True,
            horizon={
                'async_messages': []
            }
        )

        form = forms.DeletePowerDeviceForm(request)
        form.initial = {
            'power_devices': power_devices
        }

        result = form.handle(request, None)

        self.assertTrue(result)

        self.assertEqual(request.horizon['async_messages'][0][0], 'error')

        self.assertEqual(
            request.horizon['async_messages'][0][1],
            u'Failed to delete power devices: {0}'.format(
                ', '.join([i.name for i in power_devices])
            )
        )


class TionixPowerDeviceTablesTest(BaseAdminClientTestCase):

    def test_power_devices_table(self):

        table = tables.PowerDevicesTable(self.request)

        self.assertListEqual(
            sorted(table.columns.keys()),
            sorted(['multi_select', 'id', 'title', 'device_name',
                    'protocol_type', 'host', 'port', 'actions'])
        )

    def test_power_devices_table_actions(self):
        power_devices = self.power_devices.list()

        table = tables.PowerDevicesTable(self.request, data=power_devices)

        self.assertListEqual(
            sorted([action.name for action in table.get_table_actions()]),
            sorted(['filter', 'create', 'multi_device_delete'])
        )

    def test_power_devices_table_row_actions(self):
        table = tables.PowerDevicesTable(self.request)

        power_device = self.power_devices.list()[0]

        self.assertListEqual(
            sorted([action.name
                    for action in table.get_row_actions(power_device)]),
            sorted(['delete_power_device', 'clone'])
        )


class TionixPowerDeviceViewsTest(BaseAdminClientTestCase):

    def test_power_devices_index_view(self):

        view = views.PowerDevicesIndexView()
        view.request = self.request

        self.assertEqual(
            len(view.get_data()),
            POWER_DEVICE_COUNT
        )

    def test_delete_power_device_view_get_objects(self):
        power_device1 = self.power_devices.get(1)
        power_device2 = self.power_devices.get(2)

        view = views.DeletePowerDeviceView()
        view.request = get_fake_request(
            user=FakeUser(),
            get_params={
                'id': [power_device1.id, power_device2.id]
            }
        )

        self.assertListEqual(
            sorted([item.id for item in view.get_objects()]),
            sorted([power_device1.id, power_device2.id])
        )

        self.assertEqual(view.modal_header, u'Confirm delete power devices')

    def test_delete_power_device_view_context_data(self):
        power_device1 = self.power_devices.get(1)
        power_device2 = self.power_devices.get(2)

        view = views.DeletePowerDeviceView()
        view.request = get_fake_request(
            user=FakeUser(),
            get_params={
                'id': [power_device1.id, power_device2.id]
            },
            META={}
        )

        self.assertListEqual(
            sorted(view.get_context_data()['id']),
            sorted([power_device1.id, power_device2.id])
        )

    @mock.patch.object(TNXPowerDevicesClient,
                       'power_device_assigned_nodes')
    def test_delete_power_device_view_initial(self,
                                              power_device_assigned_nodes):
        power_device = self.power_devices.get(1)
        hypervisor = self.tnx_hypervisors.get(2).__dict__

        power_device_assigned_nodes.return_value = [hypervisor]

        view = views.DeletePowerDeviceView()
        view.request = get_fake_request(
            user=FakeUser(),
            get_params={
                'id': [power_device.id]
            },
            META={}
        )

        initial = view.get_initial()

        self.assertListEqual(
            sorted(initial.keys()),
            sorted(['hypervisors', 'power_devices', 'power_device_names'])
        )

        self.assertEqual(initial['hypervisors'][0]['tnx_power_state'], u'On')

    def test_panel_availability(self):

        response = self.client.get(POWER_DEVICES_INDEX_URL)

        self.assertEqual(response.status_code, 200)

    def test_panel_availability_for_no_admin(self):

        self.setActiveUser(
            id=self.user.id,
            token=self.token,
            username=self.user.name,
            domain_id=self.domain.id,
            user_domain_name=self.domain.name,
            tenant_id=self.tenant.id,
            service_catalog=self.service_catalog,
            authorized_tenants=self.context['authorized_tenants'],
            roles=[]
        )

        response = self.client.get(POWER_DEVICES_INDEX_URL)

        self.assertEqual(response.status_code, 302)
