# coding: utf-8

import logging

from collections import defaultdict

from django.core.urlresolvers import reverse_lazy
from django.utils.translation import ugettext_lazy as _, ungettext_lazy

from horizon import exceptions
from horizon import forms
from horizon import messages
from horizon import tables
from horizon.utils import memoized

from tionix_dashboard.clients import TNXPowerDevicesClient
from tionix_dashboard.clients.exceptions import UrlNotAvailableError
from tionix_dashboard.dashboards.admin.hypervisors.tables import (
    POWER_STATE_DISPLAY_CHOICES)
from tionix_dashboard.dashboards.tionix.power_devices import (
    forms as project_forms, tables as project_tables)
from tionix_dashboard.dashboards.tionix.power_devices.forms import (
    DeletePowerDeviceForm)
from tionix_dashboard.url_resolve_map import POWER_DEVICES_INDEX

logger = logging.getLogger(__name__)


class PowerDevicesIndexView(tables.DataTableView):
    """Main page view."""
    table_class = project_tables.PowerDevicesTable
    template_name = 'power_devices/index.html'
    page_title = _('Power devices')

    def get_data(self):
        request = self.request
        power_devices = []
        device_types = []
        protocols = []

        try:
            power_devices = TNXPowerDevicesClient.list(request)
            device_types = TNXPowerDevicesClient.types_list(request)
            protocols = TNXPowerDevicesClient.protocols_list(request)
        except UrlNotAvailableError:
            raise
        except Exception as err:
            logger.error(unicode(err))
            exceptions.handle(request,
                              _('Unable to retrieve power devices list.'))

        invalid_devices = defaultdict(list)
        for device in power_devices:
            if device.device_name not in device_types:
                invalid_devices[device.title].append(
                    unicode(_("Device type")))
            if device.protocol_type not in protocols:
                invalid_devices[device.title].append(
                    unicode(_("Protocol type")))

        for device, fields in invalid_devices.items():
            messages.error(
                request,
                _('Following fields of the device "{0}"'
                  ' are invalid: {1}').format(device, ", ".join(fields)))

        return power_devices


class CreatePowerDeviceView(forms.ModalFormView):
    """Create power device view."""
    form_class = project_forms.CreatePowerDeviceForm
    template_name = 'power_devices/create.html'
    modal_header = _('Create power device')
    modal_id = 'create_power_device'
    submit_label = _('Create device')
    context_object_name = 'power_devices'
    success_url = reverse_lazy(POWER_DEVICES_INDEX)


class DeletePowerDeviceView(forms.ModalFormView):
    """Delete power device view."""
    form_class = DeletePowerDeviceForm
    template_name = 'power_devices/delete.html'
    modal_id = "power_device_delete"
    submit_label = _('Delete')
    context_object_name = 'power_device'
    success_url = reverse_lazy(POWER_DEVICES_INDEX)
    hidden_title = False
    failure_msg = _('"Delete" is not available for power devices:')

    @staticmethod
    def get_modal_header(count):
        return ungettext_lazy(
            u'Confirm delete power device',
            u'Confirm delete power devices',
            count
        )

    @memoized.memoized_method
    def get_objects(self, *args, **kwargs):
        devices, failed = [], []
        for power_device_id in self.request.GET.getlist('id', []):
            try:
                obj = TNXPowerDevicesClient.get(
                    self.request, power_device_id)
            except UrlNotAvailableError:
                raise
            except Exception as err:
                logger.error(err.message)
                failed.append(power_device_id)
                continue
            devices.append(obj)

        if failed:
            messages.error(self.request, u'{0}: {1}'.format(
                self.failure_msg, u','.join(failed)))

        self.modal_header = self.get_modal_header(len(devices))

        return devices

    def get_context_data(self, **kwargs):
        context = super(DeletePowerDeviceView, self).get_context_data(
            **kwargs)
        context["id"] = self.request.GET.getlist('id')
        return context

    def get_initial(self):
        devices = self.get_objects()

        hypervisors = []

        power_state_dict = dict(POWER_STATE_DISPLAY_CHOICES)

        for device in devices:
            try:
                hyps = TNXPowerDevicesClient.power_device_assigned_nodes(
                    self.request, device.id)
            except UrlNotAvailableError:
                raise
            except Exception as err:
                logger.error(unicode(err))
            else:
                hypervisors.extend(hyps)

        for h in hypervisors:
            h.update(
                tnx_power_state=power_state_dict.get(
                    unicode(h['tnx_power_state'])))

        return {
            'power_device_names': [u'"{0}"'.format(d.name) for d in devices],
            'power_devices': devices,
            'hypervisors': hypervisors
        }


class ClonePowerDeviceView(CreatePowerDeviceView):
    """Clone power device view."""
    modal_header = _('Clone power device')
    modal_id = 'clone_power_device'
    submit_label = _('Clone device')

    def get_initial(self):
        initial = super(ClonePowerDeviceView, self).get_initial()

        try:
            power_device_dict = TNXPowerDevicesClient.get(
                self.request, self.kwargs['power_device_id']
            ).to_dict()
        except Exception as err:
            logger.error(unicode(err))
            exceptions.handle(self.request, _('Unable to get power device.'))
        else:
            power_device_dict.pop('id')
            power_device_dict.pop('host')
            power_device_dict['port_number'] = power_device_dict['port']
            initial.update(power_device_dict)

        return initial
