# coding: utf-8

import datetime
import hashlib
import pytz

from django.utils.translation import ugettext as _, ungettext_lazy

from tionix_dashboard.clients.servers import TnxServer
from tionix_dashboard.dashboards.tionix.scheduler.enums import (
    RepeatingTypeEnum, WeekDaysEnum)
from tionix_dashboard.helpers.vm_constants import ERROR
from tionix_dashboard.dashboards.tionix.scheduler import api
from tionix_dashboard.templatetags.tionix_tags import format_date


def is_server_schedulable(instance):
    if hasattr(instance, 'status'):
        return instance.status.lower() not in (ERROR,)


def get_cache_key(request):
    return hashlib.md5(request.session.session_key).hexdigest()


class SchedulingPreChecker(object):

    schedulable_servers_map = {
        TnxServer: is_server_schedulable,
    }

    actions_map = {
    }

    @classmethod
    def is_schedulable(cls, obj):

        object_type = type(obj)
        if object_type in cls.schedulable_servers_map:
            return cls.schedulable_servers_map[object_type](obj)

        return True

    @classmethod
    def is_actions_not_empty(cls, request, type_name, group_name):
        key = type_name + '_' + group_name
        actions = cls.actions_map.get(key)

        if actions is None:
            actions = api.get_available_actions(
                request=request,
                type_name=type_name,
                group_name=group_name,
                cache_key=request.session.get('user_action_uuid',
                                              get_cache_key(request)),
            )
            cls.actions_map[key] = actions

        return bool(actions)


def transform_date(field):
    def transform(obj):
        date_field = getattr(obj, field)
        tz = None
        tz_str = getattr(obj, 'timezone', '')
        if tz_str:
            tz = pytz.timezone(tz_str)
        return format_date(date_string=date_field, tz=tz)
    return transform


def get_utc_date(date, time=None):
    try:
        date = datetime.datetime.strptime(date, '%Y-%m-%d').date()
    except (ValueError, TypeError):
        date = None
    else:
        if time is not None:
            try:
                time = datetime.datetime.strptime(time, '%H:%M:%S')
            except (ValueError, TypeError):
                pass
            else:
                time = time.time()
                date = datetime.datetime.combine(date, time)

        # pylint: disable=no-value-for-parameter
        date = pytz.UTC.localize(date)

    return date


def localize_datetime(tz, date, time=None, format="medium"):
    utc_date = get_utc_date(date, time)
    date_str = utc_date.strftime('%Y-%m-%d %H:%M:%S')
    return format_date(date_string=date_str, tz=tz, format=format)


def get_task_summary(task, tz):
    summary = None
    if task.get('repeat'):
        periodic_task = task.get('periodic_task', {})
        try:
            fields = api.parse_repeating_task(task, periodic_task)
        except TypeError:
            pass
        else:
            repeat = fields.get('repeat')
            interval = fields.get('interval')
            interval = (
                int(interval)
                if isinstance(interval, basestring) and interval.isdigit()
                else 1
            )
            if repeat == RepeatingTypeEnum.DAYS:
                summary = (_("Each day") if interval == 1
                           else ungettext_lazy("Each {0} day",
                                               "Each {0} days",
                                               interval).format(interval))
            elif repeat == RepeatingTypeEnum.MINUTES:
                summary = (_("Each minute") if interval == 1
                           else ungettext_lazy("Each {0} minute",
                                               "Each {0} minutes",
                                               interval).fromat(interval))
            elif repeat == RepeatingTypeEnum.HOURS:
                summary = (_("Each hour") if interval == 1
                           else ungettext_lazy("Each {0} hour",
                                               "Each {0} hours",
                                               interval).fromat(interval))
            elif repeat == RepeatingTypeEnum.WORKING_DAYS:
                summary = _("Each week - on weekdays")
            elif repeat == RepeatingTypeEnum.DAYS_OF_THE_WEEK:
                summary = _("Each week")
                days = [unicode(WeekDaysEnum.shot_name_values.get(k))
                        for k, v in WeekDaysEnum.form_values.iteritems()
                        if v in fields]
                if days:
                    summary += u' - %s' % u', '.join(days)
            elif repeat == RepeatingTypeEnum.YEAR:
                begin_date = task.get('begin_date')
                if begin_date:
                    begin_date = localize_datetime(tz,
                                                   task.get('begin_date'),
                                                   format='d MMMM')
                summary = (_("Each year - {0}").format(begin_date)
                           if interval == 1
                           else ungettext_lazy("Each {0} year - {1}",
                                               "Each {0} years - {1}",
                                               interval).format(interval,
                                                                begin_date))
            end_date = fields.get('end_date')
            end_max_number = fields.get('end_max_number')
            if end_date:
                summary += _(", till {0}").format(
                    format_date(end_date.strftime('%Y-%m-%d %H:%M:%S'), tz=tz))
            elif end_max_number:
                summary += ungettext_lazy(
                    ", {0} time", ", {0} times", end_max_number
                ).format(end_max_number)
    else:
        summary = _("1 time")

    return summary
