# coding: utf-8

import datetime

import mock
from django import http

from django.http import QueryDict
from django.test import override_settings

from openstack_auth import policy as policy_backend

from openstack_dashboard.api import keystone

from tionix_tests.helpers import get_fake_request

from tionix_scheduler_client.client import Client as SchedulerClient
from tionix_scheduler_client.enums import ActionFieldsEnum

from tionix_dashboard.dashboards.helpers import Empty
from tionix_dashboard.dashboards.identity.projects.enum import TenantTypeEnum
from tionix_dashboard.dashboards.tionix.scheduler import (
    actions, api, forms, tables
)
from tionix_dashboard.dashboards.tionix.scheduler.api import (
    get_cron_schedule, get_cron_str)
from tionix_dashboard.dashboards.tionix.scheduler.enums import (
    RepeatingTypeEnum, TaskTypeEnum)
from tionix_dashboard.dashboards.tionix.scheduler.helpers import (
    SchedulingPreChecker
)
from tionix_dashboard.dashboards.tionix.scheduler import workflows
from tionix_dashboard.tests.helpers import (
    BaseClientTestCase, BaseAdminClientTestCase, FakeUser)


SCHEDULER_INDEX_URL = 'horizon:tionix:scheduler:index'


class BaseTionixSchedulerTestCaseMixin(object):
    """Mixin for scheduler tests"""

    scheduler = None

    def setUp(self):
        super(BaseTionixSchedulerTestCaseMixin, self).setUp()

        self.scheduler = SchedulerClient()


class BaseTionixSchedulerTestCase(BaseTionixSchedulerTestCaseMixin,
                                  BaseClientTestCase):
    """Base class for testing scheduler for simple user"""
    pass


class BaseTionixSchedulerAdminTestCase(BaseTionixSchedulerTestCaseMixin,
                                       BaseAdminClientTestCase):
    """Base class for testing scheduler for admin user"""
    pass


class TionixSchedulerActionsTest(BaseTionixSchedulerTestCase):

    def test_delete_allowed(self):

        action = actions.DeleteTaskAction()

        self.assertTrue(action.allowed(self.request, Empty(deleted=False)))

    def test_delete_not_allowed(self):

        action = actions.DeleteTaskAction()

        self.assertFalse(action.allowed(self.request, Empty(deleted=True)))

    def test_delete_task_action(self):

        action = actions.DeleteTaskAction()

        self.assertTrue(action.delete(request=self.request, obj_id=None))

    def test_get_display_key_by_value(self):

        display_tuple = (
            ('test_1', u'Test 1'),
            ('test_2', u'Test 2'),
            ('test_3', u'Test 3')
        )

        display_key_by_value = actions.get_display_key_by_value(display_tuple)

        self.assertEqual(
            display_key_by_value(display_tuple[0][1]),
            display_tuple[0][0]
        )

        self.assertEqual(
            display_key_by_value('test_4'),
            'test_4'
        )

        self.assertEqual(
            display_key_by_value(display_tuple[1][1].upper()),
            display_tuple[1][0]
        )

    def test_filter_action_attrs(self):

        action = actions.FilterAction(
            table=tables.BaseTasksTable(request=self.request)
        )

        self.assertDictEqual(
            action.get_default_attrs(),
            {'id': 'scheduler__action_scheduler_filter',
             'value': 'scheduler__scheduler_filter',
             'name': 'filter'}
        )

    def test_filter_action_param_name(self):

        action = actions.FilterAction(
            table=tables.BaseTasksTable(self.request)
        )

        self.assertEqual(
            action.get_param_name(),
            'scheduler__scheduler_filter__q'
        )

    def test_filter_action_param_name_obj_id(self):

        action = actions.FilterAction(
            table=tables.BaseTasksTable(self.request, obj_id='1')
        )

        self.assertEqual(
            action.get_param_name(),
            'scheduler__scheduler_filter__1'
        )

    # pylint: disable=no-member
    def test_scheduler_update_row(self):
        timetable, _status_code = self.scheduler.get_timetable(1)

        action = actions.SchedulerUpdateRow(
            table=tables.BaseTasksTable(self.request)
        )

        data = action.get_data(self.request, timetable.id)

        self.assertTrue(data.id, timetable.id)

    def test_hyp_scheduler_action(self):
        hypervisor = self.tnx_hypervisors.get(1)

        action = actions.HypervisorSchedulerAction()

        self.assertEqual(
            action.get_link_url(hypervisor),
            u'/tionix/scheduler/hypervisor/admin/'
            u'calendar?object_id={0}'.format(hypervisor.id)
        )

    def test_instance_scheduler_action(self):
        instance = self.tnx_servers.get(1)

        action = actions.InstanceSchedulerAction()

        self.assertEqual(
            action.get_link_url(instance),
            u'/tionix/scheduler/instance/admin/calendar?object_id={0}'.format(
                instance.id
            )
        )

    @mock.patch.object(SchedulingPreChecker, 'is_actions_not_empty',
                       mock.Mock(return_value=True))
    def test_instance_action_allowed(self):
        instance = self.tnx_servers.get(1)

        action = actions.InstanceSchedulerAction()

        self.assertTrue(action.allowed(self.request, instance))

    @mock.patch.object(SchedulingPreChecker,
                       'is_schedulable',
                       mock.Mock(return_value=False))
    def test_instance_action_not_allowed(self):
        instance = self.tnx_servers.get(3)

        action = actions.InstanceSchedulerAction()

        self.assertFalse(action.allowed(self.request, instance))

    # pylint: disable=no-member
    def test_volume_scheduler_action(self):
        volume = self.volumes.first()

        action = actions.VolumeSchedulerAction()

        self.assertEqual(
            action.get_link_url(volume),
            u'/tionix/scheduler/volume/admin/calendar?object_id={0}'.format(
                volume.id
            )
        )

    # pylint: disable=no-member
    def test_project_scheduler_action(self):
        tenant = self.tenants.list()[0]

        tenant.tnx_project_type_name = TenantTypeEnum.TYPICAL

        action = actions.ProjectSchedulerAction(datium=tenant)

        self.assertEqual(
            action.get_link_url(tenant),
            u'/tionix/scheduler/project/identity/'
            u'calendar?object_id={0}'.format(tenant.id)
        )

    # pylint: disable=no-member
    def test_vdi_project_scheduler_action(self):
        tenant = self.tenants.list()[0]

        tenant.tnx_project_type_name = TenantTypeEnum.VDI

        action = actions.ProjectSchedulerAction(datum=tenant)

        self.assertEqual(
            action.get_link_url(tenant),
            u'/tionix/scheduler/project/vdi/calendar?object_id={0}'.format(
                tenant.id
            )
        )

    def test_get_cron_schedule(self):
        begin_date = datetime.datetime.now()

        interval = 1
        expected_str = '{0} {1} * * *'.format(
            begin_date.minute, begin_date.hour)
        cron_dict = get_cron_schedule(
            begin_date=begin_date,
            task_type=TaskTypeEnum.REPEATING,
            repeat=RepeatingTypeEnum.DAYS,
            interval=interval,
            week_days=None)
        self.assertEqual(expected_str, get_cron_str(cron_dict))

        interval = 2
        expected_str = '{0} {1} */2 * *'.format(
            begin_date.minute, begin_date.hour)

        cron_dict = get_cron_schedule(
            begin_date=begin_date,
            task_type=TaskTypeEnum.REPEATING,
            repeat=RepeatingTypeEnum.DAYS,
            interval=interval,
            week_days=None)
        self.assertEqual(expected_str, get_cron_str(cron_dict))

        expected_str = '{0} {1} * * 1,3,7'.format(
            begin_date.minute, begin_date.hour)
        cron_dict = get_cron_schedule(
            begin_date=begin_date,
            task_type=TaskTypeEnum.REPEATING,
            repeat=RepeatingTypeEnum.DAYS_OF_THE_WEEK,
            week_days=['1', '3', '7'])
        self.assertEqual(expected_str, get_cron_str(cron_dict))

        expected_str = '{0} {1} * * 1-5'.format(
            begin_date.minute, begin_date.hour)
        cron_dict = get_cron_schedule(
            begin_date=begin_date,
            task_type=TaskTypeEnum.REPEATING,
            repeat=RepeatingTypeEnum.WORKING_DAYS)
        self.assertEqual(expected_str, get_cron_str(cron_dict))

        expected_str = '{0} {1} {2} {3} *'.format(
            begin_date.minute, begin_date.hour,
            begin_date.day, begin_date.month)
        cron_dict = get_cron_schedule(
            begin_date=begin_date,
            task_type=TaskTypeEnum.REPEATING,
            repeat=RepeatingTypeEnum.YEAR)
        self.assertEqual(expected_str, get_cron_str(cron_dict))

        expected_str = '{0} {1} {2} {3} *'.format(
            begin_date.minute, begin_date.hour,
            begin_date.day, begin_date.month)
        cron_dict = get_cron_schedule(
            begin_date=begin_date,
            task_type=TaskTypeEnum.SINGLE)
        self.assertEqual(expected_str, get_cron_str(cron_dict))


@mock.patch('tionix_dashboard.dashboards.utils.is_domain_supported',
            mock.Mock(return_value=True))
@override_settings(POLICY_CHECK_FUNCTION=policy_backend.check)
class TionixSchedulerTablesAdminTest(BaseTionixSchedulerAdminTestCase):

    def test_scheduler_columns(self):

        table = tables.IndexTasksTable(self.request)

        self.assertListEqual(
            sorted(table.columns.keys()),
            sorted(['multi_select', 'id', 'name', 'action', 'type',
                    'last_run_result', 'subject_type', 'subject',
                    'subject_tenant', 'deleted', 'actions'])
        )

    # pylint: disable=no-member
    def test_scheduler_actions(self):
        tasks = self.scheduler.get_timetable_list()

        table = tables.IndexTasksTable(self.request, data=tasks)
        self.assertListEqual(
            sorted([action.name for action in table.get_table_actions()]),
            sorted(['scheduler_filter', 'delete'])
        )

    def test_scheduler_row_actions(self):
        table = tables.IndexTasksTable(self.request)

        task, _status_code = self.scheduler.get_timetable(1)
        self.assertListEqual(
            sorted([action.name for action in table.get_row_actions(task)]),
            sorted(['delete', 'repeat_schedule', 'show'])
        )

        task, _status_code = self.scheduler.get_timetable(2)
        self.assertListEqual(
            sorted([action.name for action in table.get_row_actions(task)]),
            sorted(['repeat_schedule', 'show'])
        )


@mock.patch('tionix_dashboard.dashboards.utils.is_domain_supported',
            mock.Mock(return_value=True))
@override_settings(POLICY_CHECK_FUNCTION=policy_backend.check)
class TionixSchedulerTablesTest(BaseTionixSchedulerTestCase):

    def test_scheduler_columns(self):

        table = tables.IndexTasksTable(self.request)

        self.assertListEqual(
            sorted(table.columns.keys()),
            sorted(['multi_select', 'id', 'name', 'action', 'type',
                    'last_run_result', 'subject_type', 'subject', 'deleted',
                    'actions'])
        )

    def test_scheduler_row_actions(self):
        table = tables.IndexTasksTable(self.request)

        task, _status_code = self.scheduler.get_timetable(1)
        self.assertListEqual(
            sorted([action.name for action in table.get_row_actions(task)]),
            sorted(['delete', 'repeat_schedule'])
        )


@mock.patch.object(workflows.SchedulerWorkflow,
                   'get_object_name',
                   mock.Mock(return_value=u'Object with % in name'))
class TionixSchedulerWorkflowTest(BaseTionixSchedulerTestCase):

    context = None

    def setUp(self):
        super(TionixSchedulerWorkflowTest, self).setUp()
        self.request.LANGUAGE_CODE = 'en'
        test_task_name = u'1'
        self.context = {'repeat_fri': False, 'unique_name': test_task_name,
                        'task_type': u'single', 'repeat_tue': False,
                        'description': u'test', 'repeat_sat': False,
                        'user_password': u'123456', 'interval': 1,
                        'repeat_mon': False, 'repeat_sun': False,
                        'timezone': u'UTC', 'repeat': u'minutes',
                        'repeat_wed': False, 'end_date': None,
                        'begin_time': datetime.time.max,
                        'repeat_thu': False,
                        'begin_date': datetime.date.max,
                        'end': u'never', 'user_name': u'admin',
                        'end_max_number': None, 'tenant': u'test_tenant_id'}

        workflows.messages = mock.Mock()

    def test_format_message_success(self):

        test_object_id = u'1'
        context_seed_dict = {u'object_type': u'hypervisor',
                             u'tenant': u'test_tenant_id',
                             u'object_id': test_object_id,
                             u'object_group': u'admin'}

        context_seed = QueryDict('', mutable=True)
        context_seed.update(context_seed_dict)

        workflow = workflows.SchedulerWorkflow(self.request,
                                               context_seed=context_seed)

        result = workflow.handle(self.request, self.context)
        self.assertTrue(result)
        workflow.format_status_message(workflow.success_message)

    def test_format_message_failure(self):

        test_object_id = u'unexsists_id'
        context_seed_dict = {u'object_type': u'hypervisor',
                             u'tenant': u'test_tenant_id',
                             u'object_id': test_object_id,
                             u'object_group': u'admin'}

        context_seed = QueryDict('', mutable=True)
        context_seed.update(context_seed_dict)

        workflow = workflows.SchedulerWorkflow(self.request,
                                               context_seed=context_seed)

        result = workflow.handle(self.request, self.context)
        self.assertFalse(result)
        workflow.format_status_message(workflow.failure_message)

    def test_format_message_ignore(self):

        test_object_id = u'3'
        context_seed_dict = {u'object_type': u'instance',
                             u'tenant': u'test_tenant_id',
                             u'object_id': test_object_id,
                             u'object_group': u'admin'}

        context_seed = QueryDict('', mutable=True)
        context_seed.update(context_seed_dict)

        workflow = workflows.SchedulerWorkflow(self.request,
                                               context_seed=context_seed)

        result = workflow.handle(self.request, self.context)
        self.assertTrue(result)
        # pylint: disable=no-member
        self.assertTrue(workflows.messages.info.called)

    def test_format_message_mixed(self):

        test_object_id = u'1'
        test_unexists_object_id = u'unexists_id'
        context_seed_dict = {u'object_type': u'hypervisor',
                             u'tenant': u'test_tenant_id',
                             u'object_id': test_object_id,
                             u'object_group': u'admin'}

        context_seed = QueryDict('', mutable=True)
        context_seed.update(context_seed_dict)

        workflow = workflows.SchedulerWorkflow(self.request,
                                               context_seed=context_seed)
        workflow.object_ids = [test_object_id, test_unexists_object_id]

        result = workflow.handle(self.request, self.context)
        self.assertFalse(result)
        workflow.format_status_message(workflow.failure_message)
        self.assertIn('%%', workflow.success_message)
        # pylint: disable=no-member
        self.assertTrue(workflows.messages.success.called)

    @mock.patch.object(keystone, 'tenant_list',
                       mock.Mock(return_value=[[], True]))
    @mock.patch.object(
        api, 'get_available_actions',
        mock.Mock(return_value=[(ActionFieldsEnum.LAUNCH_VDI_VM, 'name')]))
    def test_repeat_vdi_launch_schedule(self):

        context_seed_dict = {'unique_name': ActionFieldsEnum.LAUNCH_VDI_VM}
        context_seed = QueryDict('', mutable=True)
        context_seed.update(context_seed_dict)

        workflow = workflows.RepeatSchedulerWorkflow(self.request,
                                                     context_seed=context_seed)

        self.assertEqual(len(workflow.steps), 4)
        launch_vdi_step = workflow.steps[3]
        for field in launch_vdi_step.action.fields.values():
            self.assertNotIn('readonly', field.widget.attrs)


class TionixSchedulerFormsTest(BaseTionixSchedulerTestCase):

    objects_to_delete = None

    def setUp(self):
        super(TionixSchedulerFormsTest, self).setUp()

        self.objects_to_delete = [Empty(id=i, name='name %s' % i)
                                  for i in xrange(3)]

    @mock.patch.object(forms.DeleteObjectWithTasksForm, 'delete')
    def test_delete_objects_with_tasks(self, delete):
        objects = self.objects_to_delete

        request = get_fake_request(
            user=FakeUser(),
            is_ajax=True,
            session={},
            get_params={
                'id': [obj.id for obj in objects]
            },
            horizon={
                'async_messages': []
            }
        )

        form = forms.DeleteObjectWithTasksForm(request)
        form.initial = {
            'redirect_url': '',
            'object_names_dict': {obj.id: obj.name for obj in objects},
            'rows': []
        }

        form.handle(request, data={})

        self.assertTrue(delete.called)

    @mock.patch.object(forms.DeleteObjectWithTasksForm, 'delete', mock.Mock())
    @mock.patch('tionix_dashboard.dashboards.tionix.scheduler.forms.'
                'delete_scheduled_tasks_list')
    def test_delete_objects_with_tasks_has_tasks(self,
                                                 delete_scheduled_tasks_list):
        objects = self.objects_to_delete

        request = get_fake_request(
            user=FakeUser(token=Empty(id=None)),
            is_ajax=True,
            session={},
            get_params={
                'id': [obj.id for obj in objects]
            },
            horizon={
                'async_messages': []
            }
        )

        form = forms.DeleteObjectWithTasksForm(request)
        form.initial = {
            'redirect_url': '',
            'object_names_dict': {obj.id: obj.name for obj in objects},
            'rows': [None]
        }

        form.handle(request, data={})

        self.assertTrue(delete_scheduled_tasks_list.called)

    @mock.patch.object(forms.DeleteObjectWithTasksForm,
                       'delete',
                       mock.Mock(side_effect=Exception))
    def test_delete_objects_with_tasks_redirect(self):
        objects = self.objects_to_delete

        request = get_fake_request(
            user=FakeUser(token=Empty(id=None)),
            is_ajax=True,
            session={},
            get_params={
                'id': [obj.id for obj in objects]
            },
            horizon={
                'async_messages': []
            }
        )

        form = forms.DeleteObjectWithTasksForm(request)
        form.initial = {
            'redirect_url': '',
            'object_names_dict': {obj.id: obj.name for obj in objects},
            'rows': []
        }

        result = form.handle(request, data={})

        self.assertIsInstance(result, http.HttpResponseRedirect)

    def test_detail_task(self):

        form = forms.TaskDetailsForm(self.request)

        self.assertTrue(form.handle(self.request, data={}))
