# coding: utf-8

import logging

from django.utils.translation import ugettext_lazy as _

from horizon import exceptions
from horizon.utils.memoized import memoized

from openstack_dashboard import api, policy
from openstack_dashboard.api.glance import is_image_public
from openstack_dashboard.api.keystone import VERSIONS

from tionix_dashboard.clients import TNXTenantClient
from tionix_dashboard.clients.base import (
    neutronclient, novaclient, glanceclient)
from tionix_dashboard.clients.exceptions import UrlNotAvailableError
from tionix_dashboard.clients.servers import TnxServer
from tionix_dashboard.dashboards.tionix.vdi.constants import VDI_TYPE
from tionix_dashboard.dashboards.utils import update_instances_list

logger = logging.getLogger(__name__)


def get_tenants_kwargs(request):
    is_admin = api.keystone.is_domain_admin(request)
    kwargs = {
        'admin': is_admin,
    }
    if is_admin:
        domain = api.keystone.get_default_domain(request, get_name=False)
        kwargs['domain'] = domain.get('id')
    else:
        kwargs['user'] = request.user.id
    return kwargs


@memoized
def get_quota_by_tenant(request, tenant_id):
    """Retrieve tenant quota"""
    nova_client = novaclient(request, project_id=tenant_id)
    return nova_client.quotas.get(tenant_id)


@memoized
def get_servers_by_tenant(request, tenant_id):
    """Retrieve tenant servers"""
    nova_client = novaclient(request, project_id=tenant_id)
    return {server.id: server.tenant_id
            for server in nova_client.servers.list()}


@memoized
def get_networks_by_tenant(request, tenant_id):
    """Retrieve tenant networks"""
    neutron_client = neutronclient(request, project_id=tenant_id)
    networks = neutron_client.list_networks().get('networks')
    return {network['id']: network for network in networks}


@memoized
def get_favor_ids_by_tenant(request, tenant_id):
    """Retrieve flavors ids available for tenant"""
    nova_client = novaclient(request, project_id=tenant_id)
    return [flavor.id for flavor in nova_client.flavors.list()]


@memoized
def get_image_by_tenant(request, tenant_id, image_id):
    """Retrieve tenant image"""
    glance_client = glanceclient(request, project_id=tenant_id)
    return glance_client.images.get(image_id)


def validate_tenant(request, tenant=None, user_tenants=None):
    """Check tenant by tnx attributes, quota and available network"""
    def _validate_tenant_fields(_tenant):
        fields = ['tnx_operation_mode', 'tnx_flavor_id', 'tnx_network_id',
                  'tnx_image_id']
        for field in fields:
            if not getattr(_tenant, field, None):
                return False
        return True

    validated = False
    tenant_id = tenant.id

    if _validate_tenant_fields(tenant):
        if tenant_id in user_tenants:
            try:
                servers = get_servers_by_tenant(request, tenant_id)
                quota = get_quota_by_tenant(request, tenant_id)
                instances = quota.instances
                instances_count = sum(tenant_id == servers[server_id]
                                      for server_id in servers)
                # -1 means no quota
                validated = (instances == -1 or
                             instances_count < instances)
            except Exception as err:
                logger.error(err)
    if validated:
        try:
            image = get_image_by_tenant(request,
                                        tenant_id,
                                        tenant.tnx_image_id)
        except Exception as err:
            logger.error(unicode(err))
            validated = False
        else:
            validated = (image.status.lower() != "deleted" and
                         (is_image_public(image) or image.owner == tenant_id))

    if validated:
        try:
            flavors = get_favor_ids_by_tenant(request, tenant_id)
        except Exception as err:
            logger.error(unicode(err))
            validated = False
        else:
            validated = tenant.tnx_flavor_id in flavors

    if validated:
        networks = get_networks_by_tenant(request, tenant_id)
        network = networks.get(tenant.tnx_network_id)
        validated = network and (network['tenant_id'] == tenant_id or
                                 network['shared'])

    return validated


@memoized
def get_projects_for_instance_launch(request):
    """Retrieve available projects for vdi instance"""

    user = request.user
    projects = []
    try:
        if policy.check((('identity', 'identity:list_role_assignments'),),
                        request):
            assignments = api.keystone.role_assignments_list(
                request, user=user, include_subtree=False)
            user_assignments = [assignment.scope['project']['id']
                                for assignment in assignments
                                if 'project' in assignment.scope]
        else:
            user_assignments = [project.id for project in
                                user.authorized_tenants]
    except Exception as err:
        logger.error(err)
        msg = _('Unable to get user assignments.')
        exceptions.handle(request, msg)
    else:

        kwargs = {'filters': {'enabled': True}}
        kwargs.update(get_tenants_kwargs(request))
        try:
            projects = [
                (tenant.id, tenant.name)
                for tenant in TNXTenantClient.list_vdi(request, **kwargs)
                if validate_tenant(request,
                                   tenant=tenant,
                                   user_tenants=user_assignments)
            ]

        except Exception as err:
            logger.error(err)
            msg = _('Failed to retrieve list of projects.')
            exceptions.handle(request, msg)

    return projects


def get_project_users(request, project):
    """Retrieve project's users list"""

    def _filter(obj):
        return (hasattr(obj, 'user') and
                'project' in obj.scope and
                obj.scope['project']['id'] == project)

    if VERSIONS.active < 3:
        project_users = [user.id for user in
                         api.keystone.user_list(request, project=project)]
    else:
        project_role_assignments = api.keystone.role_assignments_list(
            request,
            project=project
        )
        project_users = list({role_assignment.user['id']
                              for role_assignment in project_role_assignments
                              if _filter(role_assignment)})
    return project_users


def get_user_vdi_projects(request):

    projects, kwargs = [], get_tenants_kwargs(request)

    try:
        projects = TNXTenantClient.list_vdi(request, **kwargs)
    except Exception as err:
        logger.error(unicode(err))

    return projects


def get_instances_list(request, project_id=None, search_opts=None):
    instances = []
    try:
        instances = novaclient(request, project_id=project_id).servers.list(
            search_opts=search_opts
        )
    except UrlNotAvailableError:
        raise
    except Exception as err:
        logger.error(unicode(err))
        exceptions.handle(request, _('Unable to retrieve instance list.'))

    return [TnxServer(instance, request) for instance in instances]


def get_vdi_instances(request, search_opts, tenants_load=True):
    """Retrieve vdi instances"""
    instances, tenants = [], []

    # Gather our tenants to correlate against IDs
    kwargs = get_tenants_kwargs(request)
    search_opts.update(**kwargs)

    if tenants_load:
        try:
            tenants, _has_more = api.keystone.tenant_list(request, **kwargs)
        except Exception as err:
            logger.error(unicode(err))
            msg = _('Unable to retrieve instance project information.')
            exceptions.handle(request, msg)

    domain_id = api.keystone.get_default_domain(request,
                                                get_name=False).get('id')
    search_opts.update(tnx_type=VDI_TYPE, domain=domain_id)
    if kwargs['admin']:
        search_opts.update(all_tenants=True)
        instances = get_instances_list(request, search_opts=search_opts)
    else:
        search_opts.update(user=request.user.id)
        for project in get_user_vdi_projects(request):
            instances += get_instances_list(request, project.id, search_opts)

    instances = update_instances_list(request, instances, tenants=tenants)
    return instances


@memoized
def get_user_roles_by_project(request, project_id):
    """
    Retrieve user's roles in project
    :param request: django request
    :param project_id: project ID
    :return:
    """
    roles = []
    try:
        roles = api.keystone.roles_for_user(request,
                                            request.user,
                                            project=project_id)
    except Exception as err:
        logger.error(unicode(err))
    return [role.name.lower() for role in roles]
