# coding: utf-8

import logging

from django.core.urlresolvers import reverse
from django.utils.http import urlencode
from django.utils.translation import ugettext_lazy as _

from novaclient import exceptions as nova_exceptions

from horizon import tables, exceptions

from openstack_dashboard import api
from openstack_dashboard.dashboards.project.instances import (
    tables as project_tables)
from openstack_dashboard.utils.identity import IdentityMixIn

from tionix_dashboard.clients.base import novaclient
from tionix_dashboard.clients.servers import TnxServer
from tionix_dashboard.clients.tenants import TNXTenantClient
from tionix_dashboard.dashboards.license import LicenseController
from tionix_dashboard.dashboards.project.project_instances import (
    actions as project_actions
)
from tionix_dashboard.dashboards.tionix.vdi.enums import OperationModeEnum
from tionix_dashboard.dashboards.tionix.vdi.helpers import (
    get_user_roles_by_project, get_user_vdi_projects)
from tionix_dashboard.dashboards.utils import update_instances_list
from tionix_dashboard.helpers.actions import TnxFilterAction, MultiLinkAction
from tionix_dashboard.helpers.vm_constants import (
    ERROR
)


logger = logging.getLogger(__name__)


def get_operation_mode_by_name(value):
    """Return list of operation mode"""
    keys = [k for k, v in OperationModeEnum.values.items()
            if value.lower() in v.lower()]
    return ','.join(keys)


def get_instance_flavor_name(instance):
    """Return instance flavor name"""

    full_flavor, name = getattr(instance, 'full_flavor', None), ''
    if full_flavor:
        name = getattr(full_flavor, 'name', '')
    return name


class VDIInstanceFilterAction(TnxFilterAction, tables.FilterAction):
    """
    Filter VDI instances filter
    """
    name = "filter_vdi_instances"
    filter_type = "server"

    filter_choices = (
        ('name', _("Name"), True),
        ('tenant', _("Project"), False),
        ('ip', _("IP Address"), False),
        ('size', _("Size"), False),
        ('status', _("Status"), False),
        ('OS-EXT-AZ:availability_zone', _("Zone"), False),
        ('power_state', _("Power state"), True),
        ('tnx_operation_mode', _("Operation mode"), True),
        ('tnx_last_session', _("Last session"), True),
        ('id', _("Instance ID"), False),
    )

    filter_string_modifiers = {
        'power_state': project_actions.get_power_states_by_name,
        'tnx_operation_mode': get_operation_mode_by_name
    }

    field_get_data = {
        'size': get_instance_flavor_name
    }


class LaunchVDIInstanceAction(tables.LinkAction):
    """
    Action to launch new instance
    """
    name = 'launch'
    verbose_name = _('Launch VDI Instance')
    classes = ('ajax-modal', )
    url = 'horizon:tionix:vdi:instances:launch'
    icon = "cloud-upload"
    policy_rules = (("compute", "compute:create"),)
    ajax = True

    def __init__(self, attrs=None, **kwargs):
        kwargs['preempt'] = True
        super(LaunchVDIInstanceAction, self).__init__(attrs, **kwargs)


class VDIInstanceManageUsers(tables.LinkAction, IdentityMixIn):
    """Action to manage instance users"""
    url = 'horizon:tionix:vdi:instances:users'

    name = 'vdi_assign_to_users'
    classes = ('ajax-modal', )
    verbose_name = _('Assign to users')
    permissions = ('openstack.roles.admin',)
    policy_rules = (("identity", "admin_and_matching_domain_id"),)

    def allowed(self, request, datum):
        allowed = datum and datum.status.lower() not in (ERROR,)
        if allowed:
            user_roles = set(get_user_roles_by_project(request,
                                                       datum.tenant_id))
            allowed = user_roles & set(self.get_admin_roles())
        return allowed


class RebuildInstance(project_tables.RebuildInstance):
    """Action to rebuild instance"""
    url = "horizon:tionix:vdi:instances:rebuild"

    def get_link_url(self, datum=None):
        instance_id = datum.id if datum else None
        params = urlencode({'id': instance_id})
        return '?'.join([reverse(self.url), params])


class MultiRebuildInstance(MultiLinkAction):
    """ Action reset vm's state."""
    name = 'multi_rebuild'
    verbose_name = _('Rebuild Instances')
    classes = ('btn-rebuild', )
    url = "horizon:tionix:vdi:instances:rebuild"
    policy_rules = (("compute", "compute:rebuild"),)


class UpdateRow(tables.Row):
    ajax = True

    def get_data(self, request, obj_id):
        instance = None
        if api.keystone.is_domain_admin(request):
            project_ids = [request.user.project_id]
        else:
            project_ids = [project.id
                           for project in get_user_vdi_projects(request)]
        for project_id in project_ids:
            try:
                instance = novaclient(
                    request,
                    project_id=project_id
                ).servers.get(obj_id)
            except nova_exceptions.NotFound:
                continue
            except Exception as err:
                logger.error(unicode(err))
                exceptions.handle(request,
                                  _('Unable to get instance'),
                                  ignore=True)
                break
        else:
            tenants = None
            try:
                tenants = [TNXTenantClient.get(request, instance.tenant_id)]
            except Exception as err:
                logger.error(unicode(err))
                exceptions.handle(request,
                                  _('Unable to retrieve tenant information '
                                    'for instance "%s".') % obj_id,
                                  ignore=True)
            instance = update_instances_list(request,
                                             [TnxServer(instance, request)],
                                             tenants=tenants)
        return instance.pop()


class ResetStateAction(project_actions.ResetStateAction):
    """ Action reset vm's state."""
    url = 'horizon:tionix:vdi:instances:reset_state'


class MultiResetStateAction(project_actions.MultiResetStateAction):
    """ Action reset vm's state."""
    url = 'horizon:tionix:vdi:instances:reset_state'


class VDIStopInstance(project_tables.StopInstance):

    def allowed(self, request, instance):
        allowed = super(VDIStopInstance, self).allowed(request, instance)
        if instance:
            allowed &= instance.status.lower() not in (ERROR, )
        return allowed


class SessionFilterAction(TnxFilterAction):
    """Session filter"""
    filter_type = "server"

    filter_choices = [('username', _("User Name"), False),
                      ('connection_type', _("Connection"), False),
                      ('client_info', _("Client"), False),
                      ('client_os', _("OS"), False),
                      ('client_ip', _("IP"), False)]


class CloneVDILink(tables.LinkAction):
    """Clone instance action"""
    name = "clone"
    verbose_name = verbose_name_base = _("Clone Instance")
    url = "horizon:tionix:vdi:instances:clone"
    classes = ("ajax-modal",)
    policy_rules = (("compute", "compute:create"),)
    ajax = True

    def allowed(self, request, datum):
        allowed = super(CloneVDILink, self).allowed(request, datum)
        return (allowed and
                LicenseController.is_valid(request, LicenseController.VDI) and
                datum.status.lower() != ERROR)

    def get_link_url(self, datum=None):
        instance_id, project_id = (
            (datum.id, datum.tenant_id) if datum
            else (None, None)
        )
        return reverse(self.url, args=(project_id, instance_id))


class MultiMembersAndGroupsManageAction(MultiLinkAction):
    """Multi members and groups manage"""
    name = "multi_members_groups_update"
    verbose_name = _("Manage Members and Groups")
    url = "horizon:tionix:vdi:instances:members_groups_manage"
    permissions = ('openstack.roles.admin',)
    policy_rules = (("identity", "admin_and_matching_domain_id"),)
