# coding: utf-8
from django.core.urlresolvers import reverse
from django.template.defaultfilters import title
from django.utils.translation import ugettext_lazy as _

from horizon import tables
from horizon.utils import filters

from openstack_dashboard.dashboards.project.instances import (
    tables as project_tables)
from tionix_dashboard.dashboards.tionix.metrics.instances.actions import (
    MetricsInstanceShowAction)

from tionix_dashboard.helpers.fields import EmptyCharField
from tionix_dashboard.helpers.tables import (
    DTTnxPaginationMixin, DataTableFirstLastPagination,
    InstancesFilterPaginationMixin)

from tionix_dashboard.dashboards.project.project_instances import (
    actions as project_actions
)

from tionix_dashboard.dashboards.admin.instances.tables import (
    STATUS_CHOICES, TASK_STATUS_CHOICES)
from tionix_dashboard.dashboards.tionix.vdi.instances.actions import (
    MultiMembersAndGroupsManageAction)
from tionix_dashboard.dashboards.tionix.vdi.tables import BaseSessionTable
from tionix_dashboard.dashboards.tionix.scheduler.actions import (
    VDIInstanceSchedulerMultiAction, VDIInstanceSchedulerAction)
from tionix_dashboard.dashboards.tionix.vdi.constants import (
    VDI_PROJECT_DETAILS, VDI_INSTANCE_DETAILS)
from tionix_dashboard.dashboards.tionix.vdi.enums import OperationModeEnum
from tionix_dashboard.dashboards.tionix.vdi.instances import actions


def get_url_for_project(datum):
    link = VDI_PROJECT_DETAILS
    obj_id = datum.tenant_id
    return reverse(link, args=(obj_id,))


def get_url_for_details(datum):
    link = VDI_INSTANCE_DETAILS
    return reverse(link, args=(datum.tenant_id, datum.id))


class BaseVDIInstancesTable(tables.DataTable):
    name = tables.Column(
        "name",
        verbose_name=_("Name"),
        link=get_url_for_details,
        form_field=EmptyCharField(max_length=255),
        update_action=project_actions.InstanceUpdateCell,
    )
    tenant = tables.Column(
        "tenant_name",
        link=get_url_for_project,
        verbose_name=_("Project")
    )
    ip = tables.Column(
        project_tables.get_ips,
        verbose_name=_("IP Address"),
        attrs={'data-type': "ip"}
    )
    size = tables.Column(
        project_tables.get_size,
        verbose_name=_("Size")
    )
    status = tables.Column(
        "status",
        filters=(title, filters.replace_underscores),
        verbose_name=_("Status"),
        status_choices=STATUS_CHOICES,
        display_choices=project_tables.STATUS_DISPLAY_CHOICES
    )
    az = tables.Column(
        "availability_zone",
        verbose_name=_("Zone")
    )
    task = tables.Column(
        "OS-EXT-STS:task_state",
        verbose_name=_("Task"),
        empty_value=project_tables.TASK_DISPLAY_NONE,
        status_choices=TASK_STATUS_CHOICES,
        display_choices=project_tables.TASK_DISPLAY_CHOICES
    )
    state = tables.Column(
        project_tables.get_power_state,
        filters=(title, filters.replace_underscores),
        verbose_name=_("Power State"),
        display_choices=project_tables.POWER_DISPLAY_CHOICES
    )
    tnx_operation_mode = tables.Column(
        "tnx_operation_mode",
        verbose_name=_("Operation mode"),
        display_choices=OperationModeEnum.get_choices()
    )
    tnx_last_session = tables.Column(
        "tnx_last_session",
        verbose_name=_("Last session")
    )


class VDIInstancesTable(DTTnxPaginationMixin,
                        DataTableFirstLastPagination,
                        InstancesFilterPaginationMixin,
                        BaseVDIInstancesTable):
    class Meta(object):
        name = "vdi_instances"
        verbose_name = _("Instances")
        template = '_data_table.html'
        row_class = actions.UpdateRow
        status_columns = ["status", "task"]
        table_actions = (
            actions.VDIInstanceFilterAction,
            actions.LaunchVDIInstanceAction,
        )

        table_actions_menu = (
            VDIInstanceSchedulerMultiAction,
            actions.MultiResetStateAction,
            project_actions.TnxMultiTogglePause,
            project_actions.TnxMultiToggleUnPause,
            project_actions.TnxMultiToggleShelve,
            project_actions.TnxMultiToggleUnShelve,
            project_tables.StartInstance,
            actions.VDIStopInstance,
            actions.MultiRebuildInstance,
            project_actions.TnxMultiDeleteInstance,
            MultiMembersAndGroupsManageAction
        )

        row_actions = (
            project_tables.StartInstance,
            actions.VDIInstanceManageUsers,
            project_actions.TnxTogglePause,
            actions.RebuildInstance,
            project_actions.TnxToggleShelve,
            actions.VDIStopInstance,
            actions.ResetStateAction,
            VDIInstanceSchedulerAction,
            MetricsInstanceShowAction,
            project_actions.TnxDeleteInstance,
            actions.CloneVDILink
        )


class SessionTable(BaseSessionTable):

    class Meta(object):
        name = "sessions"
        verbose_name = _("Sessions")
        multi_select = False
        table_actions = (actions.SessionFilterAction,)
