# coding: utf-8
# pylint: disable=super-on-old-class, len-as-condition

import json
import logging
from collections import defaultdict

from django.core.exceptions import ValidationError
from django.utils.translation import ugettext_lazy as _

from horizon import exceptions, forms, messages, workflows
from horizon.workflows.base import (
    UpdateMembersStep, MembershipAction)

from openstack_dashboard import api
from openstack_dashboard.utils.identity import IdentityMixIn

from tionix_dashboard.clients.base import novaclient
from tionix_dashboard.dashboards.tionix.vdi.constants import VDI_INDEX
from tionix_dashboard.dashboards.tionix.vdi.helpers import (
    get_projects_for_instance_launch, get_quota_by_tenant,
    get_servers_by_tenant, get_project_users, get_user_roles_by_project)
from tionix_dashboard.dashboards.utils import (
    quotas_usages_validation, is_domain_supported)
from tionix_dashboard.helpers.fields import AutoGenNameCharField
from tionix_dashboard.helpers.views import RefererRedirectMixin
from tionix_dashboard.helpers.vm_constants import VMTypes

logger = logging.getLogger(__name__)


UPDATE_USERS_ACTION_SLUG = 'update_users'
UPDATE_GROUPS_ACTION_SLUG = 'update_groups'

PROJECT_GROUP_ENABLED = is_domain_supported()


class FieldGroup(object):
    def __init__(self, **kwargs):
        for k, v in kwargs.iteritems():
            setattr(self, k, v)


class UpdateVDIInstanceMembersMixin(object):
    """Updates vdi instance members"""

    def _update_members(self, instance, data, with_group_users=False,
                        delete_old_users=True):

        step = self.get_step(UPDATE_USERS_ACTION_SLUG)

        user_list = data.get(
            step.get_member_field_name(step.action.role.id)
        )
        if with_group_users:
            step = self.get_step(UPDATE_GROUPS_ACTION_SLUG)

            group_list = data.get(
                step.get_member_field_name(step.action.role.id)
            )
            for group in group_list:
                try:
                    users = api.keystone.user_list(
                        self.request,
                        domain=self.context.get("domain_id", None),
                        group=group)
                except Exception as err:
                    logger.error(unicode(err))
                else:
                    user_list.extend([user.id for user in users])

        tnx_users = instance.tnx_users or []
        instance_users = [user['id'] for user in tnx_users]

        for user in user_list:
            if user not in instance_users:
                instance.add_user(user)
        if delete_old_users:
            for user in instance_users:
                if user not in user_list:
                    instance.remove_user(user)


class InstanceMembershipAction(MembershipAction):

    role = None

    def _get_membership_data(self, request):
        return []

    def __init__(self, request, context, *args, **kwargs):
        super(InstanceMembershipAction, self).__init__(
            request, context, *args, **kwargs
        )

        default_field_name = self.get_default_role_field_name()
        self.fields[default_field_name] = forms.CharField(required=False)
        self.fields[default_field_name].initial = self.role.id

        field_name = self.get_member_field_name(self.role.id)
        self.fields[field_name] = forms.MultipleChoiceField(
            required=False, label=self.role.name
        )

        self.fields[field_name].choices = self._get_membership_data(request)


class InstanceUpdateMembersStep(UpdateMembersStep):

    template_name = 'vdi/instances/_launch_users_step.html'
    show_roles = False

    def contribute(self, data, context):
        if data:

            post = self.workflow.request.POST
            role = self.action_class.role

            field = self.get_member_field_name(role.id)
            context[field] = post.getlist(field)
        return context


class CreateUsersAction(InstanceMembershipAction, IdentityMixIn):

    role = FieldGroup(id=u'1', name=u'instance_users')

    def _get_membership_data(self, request):
        all_users, available_users = [], []
        project_user_dict = {}
        domain_id = api.keystone.get_effective_domain_id(self.request)
        project_id = self.initial.get("project_id", None)
        try:
            all_users = {
                user.id: user.name
                for user in api.keystone.user_list(request, domain=domain_id)}
        except Exception as err:
            logger.error(unicode(err))
        else:
            if project_id is not None:
                projects = [(project_id, project_id)]
            else:
                projects = get_projects_for_instance_launch(request)
            admin_roles = set(self.get_admin_roles())
            for project_id, _project_name in projects:
                roles = set(get_user_roles_by_project(request, project_id))
                if admin_roles & roles:
                    users = []
                    try:
                        users = get_project_users(request, project_id)
                    except Exception as err:
                        logger.error(unicode(err))
                    else:
                        available_users.extend(users)
                    project_user_dict[project_id] = users
        self.members_data = json.dumps(project_user_dict)

        # users from project
        users_list = [(user_id, all_users.get(user_id))
                      for user_id in available_users]

        return users_list

    class Meta(object):
        name = _("Instance users")
        slug = UPDATE_USERS_ACTION_SLUG


class UpdateUsersAction(InstanceMembershipAction):

    role = FieldGroup(id=u'1', name=u'instance_users')

    # pylint: disable=too-many-nested-blocks
    def _get_membership_data(self, request):
        users_list = []
        domain_id = api.keystone.get_effective_domain_id(self.request)
        try:
            all_users = {
                user.id: user.name
                for user in api.keystone.user_list(request, domain=domain_id)}
        except Exception as err:
            logger.error(unicode(err))
        else:
            project_id = getattr(self.initial['instance'],
                                 'tnx_tenant_id',
                                 None)
            project_users = get_project_users(request, project_id)
            users_list = {(user_id, all_users.get(user_id))
                          for user_id in project_users}

        return users_list

    def __init__(self, request, context, *args, **kwargs):
        super(UpdateUsersAction, self).__init__(
            request, context, *args, **kwargs
        )
        instance_users = [
            user['id']
            for user in getattr(self.initial["instance"], 'tnx_users', [])]

        # pylint: disable=no-member
        field_name = self.get_member_field_name(self.role.id)
        self.fields[field_name].initial = instance_users

        # sort by name
        for field_name in self.fields:
            if field_name.startswith(self.slug):
                # sort by name
                self.fields[field_name].choices.sort(
                    key=lambda x: x[1].lower())

    class Meta(object):
        name = _("Instance users")
        slug = UPDATE_USERS_ACTION_SLUG


class CreateUsersStep(InstanceUpdateMembersStep):
    action_class = CreateUsersAction

    available_list_title = _("All Users")
    members_list_title = _("Instance users")
    no_available_text = _("No users found.")
    no_members_text = _("No users.")


class UpdateUsersStep(CreateUsersStep):
    action_class = UpdateUsersAction
    template_name = 'vdi/_manage_users.html'
    depends_on = ('instance',)


class CreateGroupsAction(InstanceMembershipAction, IdentityMixIn):

    role = FieldGroup(id=u'1', name=u'instance_groups')

    # pylint: disable=too-many-nested-blocks
    def _get_membership_data(self, request):
        all_groups, available_groups = [], []
        project_group_dict = defaultdict(list)
        domain_id = self.initial.get("domain_id", None)
        project_id = self.initial.get("project_id", None)

        if api.keystone.policy.check(
                (("identity", "identity:list_groups"),), request):
            kwargs = {'domain': domain_id}
        else:
            kwargs = {'user': request.user.id}

        # pylint: disable=too-many-nested-blocks
        try:
            all_groups = {
                group.id: group.name
                for group in api.keystone.group_list(request, **kwargs)
            }
        except Exception as err:
            logger.error(unicode(err))
        else:
            if project_id is not None:
                projects = [(project_id, project_id)]
            else:
                projects = get_projects_for_instance_launch(request)
            for project_id, _project_name in projects:
                project_groups = api.keystone.get_project_groups_roles(
                    request, project_id
                ).keys()
                available_groups.extend(project_groups)
                project_group_dict[project_id] = project_groups
        self.members_data = json.dumps(project_group_dict)

        # groups from project
        groups_list = [(user_id, all_groups.get(user_id))
                       for user_id in available_groups]

        return groups_list

    class Meta(object):
        name = _("Instance groups")
        slug = UPDATE_GROUPS_ACTION_SLUG


class CreateGroupsStep(InstanceUpdateMembersStep):
    action_class = CreateGroupsAction

    depends_on = ('domain_id',)

    available_list_title = _("All Groups")
    members_list_title = _("Project Groups")
    no_available_text = _("No groups found.")
    no_members_text = _("No groups.")


class UpdateVDIInstanceMembers(RefererRedirectMixin,
                               UpdateVDIInstanceMembersMixin,
                               workflows.Workflow):
    success_message = _('Instance user list updated.')
    failure_message = _('Unable to update user list of instance.')

    success_url = "horizon:tionix:vdi:index"
    name = _("Assign to users")

    slug = 'vdi_instances_members'

    if PROJECT_GROUP_ENABLED:
        default_steps = (UpdateUsersStep,
                         CreateGroupsStep)
    else:
        default_steps = (UpdateUsersStep,)

    def handle(self, request, context):
        self._update_members(self.context['instance'],
                             context,
                             with_group_users=PROJECT_GROUP_ENABLED)
        return True


class VDIInstanceLaunchDetailsAction(workflows.Action):
    """Instance launch details action"""

    project = forms.ChoiceField(
        label=_('VDI Project'),
        widget=forms.Select(attrs={
            'ng-model': 'currentProject',
        }),
    )
    instances_count = forms.IntegerField(
        label=_('Instance count'),
        min_value=1,
        initial=1,
        widget=forms.NumberInput(attrs={
            'ng-model': 'currentInstanceCount',
            'validate-number-min': '1',
        }),
    )
    name = AutoGenNameCharField(
        label=_('Instance name'),
        max_length=255,
        required=False
    )

    def __init__(self, request, context, *args, **kwargs):
        super(VDIInstanceLaunchDetailsAction, self).__init__(request,
                                                             context,
                                                             *args,
                                                             **kwargs)
        self._populate_quotas()
        try:
            projects = self._populate_project_choices(request)
        except ValueError as err:
            messages.info(request, unicode(err))
            raise
        else:
            self.fields['project'].choices = projects

    def _populate_quotas(self):
        self.quotas = []
        tenant_ids = [tenant_id for tenant_id, _tenant_name in
                      self.fields['project'].choices if tenant_id]

        for tenant_id in tenant_ids:
            try:
                tenant = {'id': tenant_id}
                servers = get_servers_by_tenant(self.request, tenant_id)
                quota = get_quota_by_tenant(self.request, tenant_id)

                instances_quota = quota.instances

                instances_usage = sum(tenant_id == servers[server_id]
                                      for server_id in servers)
                instances_available = instances_quota - instances_usage

                tenant.update({'instances_quota': instances_quota,
                               'instances_usage': instances_usage,
                               'instances_available': instances_available})

                self.quotas.append(tenant)
            except Exception as err:
                logger.error(err)

    @staticmethod
    def _populate_project_choices(request):
        projects = get_projects_for_instance_launch(request)
        if projects:
            projects = [('', _('Select project'))] + projects
        else:
            raise ValueError(_('No projects available'))

        return projects

    def clean_instances_count(self):
        instances_count = self.cleaned_data['instances_count']
        tenant_id = self.cleaned_data.get('project')
        if tenant_id is None:
            return instances_count

        nova_cl = novaclient(self.request, project_id=tenant_id)
        try:
            tenant = nova_cl.tnx_tenants.get(tenant_id)
        except Exception as err:
            logger.error(err)
            raise ValidationError(_('Unable to get VDI project.'))

        flavor_id = tenant.tnx_flavor_id

        quotas_usages_validation(self.request, flavor_id=flavor_id,
                                 tenant_id=tenant_id,
                                 instances_count=instances_count)

        return instances_count

    class Meta(object):
        name = _("Details")


class VDIInstanceLaunchDetailsStep(workflows.Step):
    """Instance launch details step"""

    action_class = VDIInstanceLaunchDetailsAction
    contributes = ('project', 'instances_count', 'name')
    template_name = 'vdi/instances/_launch_details_step.html'


class VDIInstanceLaunchWorkflow(UpdateVDIInstanceMembersMixin,
                                workflows.Workflow,
                                IdentityMixIn):
    """Instance launch workflow"""
    success_message = _('Successfully launched instance(s).')
    failure_message = _('Unable to launch instance(s).')
    success_url = "horizon:tionix:vdi:index"
    name = _("Launch VDI Instance")
    slug = 'vdi_instances_launch'
    wizard = True

    if PROJECT_GROUP_ENABLED:
        default_steps = (VDIInstanceLaunchDetailsStep,
                         CreateUsersStep,
                         CreateGroupsStep)
    else:
        default_steps = (VDIInstanceLaunchDetailsStep,
                         CreateUsersStep)

    def handle(self, request, context):
        result = True

        project_id = context['project']
        instances_count = int(context['instances_count'])

        nova_client = novaclient(request, project_id=project_id)

        instance_name = context['name']

        try:
            server = nova_client.servers.create(
                name=instance_name,
                image=None,
                flavor=None,
                min_count=instances_count,
                max_count=instances_count,
                tnx_type=int(VMTypes.VDI),
                tnx_tenant_id=project_id
            )
        except Exception as err:
            result = False
            logger.error(unicode(err))
            exceptions.handle(request, err.message, redirect=VDI_INDEX)
        else:
            user_roles = get_user_roles_by_project(request, server.tenant_id)
            if set(user_roles) & set(self.get_admin_roles()):
                reservation_id = getattr(server,
                                         'OS-EXT-SRV-ATTR:reservation_id',
                                         None)
                try:
                    if reservation_id:
                        instances = novaclient(request).tnx_servers.list(
                            search_opts={'all_tenants': True,
                                         'reservation_id': reservation_id}
                        )
                    else:
                        instances = [
                            novaclient(request).tnx_servers.get(server.id)
                        ]
                except Exception as err:
                    logger.error(unicode(err))
                    instances = []

                for instance in instances:
                    self._update_members(
                        instance,
                        context,
                        with_group_users=PROJECT_GROUP_ENABLED)

        return result


class CloneVDIInstance(VDIInstanceLaunchWorkflow):
    """Clone vdi instance workflow"""
    slug = "clone_vdi_instance"
    name = _("Clone Instance")
    finalize_button_name = _("Clone")
    success_message = _('Cloned %(count)s named "%(name)s".')
    failure_message = _('Unable to clone %(count)s named "%(name)s".')
    success_url = "horizon:tionix:vdi:index"

    default_steps = (VDIInstanceLaunchDetailsStep,)

    def __init__(self, request=None, context_seed=None, entry_point=None,
                 *args, **kwargs):

        self.is_admin = is_admin = api.keystone.is_domain_admin(request)
        if is_admin and PROJECT_GROUP_ENABLED:
            self.default_steps += (UpdateUsersStep, CreateGroupsStep)
        elif is_admin:
            self.default_steps += (UpdateUsersStep,)

        super(CloneVDIInstance, self).__init__(
            request=request, context_seed=context_seed,
            entry_point=entry_point, *args, **kwargs
        )

        for step in self.steps:
            for field_name, field in step.action.fields.items():
                value = context_seed.get(field_name)
                if value:
                    field.initial = value


class MultiUpdateMembers(UpdateVDIInstanceMembers):
    slug = 'multi_vdi_instances_members'
    default_steps = (CreateUsersStep,)

    def handle(self, request, context):
        for instance in self.context['instances']:
            self._update_members(instance, context)
        return True


class MultiCreateUsersStep(CreateUsersStep):
    depends_on = ('domain_id', 'project_id', 'instances')


class MultiCreateGroupsStep(CreateGroupsStep):
    depends_on = ('domain_id', 'project_id', 'instances')


class MultiUpdateMembersWithGroups(RefererRedirectMixin,
                                   UpdateVDIInstanceMembersMixin,
                                   workflows.Workflow):
    success_message = _('Instances user and group lists updated.')
    failure_message = _('Unable to update user and group lists of instances.')

    success_url = "horizon:tionix:vdi:index"
    name = _("Assign to users")

    slug = 'multi_members_groups'

    default_steps = (MultiCreateUsersStep, MultiCreateGroupsStep)

    # pylint: disable=arguments-differ
    def get_absolute_url(self):
        return self.request.get_full_path()

    def handle(self, request, context):
        for instance in self.context['instances']:
            self._update_members(instance,
                                 context,
                                 with_group_users=True,
                                 delete_old_users=False)
        return True
