# coding: utf-8

import logging

from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _
from horizon import tables, exceptions
from openstack_dashboard import api, policy

from tionix_dashboard.clients.exceptions import UrlNotAvailableError
from tionix_dashboard.clients.tenants import TNXTenantClient
from tionix_dashboard.dashboards.tionix.vdi.constants import VDI_INDEX
from tionix_dashboard.dashboards.tionix.vdi.enums import OperationModeEnum
from tionix_dashboard.helpers.actions import (
    TnxFilterAction, TnxUpdateAction, MultiLinkAction)
from tionix_dashboard.helpers.utils import (
    get_yes_no_filter_value, get_filter_value_by_dict)
from tionix_dashboard.url_resolve_map import (
    VDI_PROJECTS_CREATE, VDI_PROJECTS_UPDATE, VDI_PROJECTS_CLONE,
    VDI_PROJECTS_MULTI_UPDATE)

logger = logging.getLogger(__name__)


class CreateVDIProjectAction(tables.LinkAction):
    """
    Action to create vdi project
    """
    name = 'create'
    verbose_name = _('Create VDI Project')
    classes = ('ajax-modal',)
    url = VDI_PROJECTS_CREATE

    policy_rules = (("identity", "admin_and_matching_domain_id"),)


class EditVDIProjectAction(tables.LinkAction):
    """
    Action to edit project
    """
    name = 'vdi_edit_project'
    verbose_name = _('Edit Project')
    classes = ('ajax-modal',)
    url = VDI_PROJECTS_UPDATE

    policy_rules = (("identity", "admin_and_matching_domain_id"),)


class EditVDIProjectMultiAction(MultiLinkAction):
    """
    Action to edit project
    """
    name = 'multi_vdi_edit_project'
    verbose_name = _('Edit Projects')
    url = VDI_PROJECTS_MULTI_UPDATE

    policy_rules = (("identity", "admin_and_matching_domain_id"),)


class CloneVDIProjectAction(tables.LinkAction):
    """
    Action to clone project
    """
    name = 'vdi_clone_project'
    verbose_name = _('Clone Project')
    classes = ('ajax-modal',)
    url = VDI_PROJECTS_CLONE

    policy_rules = (("identity", "admin_and_matching_domain_id"),)


class UsageLink(tables.LinkAction):
    name = "usage"
    verbose_name = _("View Usage")
    url = "horizon:identity:projects:usage"
    icon = "stats"
    policy_rules = (("compute", "compute_extension:simple_tenant_usage:show"),)

    def allowed(self, request, project):
        return request.user.is_superuser


class VDIProjectFilterAction(TnxFilterAction, tables.FilterAction):
    """
    Filter VDI projects filter
    """
    name = "filter_vdi_projects"
    filter_type = "server"

    filter_choices = (
        ('name', _("Name"), False),
        ('enabled', _("Enabled"), False),
        ('tnx_operation_mode', _("Operation Mode"), True),
    )

    filter_string_modifiers = {
        'enabled': get_yes_no_filter_value(),
        'tnx_operation_mode': (
            get_filter_value_by_dict(OperationModeEnum.values,
                                     not_found_none=False))
    }


class UpdateCell(TnxUpdateAction):
    """Inline project update"""

    def allowed(self, request, project, cell):
        policy_rule = (("identity", "identity:update_project"),)
        policy_target = {'target.project.domain_id': project.domain_id}
        return (
            (cell.column.name != 'enabled' or
             request.user.project_id != cell.datum.id) and
            api.keystone.keystone_can_edit_project() and
            policy.check(policy_rule, request, target=policy_target))

    def update_cell(self, request, datum, obj_id, cell_name, new_cell_value):
        result = True
        # Server couldn't pass boolean false type
        if isinstance(new_cell_value, bool):
            new_cell_value = unicode(new_cell_value)
        try:
            TNXTenantClient.update(request,
                                   obj_id,
                                   **{cell_name: new_cell_value})
        except Exception as err:
            logger.error(unicode(err))
            exceptions.handle(request, ignore=True)
            result = False
        return result


class UpdateRow(tables.Row):
    ajax = True

    def get_data(self, request, obj_id):
        try:
            tenant = TNXTenantClient.get(request, obj_id)
        except UrlNotAvailableError:
            raise
        except Exception as err:
            logger.error(unicode(err))
            tenant = None
            exceptions.handle(request,
                              _('Unable to get tenant.'),
                              redirect=reverse(VDI_INDEX))

        return tenant


class ProjectSessionFilterAction(TnxFilterAction):
    """Session filter"""
    filter_type = "server"

    filter_choices = [
        ('instance_name', _('Instance name'), False),
        ('username', _("User Name"), False),
        ('connection_type', _("Connection"), False),
        ('client_info', _("Client"), False),
        ('client_os', _("OS"), False),
        ('client_ip', _("IP"), False)
    ]
