# coding: utf-8

import json

import logging

from django.utils.translation import ugettext_lazy as _

from horizon import exceptions
from horizon.utils import functions
from horizon.utils.memoized import memoized

from keystoneclient.exceptions import AuthorizationFailure, Unauthorized

from neutronclient.common.exceptions import NotFound

from openstack_dashboard import api
from openstack_dashboard.api import neutron
from openstack_dashboard.dashboards.project.networks.subnets import utils

from tionix_dashboard.clients.base import neutronclient, keystoneclient

logger = logging.getLogger(__name__)


class ProjectNetworkMixin(object):

    def _create_network(self, request, data, auth_token=None):
        try:
            params = {'name': data['net_name'],
                      'admin_state_up': (data['admin_state'] == 'True'),
                      'shared': data['shared']}
            if api.neutron.is_port_profiles_supported():
                params['n1kv:profile'] = data['net_profile_id']

            body = {'network': params}
            network = neutronclient(
                request, auth_token=auth_token).create_network(body=body)
            return network
        except Exception as err:
            logger.error(unicode(err))
            return None

    def _setup_subnet_parameters(self, params, data, is_create=True):
        is_update = not is_create
        params['enable_dhcp'] = data['enable_dhcp']
        if int(data['ip_version']) == 6:
            ipv6_modes = utils.get_ipv6_modes_attrs_from_menu(
                data['ipv6_modes'])
            if ipv6_modes[0] and is_create:
                params['ipv6_ra_mode'] = ipv6_modes[0]
            if ipv6_modes[1] and is_create:
                params['ipv6_address_mode'] = ipv6_modes[1]
        if data['allocation_pools']:
            pools = [dict(zip(['start', 'end'], pool.strip().split(',')))
                     for pool in data['allocation_pools'].split('\n')
                     if pool.strip()]
            params['allocation_pools'] = pools
        if data['host_routes'] or is_update:
            routes = [dict(zip(['destination', 'nexthop'],
                               route.strip().split(',')))
                      for route in data['host_routes'].split('\n')
                      if route.strip()]
            params['host_routes'] = routes
        if data['dns_nameservers'] or is_update:
            nameservers = [ns.strip()
                           for ns in data['dns_nameservers'].split('\n')
                           if ns.strip()]
            params['dns_nameservers'] = nameservers

    def _create_subnet(self, request, data, network=None, auth_token=None):
        if network:
            network_id = network['id']
        else:
            network_id = self.context.get('network_id')
        try:
            params = {'network_id': network_id,
                      'name': data['subnet_name']}
            if 'cidr' in data and data['cidr']:
                params['cidr'] = data['cidr']
            if 'ip_version' in data and data['ip_version']:
                params['ip_version'] = int(data['ip_version'])
            if data['no_gateway']:
                params['gateway_ip'] = None
            elif data['gateway_ip']:
                params['gateway_ip'] = data['gateway_ip']
            if data.get('subnetpool'):
                params['subnetpool_id'] = data['subnetpool']
                if data.get('prefixlen'):
                    params['prefixlen'] = data['prefixlen']

            body = {'subnet': {'network_id': network_id}}

            self._setup_subnet_parameters(params, data)
            body['subnet'].update(params)

            subnet = neutronclient(
                request, auth_token=auth_token).create_subnet(body=body)
            return subnet
        except Exception as err:
            logger.error(unicode(err))
            return None

    def _delete_network(self, request, network, auth_token=None):
        try:
            neutronclient(request, auth_token=auth_token).delete_network(
                network['id'])
        except Exception as err:
            logger.error(unicode(err))

    # pylint: disable=no-else-return
    def _handle_network(self, request, data, auth_token):
        network = self._create_network(request, data, auth_token=auth_token)
        if not network:
            exceptions.handle(request, _(
                'Unable to create network "{0}".').format(data['net_name']))
            return None
        if not data['with_subnet']:
            return network
        subnet = self._create_subnet(request, data, network['network'],
                                     auth_token=auth_token)
        if subnet:
            return network
        else:
            exceptions.handle(request, _(
                'Unable to create subnet "{0}".').format(data['subnet_name']))
            try:
                self._delete_network(request, network['network'],
                                     auth_token=auth_token)
            except Exception as err:
                logger.error(unicode(err))
            return None

    def _get_network_id(self, request, network_id=None, network_list=None,
                        tenant_id=None, tnx_tenant_name=None):

        new_network_id = None

        if network_id:
            try:
                neutron.network_get(request, network_id)
            except NotFound:
                if network_list:
                    network_list = json.loads(network_list)
                    try:
                        keystone_client = keystoneclient(
                            request, project_id=tenant_id)
                    except (AuthorizationFailure, Unauthorized):
                        exceptions.handle(
                            request, _('Unable to authorize in project '
                                       '"{0}".').format(tnx_tenant_name))
                    else:
                        tenant_auth_token = keystone_client.auth_token
                        new_network_data = network_list.get(network_id)

                        if new_network_data:
                            network = self._handle_network(request,
                                                           new_network_data,
                                                           tenant_auth_token)
                            if network:
                                new_network_id = network['network']['id']
            else:
                new_network_id = network_id

        return new_network_id

    def _update_tenant_network(self, request, novaclient=None, tenant_id=None,
                               network_id=None):
        data = {'network_id': network_id}

        try:
            novaclient.tnx_tenants.update(tenant_id, **data)
        except Exception as err:
            exceptions.handle(request,
                              _('Unable to assign network for VDI '
                                'project "{0}".').format(data['name']))
            logger.error(unicode(err))
            return False
        return True


def get_network_field_data(request, shared=True, include_empty_option=False):
    """Retrieve networks data for field"""
    params, networks = {'shared': shared}, []
    try:
        networks = [
            (network.id, network.name)
            for network in neutron.network_list(request, **params)]
    except Exception as err:
        logger.error(err)
        exceptions.handle(request, _('Unable to retrieve shared network list'))

    if not networks:
        networks = [("", _("No networks available"))]
    elif include_empty_option:
        networks = [("", _("Select Network"))] + networks

    return networks


@memoized
def get_available_images(request, project_id=None):
    """Returns a list of images that are public or owned by the given
    project_id. If project_id is not specified, only public images
    are returned.
    """
    images = []
    public = {"is_public": True,
              "status": "active"}
    try:
        images, _more, _prev = api.glance.image_list_detailed(
            request, filters=public)
        images = [image for image in images if image.is_public]
    except Exception as err:
        logger.error(unicode(err))
        exceptions.handle(request, _("Unable to retrieve public images."))

    if project_id:
        owner = {"property-owner_id": project_id,
                 "status": "active"}
        try:
            owned_images, _more, _prev = api.glance.image_list_detailed(
                request, filters=owner)
            images.extend(owned_images)
        except Exception as err:
            logger.error(unicode(err))
            exceptions.handle(request,
                              _("Unable to retrieve images for "
                                "the current project."))

    # Remove duplicate images
    image_ids = []
    final_images = []
    for image in images:
        if image.id not in image_ids:
            image_ids.append(image.id)
            final_images.append(image)
    return [image for image in final_images
            if image.container_format not in ('aki', 'ari')]


def get_image_field_data(request, context):
    """Retrieve images data for field"""
    choices = []
    images = get_available_images(request, context.get('project_id'))
    if images:
        choices.append(("", _("Select Image")))
        for image in images:
            image.bytes = getattr(
                image, 'virtual_size', None) or image.size
            image.volume_size = max(
                image.min_disk, functions.bytes_to_gigabytes(image.bytes))
            choices.append((image.id, image))
            if context.get('image_id') == image.id:
                if 'volume_size' not in context:
                    context['volume_size'] = image.volume_size
    else:
        choices.append(("", _("No images available")))
    return choices


def get_availability_zone_data(request):
    zones = []
    try:
        zones = api.nova.availability_zone_list(request)
    except Exception as err:
        logger.error(unicode(err))
        exceptions.handle(request,
                          _('Unable to retrieve availability zones.'))

    zone_list = [(zone.zoneName, zone.zoneName)
                 for zone in zones if zone.zoneState['available']]
    zone_list.sort(key=lambda x: x[1].lower())
    if not zone_list:
        zone_list.insert(0, ("", _("No availability zones found")))
    else:
        zone_list.insert(0, ("", _("Any Availability Zone")))
    return zone_list
