# coding: utf-8
from django.core.urlresolvers import reverse
from django.template import defaultfilters as filters
from django.utils.translation import ugettext_lazy as _

from horizon import forms, tables

from openstack_dashboard import policy

from tionix_dashboard.dashboards.tionix.vdi.tables import BaseSessionTable
from tionix_dashboard.templatetags.tionix_tags import format_date
from tionix_dashboard.dashboards.identity.projects.actions import (
    TnxDeleteTenant, TnxMultiDeleteTenant
)
from tionix_dashboard.dashboards.identity.projects.tables import (
    BaseInstanceAuditLog)
from tionix_dashboard.dashboards.tionix.scheduler.actions import (
    VDIProjectSchedulerAction, VDIProjectSchedulerMultiAction
)
from tionix_dashboard.dashboards.tionix.vdi.constants import (
    VDI_PROJECT_DETAILS)
from tionix_dashboard.dashboards.tionix.vdi.enums import OperationModeEnum
from tionix_dashboard.dashboards.tionix.vdi.projects import actions
from tionix_dashboard.dashboards.tionix.vdi.projects.actions import (
    EditVDIProjectMultiAction)
from tionix_dashboard.dashboards.utils import is_domain_supported
from tionix_dashboard.url_resolve_map import VDI_INSTANCE_DETAILS
from tionix_dashboard.dashboards.tionix.vdi.projects.actions import (
    ProjectSessionFilterAction)


class VDIProjectsTable(tables.DataTable):
    name = tables.Column(
        'name',
        verbose_name=_('Name'),
        link=VDI_PROJECT_DETAILS,
        form_field=forms.CharField(max_length=64),
        update_action=actions.UpdateCell
    )
    description = tables.Column(
        lambda obj: getattr(obj, 'description', None),
        verbose_name=_('Description'),
        form_field=forms.CharField(
            widget=forms.Textarea(attrs={'rows': 4}),
            required=False),
        update_action=actions.UpdateCell
    )

    if is_domain_supported():
        domain_name = tables.Column(
            'domain_name', verbose_name=_('Domain Name'))
    enabled = tables.Column(
        'enabled',
        verbose_name=_('Enabled'),
        filters=(filters.yesno, filters.capfirst),
        form_field=forms.BooleanField(
            label=_('Enabled'),
            required=False),
        update_action=actions.UpdateCell
    )
    tnx_operation_mode = tables.Column(
        'tnx_operation_mode',
        verbose_name=_('Operation Mode'),
        display_choices=OperationModeEnum.get_choices()
    )
    tnx_instances_count = tables.Column(
        'tnx_instances_count',
        verbose_name=_("Instances")
    )

    class Meta(object):
        name = "vdi_projects"
        verbose_name = _("Projects")
        row_class = actions.UpdateRow
        table_actions = (
            actions.VDIProjectFilterAction,
            actions.CreateVDIProjectAction,
            TnxMultiDeleteTenant
        )

        table_actions_menu = (
            VDIProjectSchedulerMultiAction,
            EditVDIProjectMultiAction
        )

        row_actions = (
            actions.CloneVDIProjectAction,
            TnxDeleteTenant,
            actions.EditVDIProjectAction,
            VDIProjectSchedulerAction
        )

    def __init__(self, request, data=None, needs_form_wrapper=None, **kwargs):
        super(VDIProjectsTable, self).__init__(
            request,
            data=data,
            needs_form_wrapper=needs_form_wrapper,
            **kwargs)
        self.columns['name'].get_link_url = self.get_project_detail_link

    def get_project_detail_link(self, project):
        """return link to project"""
        policy_rule = (("identity", "identity:get_project"),)
        policy_target = {
            'project': project,
            'target.project.domain_id': project.domain_id,
        }
        if policy.check(policy_rule, self.request, target=policy_target):
            return reverse(VDI_PROJECT_DETAILS, args=(project.id,))


def get_url_for_instance(datum):
    project_id = getattr(datum, 'project_id', None)
    obj_id = getattr(datum, 'instance_id', None)
    return reverse(VDI_INSTANCE_DETAILS, args=(project_id, obj_id,))


class TnxVDIInstancesAuditLog(BaseInstanceAuditLog):
    instance_name = tables.Column('instance_name',
                                  verbose_name=_('Instance'),
                                  link=get_url_for_instance)
    start_time = tables.Column('start_time', verbose_name=_('Start time'),
                               filters=(format_date, ))

    class Meta(object):
        name = 'audit'
        verbose_name = _('Instances Action List')
        multi_select = False
        table_actions = (tables.FilterAction,)


class ProjectsSessionTable(BaseSessionTable):

    instance_name = tables.Column(
        "instance_name",
        verbose_name=_("Instance name"),
        link=get_url_for_instance,
    )

    class Meta(object):
        name = "project_sessions"
        verbose_name = _("Project Sessions")
        multi_select = False
        table_actions = (ProjectSessionFilterAction,)
        columns = ('instance_name', 'username', 'created_at', 'closed_at',
                   'connection_type', 'client_info', 'client_os', 'client_ip')
