# coding: utf-8

import logging

from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _

from horizon import exceptions, workflows
from horizon.utils import memoized

from openstack_dashboard import api
from openstack_dashboard import policy
from openstack_dashboard.dashboards.identity.projects.views import (
    CreateProjectView, UpdateProjectView, PROJECT_INFO_FIELDS)
from openstack_dashboard.usage import quotas

from tionix_dashboard.clients.tenants import TNXTenantClient
from tionix_dashboard.dashboards.identity.projects import (
    views as projects_views)
from tionix_dashboard.dashboards.tionix.vdi import constants
from tionix_dashboard.dashboards.tionix.vdi.constants import VDI_PROJECT_INDEX
from tionix_dashboard.dashboards.tionix.vdi.helpers import get_tenants_kwargs
from tionix_dashboard.dashboards.tionix.vdi.projects import (
    workflows as project_workflows)
from tionix_dashboard.dashboards.tionix.vdi.projects.tables import (
    VDIProjectsTable
)
from tionix_dashboard.dashboards.tionix.vdi.projects.tabs import (
    ProjectDetailTabs)
from tionix_dashboard.dashboards.tionix.vdi.projects.workflows import (
    CreateDefaultNetworkWorkflow)


logger = logging.getLogger(__name__)


class CreateVDIProjectView(CreateProjectView):
    """Create VDI project view"""
    workflow_class = project_workflows.CreateVDIProject

    def get_initial(self):
        # pylint: disable=bad-super-call
        initial = super(CreateProjectView, self).get_initial()

        # Set the domain of the project
        domain = api.keystone.get_default_domain(self.request)
        initial["domain_id"] = domain.id
        initial["domain_name"] = domain.name

        # get initial quota defaults
        if is_project_quotas_disabled(self.request):
            self.workflow_class = project_workflows.CreateVDIProjectNoQuota
        elif api.keystone.is_domain_admin(self.request):
            try:
                quota_defaults = quotas.get_default_quota_data(self.request)

                try:
                    if (api.base.is_service_enabled(
                            self.request, 'network') and
                            api.neutron.is_quotas_extension_supported(
                                self.request)):
                        project_id = self.request.user.project_id
                        quota_defaults += api.neutron.tenant_quota_get(
                            self.request, tenant_id=project_id)
                except Exception as err:
                    logger.error(unicode(err))
                    error_msg = _('Unable to retrieve default Neutron quota '
                                  'values.')
                    self.add_error_to_step(error_msg, 'create_quotas')

                for field in quotas.QUOTA_FIELDS:
                    initial[field] = quota_defaults.get(field).limit

            except Exception as err:
                logger.error(unicode(err))
                error_msg = _('Unable to retrieve default quota values.')
                self.add_error_to_step(error_msg, 'create_quotas')

        return initial


def is_project_quotas_disabled(request):
    """Checks if quotas enabled"""
    return (api.keystone.is_multi_domain_enabled() and
            not api.keystone.is_domain_admin(request))


class UpdateVDIProjectView(UpdateProjectView):
    """Update VDI project view"""
    workflow_class = project_workflows.UpdateVDIProject

    def get_initial(self):

        # pylint: disable=bad-super-call
        initial = super(UpdateProjectView, self).get_initial()

        project_id = self.kwargs['tenant_id']
        initial['project_id'] = project_id

        try:
            # get initial project info
            project_info = api.keystone.tenant_get(self.request, project_id,
                                                   admin=True)
            for field in PROJECT_INFO_FIELDS:
                initial[field] = getattr(project_info, field, None)

            # Retrieve the domain name where the project belong
            if api.keystone.VERSIONS.active >= 3:
                try:
                    if policy.check((("identity", "identity:get_domain"),),
                                    self.request):
                        domain = api.keystone.domain_get(self.request,
                                                         initial["domain_id"])
                        initial["domain_name"] = domain.name

                    else:
                        domain = api.keystone.get_default_domain(self.request)
                        initial["domain_name"] = domain.name

                except Exception as err:
                    logger.error(unicode(err))
                    exceptions.handle(self.request,
                                      _('Unable to retrieve project domain.'),
                                      redirect=reverse(VDI_PROJECT_INDEX))

            if is_project_quotas_disabled(self.request):
                self.workflow_class = project_workflows.UpdateVDIProjectNoQuota
            elif api.keystone.is_domain_admin(self.request):
                # get initial project quota
                quota_data = quotas.get_tenant_quota_data(self.request,
                                                          tenant_id=project_id)
                if (api.base.is_service_enabled(self.request, 'network') and
                        api.neutron.is_quotas_extension_supported(
                            self.request)):
                    quota_data += api.neutron.tenant_quota_get(
                        self.request, tenant_id=project_id)
                for field in quotas.QUOTA_FIELDS:
                    initial[field] = quota_data.get(field).limit
        except Exception as err:
            logger.error(unicode(err))
            exceptions.handle(self.request,
                              _('Unable to retrieve project details.'),
                              redirect=reverse(VDI_PROJECT_INDEX))

        try:
            tenant = TNXTenantClient.get(self.request, initial['project_id'])
        except Exception as err:
            logger.error(unicode(err))
            exceptions.handle(
                self.request,
                _('Unable to retrieve VDI project information.'))
        else:
            initial.update(
                flavor=tenant.tnx_flavor_id,
                image_id=tenant.tnx_image_id,
                tnx_operation_mode=tenant.tnx_operation_mode,
                default_network=tenant.tnx_network_id,
                availability_zone=tenant.tnx_availability_zone
            )

        return initial


class MultiUpdateVDIProjectView(workflows.WorkflowView):
    workflow_class = project_workflows.MultiUpdateVDIProject

    def get_initial(self):

        initial = super(MultiUpdateVDIProjectView, self).get_initial()

        if is_project_quotas_disabled(self.request):
            self.workflow_class = (
                project_workflows.MultiUpdateVDIProjectNoQuota
            )

        request_tenants = self.request.GET.getlist('id', [])

        # Gather our tenants to correlate against IDs
        kwargs = get_tenants_kwargs(self.request)
        try:
            tenants, _has_more = api.keystone.tenant_list(
                self.request, **kwargs)
        except Exception as err:
            tenants = []
            logger.error(unicode(err))
            exceptions.handle(self.request,
                              _('Unable to retrieve projects list.'),
                              redirect=reverse(VDI_PROJECT_INDEX))
        else:
            tenants = {tenant.id: tenant.name for tenant in tenants
                       if tenant.id in request_tenants}
        initial['projects'] = tenants
        initial['domain_id'] = api.keystone.get_default_domain(
            self.request, get_name=False).id

        return initial


class CloneVDIProjectView(UpdateVDIProjectView):
    """Clone VDI project view"""
    workflow_class = project_workflows.CloneVDIProject

    def get_initial(self):
        initial = super(CloneVDIProjectView, self).get_initial()

        if 'name' in initial:
            initial['name'] = u'Clone of {0}'.format(initial['name'])

        return initial


class CreateVDIProjectNetworkView(workflows.WorkflowView):
    """Create VDI project network view"""
    workflow_class = CreateDefaultNetworkWorkflow
    ajax_template_name = 'vdi/_create_network.html'


class DetailVDIProjectView(projects_views.TnxDetailProjectView):
    """VDI project's details view"""
    tab_group_class = ProjectDetailTabs
    redirect_url = VDI_PROJECT_INDEX
    page_title = _("VDI Project Details: {{ project.name }}")

    def get_context_data(self, **kwargs):
        """Context data"""
        context = super(DetailVDIProjectView, self).get_context_data(**kwargs)
        project = self.get_data()
        table = VDIProjectsTable(self.request)
        context["project"] = project
        context["url"] = reverse(constants.VDI_PROJECT_INDEX)
        context["actions"] = table.render_row_actions(project)
        return context

    @memoized.memoized_method
    def get_data(self):
        """Gets project data"""
        project = None
        try:
            project = TNXTenantClient.get(self.request,
                                          self.kwargs['tenant_id'])
        except Exception as err:
            logger.error(unicode(err))
            exceptions.handle(self.request,
                              _('Unable to retrieve project details.'),
                              redirect=reverse(constants.VDI_PROJECT_INDEX))
        return project
