# coding: utf-8
# pylint: disable=no-member,super-on-old-class,non-parent-init-called
# pylint: disable=too-many-lines, duplicate-code

import copy
import json
import logging
import uuid
from collections import defaultdict

from django.core import validators
from django.forms import ValidationError
from django.template.defaultfilters import filesizeformat
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext_lazy as _

from horizon import exceptions
from horizon import forms
from horizon import messages
from horizon import workflows

from neutronclient.common import exceptions as neutron_exceptions
from novaclient import exceptions as nova_exceptions

from openstack_dashboard.api import (
    keystone, neutron, nova, cinder, base)
from openstack_dashboard import policy
from openstack_dashboard.dashboards.identity.projects import (
    workflows as proj_workflows)
from openstack_dashboard.dashboards.project.instances import (
    utils as instance_utils)
from openstack_dashboard.dashboards.project.networks import (
    workflows as network_workflows)
from openstack_dashboard.usage import quotas

from tionix_dashboard.clients.base import novaclient
from tionix_dashboard.clients.tenants import TNXTenantClient

from tionix_dashboard.dashboards.identity.projects.actions import (
    TnxUpdateProjectMembersAction)
from tionix_dashboard.dashboards.identity.projects.workflows import (
    post_remove_project_cache, CreateProjectQuota, UpdateProjectQuotaAction,
    UpdateProjectQuota)
from tionix_dashboard.dashboards.tionix.vdi.enums import OperationModeEnum
from tionix_dashboard.dashboards.tionix.vdi.exceptions import (
    UpdateVDIProjectNetworkException)
from tionix_dashboard.dashboards.tionix.vdi.projects.helpers import (
    ProjectNetworkMixin, get_network_field_data, get_image_field_data,
    get_availability_zone_data)
from tionix_dashboard.dashboards.utils import (
    get_available_image, is_domain_supported)
from tionix_dashboard.helpers.fields import AutoGenNameCharField
from tionix_dashboard.helpers.vdi_constants import VDI_PROJECT_TYPE
from tionix_dashboard.url_resolve_map import VDI_CREATE_NETWORK

logger = logging.getLogger(__name__)

PROJECT_GROUP_ENABLED = is_domain_supported()


class NetworkDynamicField(forms.DynamicChoiceField):
    def validate(self, value):
        if not value:
            super(NetworkDynamicField, self).validate(value)


class CreateProjectDefaultNetworkAction(workflows.Action):
    default_network = NetworkDynamicField(
        label=_("Default network"),
        add_item_link=VDI_CREATE_NETWORK)

    def populate_default_network_choices(self, request, context):
        return get_network_field_data(request, include_empty_option=True)

    class Meta(object):
        name = _("Default network parameters")
        help_text = _("Default network parameters")


class CreateProjectDefaultNetwork(workflows.Step):
    action_class = CreateProjectDefaultNetworkAction
    template_name = 'vdi/_edit_default_network.html'

    contributes = ('default_network', )


class NetworkObject(object):
    def __init__(self, object_id, name):
        super(NetworkObject, self).__init__()
        self.id = object_id
        self.name = name


class CreateSubnetInfoAction(network_workflows.CreateSubnetInfoAction):
    class Meta(object):
        name = _("Subnet")
        help_text = _('Creates a subnet associated with the network.'
                      ' You need to enter a valid "Network Address"'
                      ' and "Gateway IP". If you did not enter the'
                      ' "Gateway IP", the first value of a network'
                      ' will be assigned by default. If you do not want'
                      ' gateway please check the "Disable Gateway" checkbox.'
                      ' Advanced configuration is available by clicking on'
                      ' the "Subnet Details" tab.')

    def __init__(self, request, context, *args, **kwargs):
        super(CreateSubnetInfoAction, self).__init__(
            request, context, *args, **kwargs)
        self.fields.pop('with_subnet', None)
        self.context = context

    def clean(self):
        cleaned_data = super(CreateSubnetInfoAction, self).clean()
        with_subnet = self.context.get('with_subnet', False)
        if not with_subnet:
            return cleaned_data
        self._check_subnet_data(cleaned_data)
        return cleaned_data


class CreateSubnetInfo(network_workflows.CreateSubnetInfo):
    action_class = CreateSubnetInfoAction


class CreateNetwork(network_workflows.CreateNetwork):
    default_steps = (network_workflows.CreateNetworkInfo,
                     CreateSubnetInfo,
                     network_workflows.CreateSubnetDetail)


class CreateDefaultNetworkWorkflow(CreateNetwork):
    success_message = _('Network "%s" will be assigned '
                        'as default for project.')
    wizard = False

    def handle(self, request, data):
        temp_network_id = unicode(uuid.uuid4())
        self.object = NetworkObject(temp_network_id, data['net_name'])
        vdi_default_network = request.session.get('vdi_default_network', '{}')
        vdi_default_network = json.loads(vdi_default_network)
        vdi_default_network[temp_network_id] = data
        request.session['vdi_default_network'] = (
            json.dumps(vdi_default_network))
        return True


class UpdateVDIProjectMembersAction(TnxUpdateProjectMembersAction):
    def __init__(self, request, *args, **kwargs):
        super(UpdateVDIProjectMembersAction, self).__init__(request, *args,
                                                            **kwargs)
        project_id = self.initial.get('project_id')

        if project_id:
            try:
                users_roles = keystone.get_project_users_roles(request,
                                                               project_id)
            except Exception as err:
                exceptions.handle(request, 'Unable to retrieve user list. '
                                           'Please try again later.')
                logger.error(err)
                users_roles = {}
            for user_id in users_roles:
                roles_ids = users_roles[user_id]
                for role_id in roles_ids:
                    field_name = self.get_member_field_name(role_id)
                    self.fields[field_name].initial.append(user_id)
        else:
            admin_roles = None
            try:
                admin_roles = [role for role in keystone.role_list(request) if
                               role.name == 'admin']
            except Exception as err:
                logger.error(unicode(err))

            if admin_roles:
                field_name = self.get_member_field_name(admin_roles[0].id)
                self.fields[field_name].initial = [request.user.id]

    class Meta(object):
        name = _("Project Members")
        slug = proj_workflows.PROJECT_USER_MEMBER_SLUG

    def _validate_members(self, data):
        """Validate project members list

        :raises django.forms.ValidationError
        """

        user = self.request.user
        roles = keystone.role_list(self.request)
        for role in roles:
            field_name = self.get_member_field_name(role.id)
            user_ids = data.get(field_name)
            if user.id in user_ids:
                break
        else:
            raise ValidationError(
                _(u"User '{}' should be in project members list. ").format(
                    user.username)
            )

    def clean(self):
        data = super(UpdateVDIProjectMembersAction, self).clean()
        self._validate_members(data)
        return data


class UpdateVDIProjectMembersStep(proj_workflows.UpdateProjectMembers):
    action_class = UpdateVDIProjectMembersAction
    template_name = 'vdi/_manage_users.html'


class MultiUpdateVDIProjectMembersStep(proj_workflows.UpdateProjectMembers):
    depends_on = ('domain_id',)


class MultiUpdateVDIProjectGroupsStep(proj_workflows.UpdateProjectGroups):
    depends_on = ('domain_id',)


class CreateProjectDefaultAction(workflows.Action):
    flavor = forms.ChoiceField(
        label=_("Flavor"),
        help_text=_("Size of image to launch."))

    image_id = forms.ChoiceField(
        label=_("Image Name"),
        widget=forms.SelectWidget(
            data_attrs=('volume_size',),
            transform=lambda x: ("%s (%s)" % (x.name,
                                              filesizeformat(x.bytes)))))

    tnx_operation_mode = forms.ChoiceField(
        label=_("Operation mode"),
        choices=OperationModeEnum.get_choices())

    class Meta(object):
        name = _("Default parameters")
        help_text_template = "vdi/_flavors_and_quotas.html"
        help_text = _("Default parameters")

    def get_help_text(self, extra_context=None):
        extra = {} if extra_context is None else extra_context
        try:
            flavors = json.dumps(
                [f.to_dict() for f in instance_utils.flavor_list(self.request)]
            )
        except Exception as err:
            logger.error(unicode(err))
            exceptions.handle(self.request,
                              _("Unable to retrieve flavor information."))
        else:
            extra['flavors'] = flavors
            extra['resize_instance'] = True

        return super(CreateProjectDefaultAction, self).get_help_text(extra)

    def populate_flavor_choices(self, request, context):
        return instance_utils.flavor_field_data(request,
                                                include_empty_option=True)

    def populate_image_id_choices(self, request, context):
        return get_image_field_data(request, context)


class ProjectDefaultActionWithCheck(object):

    def __init__(self, request, initial, *args, **kwargs):
        assert isinstance(self, workflows.Action)
        super(ProjectDefaultActionWithCheck, self).__init__(
            request, initial, *args, **kwargs)

        if request.method == 'GET':
            self._check_tenant_defaults(initial)

    def _check_tenant_defaults(self, initial):
        """Resets initial if object is not available"""
        self.cleaned_data = getattr(self, 'cleaned_data', {})

        if 'flavor' in initial:
            flavor_id = initial.get('flavor')
            try:
                # flavors.get returns deleted flavors. Using get_keys()
                # to verify that flavor is not deleted
                nova.flavor_get(self.request, flavor_id).get_keys()
            except nova_exceptions.NotFound:
                self.add_error(
                    'flavor',
                    _(u"Flavor with id '{0}' not found. Please choose valid"
                      u" flavor.").format(flavor_id)
                )
                initial['flavor'] = ''

        if 'image_id' in initial:
            image_id = initial.get('image_id')
            if get_available_image(self.request, image_id) is None:
                self.add_error(
                    'image_id',
                    _(u"Image with id '{0}' not found. Please choose valid "
                      u"image.").format(image_id)
                )
                initial['image_id'] = ''


class UpdateProjectDefaultAction(ProjectDefaultActionWithCheck,
                                 CreateProjectDefaultAction):

    tnx_operation_mode = forms.ChoiceField(
        label=_("Operation mode"),
        choices=OperationModeEnum.get_choices(),
        widget=forms.Select(attrs={'disabled': 'disabled'})
    )

    class Meta(object):
        name = _("Default parameters")
        help_text_template = "vdi/_flavors_and_quotas.html"
        help_text = _("Default parameters")


class CloneProjectDefaultAction(ProjectDefaultActionWithCheck,
                                CreateProjectDefaultAction):

    class Meta(object):
        name = _("Default parameters")
        help_text_template = "vdi/_flavors_and_quotas.html"
        help_text = _("Default parameters")


class UpdateProjectDefaultNetworkAction(CreateProjectDefaultNetworkAction):
    class Meta(object):
        name = _("Default network parameters")
        help_text = _("Default network parameters")

    def __init__(self, request, initial, *args, **kwargs):
        super(UpdateProjectDefaultNetworkAction, self).__init__(
            request, initial, *args, **kwargs)

        try:
            networks = neutron.network_list_for_tenant(request,
                                                       initial['project_id'])
        except Exception as err:
            logger.error(unicode(err))
            exceptions.handle(request, _('Unable to retrieve VDI '
                                         'project information.'))
        else:
            networks = [(network.id, network.name) for network in networks]
            self.fields['default_network'].choices = networks
            self.fields['default_network'].choices.insert(
                0, ('', _('Select Network'))
            )
            if self.request.method == "GET":
                self._check_tenant_defaults(initial, networks)

    def _check_tenant_defaults(self, initial, networks):
        """Resets initial if object is not available"""
        self.cleaned_data = getattr(self, 'cleaned_data', {})

        if 'default_network' in initial:
            default_network = initial.get('default_network')
            try:
                neutron.network_get(self.request, default_network)

            except neutron_exceptions.NotFound as err:
                logger.error(unicode(err))
                self.add_error(
                    'default_network',
                    _(u"Network with id '{0}' not found. Please choose valid"
                      u" network.").format(default_network)
                )
                initial['default_network'] = ''
            else:
                if default_network not in zip(*networks)[0]:
                    self.add_error(
                        'default_network',
                        _(u"Network with id '{0}' is not available. Please "
                          u"choose a valid network.").format(default_network)
                    )


class UpdateProjectDefaultNetwork(CreateProjectDefaultNetwork):
    action_class = UpdateProjectDefaultNetworkAction
    template_name = 'vdi/_edit_default_network.html'


class CreateProjectDefault(workflows.Step):
    action_class = CreateProjectDefaultAction

    contributes = ("flavors", "flavor", "image_id", "tnx_operation_mode",)


class CloneProjectDefault(CreateProjectDefault):
    action_class = CloneProjectDefaultAction


class UpdateProjectDefault(CreateProjectDefault):
    action_class = UpdateProjectDefaultAction


class CreateProjectInfoAction(workflows.Action):
    # Hide the domain_id and domain_name by default
    domain_id = forms.CharField(label=_("Domain ID"),
                                required=False,
                                widget=forms.HiddenInput())
    domain_name = forms.CharField(label=_("Domain Name"),
                                  required=False,
                                  widget=forms.HiddenInput())
    availability_zone = forms.ChoiceField(label=_("Availability Zone"),
                                          required=False)
    name = AutoGenNameCharField(label=_("Name"), max_length=64)
    description = forms.CharField(
        widget=forms.widgets.Textarea(attrs={'rows': 4}),
        label=_("Description"),
        required=False)

    def __init__(self, request, *args, **kwargs):
        super(CreateProjectInfoAction, self).__init__(request, *args, **kwargs)
        # For keystone V3, display the two fields in read-only
        if keystone.VERSIONS.active >= 3:
            readonly_input = forms.TextInput(attrs={'readonly': 'readonly'})
            self.fields["domain_id"].widget = readonly_input
            self.fields["domain_name"].widget = readonly_input

    class Meta(object):
        name = _("Project Information")
        help_text = _("Create a project to organize users.")

    def _get_projects_by_name(self, project_name):
        kwargs = {
            'filters': {
                'name': project_name
            }
        }
        if policy.check((("identity", "identity:list_projects"),),
                        self.request):
            kwargs.update(
                {'domain': keystone.get_effective_domain_id(self.request)}
            )
        elif policy.check((("identity", "identity:list_user_projects"),),
                          self.request):
            kwargs.update({
                'user': self.request.user.id,
                'admin': False
            })
        return TNXTenantClient.list(self.request, **kwargs)

    def clean_name(self):
        project_name = self.cleaned_data['name']
        projects = self._get_projects_by_name(project_name)
        if projects:
            failure_message = _(u'Project name "{0}" is already used.'
                                u'').format(project_name)
            raise ValidationError(failure_message)
        return project_name

    def populate_availability_zone_choices(self, request, context):
        return get_availability_zone_data(request)


class CreateProjectInfo(proj_workflows.CreateProjectInfo):
    action_class = CreateProjectInfoAction
    contributes = ("domain_id", "domain_name", "project_id", "name",
                   "description", "enabled", "availability_zone")


class UpdateProjectInfoAction(CreateProjectInfoAction):
    enabled = forms.BooleanField(required=False, label=_("Enabled"))

    def __init__(self, request, initial, *args, **kwargs):
        super(UpdateProjectInfoAction, self).__init__(
            request, initial, *args, **kwargs)
        if initial['project_id'] == request.user.project_id:
            self.fields['enabled'].widget.attrs['disabled'] = True
            self.fields['enabled'].help_text = _(
                'You cannot disable your current project')

        if request.method == 'GET':
            self._check_tenant_defaults(initial)

    def clean(self):
        cleaned_data = super(UpdateProjectInfoAction, self).clean()
        # NOTE(tsufiev): in case the current project is being edited, its
        # 'enabled' field is disabled to prevent changing the field value
        # which is always `True` for the current project (because the user
        # logged in it). Since Django treats disabled checkbox as providing
        # `False` value even if its initial value is `True`, we need to
        # restore the original `True` value of 'enabled' field here.
        if self.fields['enabled'].widget.attrs.get('disabled', False):
            cleaned_data['enabled'] = True
        return cleaned_data

    def clean_name(self):
        project_name = self.cleaned_data['name']
        project_id = self.initial.get('project_id')
        projects = self._get_projects_by_name(project_name)
        if projects and projects[0].id != project_id:
            failure_message = _(u'Project name "{0}" is already used.'
                                u'').format(project_name)
            raise ValidationError(failure_message)
        return project_name

    class Meta(object):
        name = _("Project Information")
        slug = 'update_info'
        help_text = _("Edit the project details.")

    def _check_tenant_defaults(self, initial):
        """Resets initial if object is not available"""
        self.cleaned_data = getattr(self, 'cleaned_data', {})

        availability_zone = initial.get('availability_zone')
        if availability_zone:
            availability_zone_valid = False
            zones = get_availability_zone_data(self.request)
            if zones:
                zone_names = zip(*zones)[0]
                availability_zone_valid = availability_zone in zone_names

            if not availability_zone_valid:
                self.add_error(
                    'availability_zone',
                    _(u"Availability zone is not available. "
                      u"Please choose valid availability zone.")
                )
                initial['availability_zone'] = ''


class UpdateProjectInfo(proj_workflows.UpdateProjectInfo):
    action_class = UpdateProjectInfoAction
    contributes = ("domain_id", "domain_name", "availability_zone", "name",
                   "description", "enabled")


class CreateVDIProject(ProjectNetworkMixin,
                       proj_workflows.CreateProject):
    success_message = _('Created new project "%s".')
    failure_message = _('Error creating project "%s".')
    success_url = "horizon:tionix:vdi:index"
    name = _("Create VDI Project")

    default_steps = (CreateProjectInfo,
                     CreateProjectDefault,
                     CreateProjectDefaultNetwork,
                     UpdateVDIProjectMembersStep,
                     CreateProjectQuota)

    # pylint: disable=super-init-not-called
    def __init__(self, request=None, context_seed=None, entry_point=None,
                 *args, **kwargs):

        if PROJECT_GROUP_ENABLED:
            self.default_steps = (CreateProjectInfo,
                                  CreateProjectDefault,
                                  CreateProjectDefaultNetwork,
                                  UpdateVDIProjectMembersStep,
                                  proj_workflows.UpdateProjectGroups,
                                  CreateProjectQuota)
        workflows.Workflow.__init__(self, request=request,
                                    context_seed=context_seed,
                                    entry_point=entry_point,
                                    *args,
                                    **kwargs)

    @post_remove_project_cache
    def handle(self, request, data):
        saving_network_id = data['default_network']
        vdi_default_networks = request.session.pop('vdi_default_network', None)

        nova_client = novaclient(request)

        data['domain'] = data.pop('domain_id')
        data.pop('domain_name')
        data['project_type'] = VDI_PROJECT_TYPE
        data['network_id'] = None
        data['flavor_id'] = data.pop('flavor', None)

        try:
            response = nova_client.tnx_tenants.create(**data)
        except nova_exceptions.Conflict:
            self.failure_message = _('Project name "{0}" is already '
                                     'used.').format(data['name'])
            result = False
        except Exception as err:
            logger.error(unicode(err))
            result = False
        else:
            _code, tenant = response
            tenant = tenant['tenant']
            tenant_id = tenant['id']

            self._update_project_members(request, data, tenant_id)
            if PROJECT_GROUP_ENABLED:
                self._update_project_groups(request, data, tenant_id)
            if keystone.is_domain_admin(request):
                self._update_project_quota(request, data, tenant_id)

            network_id = self._get_network_id(
                request,
                network_id=saving_network_id,
                network_list=vdi_default_networks,
                tenant_id=tenant_id,
                tnx_tenant_name=data['name'])

            if network_id:
                result = self._update_tenant_network(request,
                                                     novaclient=nova_client,
                                                     tenant_id=tenant_id,
                                                     network_id=network_id)
            else:
                result = False
                exceptions.handle(
                    request,
                    _(
                        'Unable to create network for VDI project "{0}".'
                    ).format(data['name']))

        return result


class CreateVDIProjectNoQuota(CreateVDIProject):

    default_steps = (CreateProjectInfo,
                     CreateProjectDefault,
                     CreateProjectDefaultNetwork,
                     UpdateVDIProjectMembersStep)

    # pylint: disable=super-init-not-called
    def __init__(self, request=None, context_seed=None, entry_point=None,
                 *args, **kwargs):

        if PROJECT_GROUP_ENABLED:
            self.default_steps = (CreateProjectInfo,
                                  CreateProjectDefault,
                                  CreateProjectDefaultNetwork,
                                  UpdateVDIProjectMembersStep,
                                  proj_workflows.UpdateProjectGroups)

        workflows.Workflow.__init__(self, request=request,
                                    context_seed=context_seed,
                                    entry_point=entry_point,
                                    *args,
                                    **kwargs)


class UpdateVDIProject(ProjectNetworkMixin,
                       proj_workflows.UpdateProject):
    success_message = _('Project "%s" has been updated.')
    failure_message = _('Error updating project "%s".')

    success_url = "horizon:tionix:vdi:index"
    name = _("Update VDI Project")

    if PROJECT_GROUP_ENABLED:
        default_steps = (UpdateProjectInfo,
                         UpdateProjectDefault,
                         UpdateProjectDefaultNetwork,
                         UpdateVDIProjectMembersStep,
                         proj_workflows.UpdateProjectGroups,
                         UpdateProjectQuota)
    else:
        default_steps = (proj_workflows.UpdateProjectInfo,
                         UpdateProjectDefault,
                         UpdateProjectDefaultNetwork,
                         UpdateVDIProjectMembersStep,
                         UpdateProjectQuota)

    # pylint: disable=super-init-not-called
    def __init__(self, request=None, context_seed=None, entry_point=None,
                 *args, **kwargs):
        workflows.Workflow.__init__(self,
                                    request=request,
                                    context_seed=context_seed,
                                    entry_point=entry_point,
                                    *args,
                                    **kwargs)

    def get_absolute_url(self):
        return self.request.get_full_path()

    # pylint: disable=too-many-arguments
    def _update_vdi_project(self, request, nova_client, tenant_id,
                            vdi_default_networks, default_network_id,
                            tenant_data, data):

        tenant = nova_client.tnx_tenants.update(tenant_id, **tenant_data)

        self._update_project_members(request, data, tenant_id)
        if PROJECT_GROUP_ENABLED:
            self._update_project_groups(request,
                                        data,
                                        tenant_id,
                                        data['domain_id'])
        if keystone.is_domain_admin(request):
            self._update_project_quota(request, data, tenant_id)

        if default_network_id:
            network_id = self._get_network_id(
                request,
                network_id=default_network_id,
                network_list=vdi_default_networks,
                tenant_id=tenant_id,
                tnx_tenant_name=tenant.name)

            if network_id:
                self._update_tenant_network(
                    request,
                    novaclient=nova_client,
                    tenant_id=tenant_id,
                    network_id=network_id)
            else:
                raise UpdateVDIProjectNetworkException()

    @post_remove_project_cache
    def handle(self, request, data):
        vdi_default_networks = request.session.pop('vdi_default_network', None)

        nova_client = novaclient(request)

        data['enabled'] = unicode(data['enabled'])
        data['flavor_id'] = data.pop('flavor', None)
        tenant_data = copy.copy(data)
        tenant_data.pop('domain_id')
        tenant_data.pop('domain_name')
        tenant_id = tenant_data.pop('project_id')
        default_network_id = data.pop('default_network')

        result = True
        try:
            self._update_vdi_project(request,
                                     nova_client,
                                     tenant_id,
                                     vdi_default_networks,
                                     default_network_id,
                                     tenant_data,
                                     data)
        except nova_exceptions.Conflict:
            self.failure_message = _('Project name "{0}" is already '
                                     'used.').format(data['name'])
            result = False
        except UpdateVDIProjectNetworkException:
            messages.info(request,
                          _('Unable to create network for VDI project '
                            '"{0}".').format(data['name']))
        except Exception as err:
            logger.error(unicode(err))
            result = False

        return result


class OptionalFieldsActionMixin(object):
    """Makes all fields optional"""

    def __init__(self, request, context, *args, **kwargs):
        assert isinstance(self, workflows.Action)
        super(OptionalFieldsActionMixin, self).__init__(request,
                                                        context,
                                                        *args,
                                                        **kwargs)
        for field in self.fields.values():
            field.required = False


class MultiUpdateProjectInfoAction(workflows.Action):

    description = forms.CharField(
        widget=forms.widgets.Textarea(attrs={'rows': 4}),
        label=_("Description"),
        required=False)
    enabled = forms.BooleanField(label=_("Enabled"),
                                 required=False,
                                 initial=True)

    def clean(self):
        cleaned_data = super(MultiUpdateProjectInfoAction, self).clean()
        projects = self.initial['projects']
        current_project_id = self.request.user.tenant_id
        if current_project_id in projects and not cleaned_data['enabled']:
            raise forms.ValidationError(
                _('Current project "{0}" is selected. You cannot disable your '
                  'current project.').format(projects[current_project_id])
            )
        return cleaned_data

    class Meta(object):
        name = _("Project Information")
        slug = 'update_info'
        help_text = _("Edit the project details.")


class MultiUpdateProjectInfo(proj_workflows.UpdateProjectInfo):
    action_class = MultiUpdateProjectInfoAction
    template_name = proj_workflows.COMMON_HORIZONTAL_TEMPLATE
    depends_on = ("projects",)
    contributes = ("description",
                   "enabled")


class MultiUpdateProjectDefaultAction(OptionalFieldsActionMixin,
                                      CreateProjectDefaultAction):

    tnx_operation_mode = None

    def populate_flavor_choices(self, request, context):
        return instance_utils.flavor_field_data(request, True)

    class Meta(object):
        name = _("Default parameters")
        help_text_template = "vdi/_flavors_and_quotas.html"
        help_text = _("Default parameters")


class MultiUpdateProjectDefault(CreateProjectDefault):
    action_class = MultiUpdateProjectDefaultAction


class MultiUpdateProjectDefaultNetworkAction(
        OptionalFieldsActionMixin,
        CreateProjectDefaultNetworkAction):

    def populate_default_network_choices(self, request, context):
        return get_network_field_data(request, include_empty_option=True)

    class Meta(object):
        name = _("Default network parameters")
        help_text = _("Default network parameters")


class MultiUpdateProjectDefaultNetwork(CreateProjectDefaultNetwork):
    action_class = MultiUpdateProjectDefaultNetworkAction


class MultiUpdateProjectQuotaAction(OptionalFieldsActionMixin,
                                    UpdateProjectQuotaAction):

    def clean(self):
        bad_projects = defaultdict(list)
        for project_id, project_name in self.initial['projects'].items():
            usages = quotas.tenant_quota_usages(self.request,
                                                tenant_id=project_id)
            # Validate the quota values before updating quotas.
            for key, value in self.cleaned_data.items():
                used = usages[key].get('used', 0)
                if value is not None and 0 <= value < used:
                    bad_projects[project_name].append(
                        _('{used} {key} used').format(
                            used=used, key=quotas.QUOTA_NAMES.get(key, key)
                        )
                    )
        if bad_projects:
            value_str = u''.join(
                u'<p><strong>{0}</strong>: '
                u'{1};</p>'.format(project, u', '.join(values))
                for project, values in bad_projects.items()
            )
            msg = _('Quota value(s) cannot be less than the current usage '
                    'value(s): {0}').format(value_str)
            raise forms.ValidationError(mark_safe(msg))
        return self.cleaned_data

    class Meta(object):
        name = _("Quota")
        slug = 'update_quotas'
        help_text = _("Set maximum quotas for the project.")
        permissions = ('openstack.roles.admin', 'openstack.services.compute')


class MultiUpdateProjectQuota(UpdateProjectQuota):
    action_class = MultiUpdateProjectQuotaAction
    depends_on = ("projects", )


class MultiUpdateVDIProject(UpdateVDIProject):
    success_message = _('Successfully updated projects: "{0}".')
    failure_message = _('Unable to update projects "{0}".')

    name = _("Update VDI Projects")

    if PROJECT_GROUP_ENABLED:
        default_steps = (MultiUpdateProjectInfo,
                         MultiUpdateProjectDefault,
                         MultiUpdateProjectDefaultNetwork,
                         MultiUpdateVDIProjectMembersStep,
                         MultiUpdateVDIProjectGroupsStep,
                         MultiUpdateProjectQuota)
    else:
        default_steps = (MultiUpdateProjectInfo,
                         MultiUpdateProjectDefault,
                         MultiUpdateProjectDefaultNetwork,
                         MultiUpdateVDIProjectMembersStep,
                         MultiUpdateProjectQuota)

    def _update_project_members(self, request, data, project_id):
        """Updates members without deleting existing"""
        # update project members
        users_to_modify = 0
        # Project-user member step
        member_step = self.get_step(proj_workflows.PROJECT_USER_MEMBER_SLUG)
        result = True
        try:
            # Get our role options
            available_roles = self._get_available_roles(request)
            # Get the users currently associated with this project so we
            # can diff against it.
            users_roles = keystone.get_project_users_roles(
                request, project=project_id)
            users_to_modify = len(users_roles)

            # TODO(bpokorny): The following lines are needed to make sure we
            # only modify roles for users who are in the current domain.
            # Otherwise, we'll end up removing roles for users who have roles
            # on the project but aren't in the domain.  For now, Horizon won't
            # support managing roles across domains.  The Keystone CLI
            # supports it, so we may want to add that in the future.
            all_users = keystone.user_list(request, domain=data['domain_id'])
            users_dict = {user.id: user.name for user in all_users}

            for user_id in users_roles.keys():
                # Don't remove roles if the user isn't in the domain
                if user_id not in users_dict:
                    users_to_modify -= 1
                    continue

                # Check if there have been any changes in the roles of
                # Existing project members.
                current_role_ids = list(users_roles[user_id])
                self._add_roles_to_users(
                    request, data, project_id, user_id, current_role_ids,
                    available_roles)
                users_to_modify -= 1

            # Grant new roles on the project.
            for role in available_roles:
                field_name = member_step.get_member_field_name(role.id)
                # Count how many users may be added for exception handling.
                users_to_modify += len(data[field_name])
            for role in available_roles:
                users_added = 0
                field_name = member_step.get_member_field_name(role.id)
                for user_id in data[field_name]:
                    if user_id not in users_roles:
                        keystone.add_tenant_user_role(request,
                                                      project=project_id,
                                                      user=user_id,
                                                      role=role.id)
                    users_added += 1
                users_to_modify -= users_added
        except Exception as err:
            logger.error(unicode(err))
            group_msg = (
                _(", update project groups") if PROJECT_GROUP_ENABLED else ''
            )
            exceptions.handle(request,
                              _('Failed to modify %(users_to_modify)s'
                                ' project members%(group_msg)s and '
                                'update project quotas.')
                              % {'users_to_modify': users_to_modify,
                                 'group_msg': group_msg})
            result = False
        return result

    def _update_project_groups(self, request, data, project_id, domain_id):
        """Updates groups without deleting existing"""
        # update project groups
        groups_to_modify = 0
        member_step = self.get_step(proj_workflows.PROJECT_GROUP_MEMBER_SLUG)
        result = True
        try:
            available_roles = self._get_available_roles(request)
            # Get the groups currently associated with this project so we
            # can diff against it.
            project_groups = keystone.group_list(request,
                                                 domain=domain_id,
                                                 project=project_id)
            groups_to_modify = len(project_groups)
            for group in project_groups:
                # Check if there have been any changes in the roles of
                # Existing project members.
                current_roles = keystone.roles_for_group(
                    self.request,
                    group=group.id,
                    project=project_id)
                current_role_ids = [role.id for role in current_roles]
                for role in available_roles:
                    # Check if the group is in the list of groups with
                    # this role.
                    field_name = member_step.get_member_field_name(role.id)
                    if group.id in data[field_name]:
                        # Add it if necessary
                        if role.id not in current_role_ids:
                            # group role has changed
                            keystone.add_group_role(
                                request,
                                role=role.id,
                                group=group.id,
                                project=project_id)
                groups_to_modify -= 1

            # Grant new roles on the project.
            for role in available_roles:
                field_name = member_step.get_member_field_name(role.id)
                # Count how many groups may be added for error handling.
                groups_to_modify += len(data[field_name])
            for role in available_roles:
                groups_added = 0
                field_name = member_step.get_member_field_name(role.id)
                for group_id in data[field_name]:
                    if not [x for x in project_groups if group_id == x.id]:
                        keystone.add_group_role(request,
                                                role=role.id,
                                                group=group_id,
                                                project=project_id)
                    groups_added += 1
                groups_to_modify -= groups_added
        except Exception as err:
            logger.error(unicode(err))
            exceptions.handle(request,
                              _('Failed to modify %s project '
                                'members, update project groups '
                                'and update project quotas.')
                              % groups_to_modify)
            result = False
        return result

    def _update_project_quota(self, request, data, project_id):
        """Updates only filled quotas"""
        # Update the project quota.
        nova_data = dict([(key, data[key])
                          for key in quotas.NOVA_QUOTA_FIELDS if key in data])
        nova.tenant_quota_update(request, project_id, **nova_data)

        if cinder.is_volume_service_enabled(request):
            cinder_data = dict([(key, data[key])
                                for key in quotas.CINDER_QUOTA_FIELDS
                                if key in data])
            cinder.tenant_quota_update(request,
                                       project_id,
                                       **cinder_data)

        if (base.is_service_enabled(request, 'network') and
                neutron.is_quotas_extension_supported(request)):
            disabled_quotas = quotas.get_disabled_quotas(request)
            neutron_data = dict(
                [(key, data[key])
                 for key in quotas.NEUTRON_QUOTA_FIELDS
                 if key in data and key not in disabled_quotas]
            )

            neutron.tenant_quota_update(request, project_id, **neutron_data)

    @post_remove_project_cache
    def handle(self, request, data):
        def _check_data(key, value):
            return not all([
                value in validators.EMPTY_VALUES,
                not key.startswith(proj_workflows.PROJECT_USER_MEMBER_SLUG),
                not key.startswith(proj_workflows.PROJECT_GROUP_MEMBER_SLUG)])

        vdi_default_networks = request.session.pop('vdi_default_network', None)

        nova_client = novaclient(request)

        data['enabled'] = unicode(data['enabled'])
        data['flavor_id'] = data.pop('flavor', None)
        default_network_id = data.pop('default_network', None)
        # Remove all empty fields
        data = {key: value for key, value in data.items()
                if _check_data(key, value)}
        tenant_data = copy.copy(data)
        tenant_data.pop('domain_id')

        success, network_failed, failed = [], [], []
        for tenant_id, name in self.context_seed['projects'].items():

            try:
                self._update_vdi_project(request,
                                         nova_client,
                                         tenant_id,
                                         vdi_default_networks,
                                         default_network_id,
                                         tenant_data,
                                         data)
                success.append(name)
            except UpdateVDIProjectNetworkException:
                network_failed.append(name)
            except Exception as err:
                logger.error(unicode(err))
                failed.append(name)

        result = True
        if success:
            self.success_message = self.success_message.format(
                '", "'.join(success)
            )

        if failed:
            result = False
            self.failure_message = self.failure_message.format(
                '", "'.join(failed)
            )

        if network_failed:
            messages.info(
                request,
                _('Unable to create network for VDI '
                  'projects: "{0}".').format('", "'.join(failed)))

        return result


class CloneVDIProject(CreateVDIProject):
    success_message = _('Project "%s" has been cloned.')
    failure_message = _('Error cloning project "%s".')

    success_url = "horizon:tionix:vdi:index"
    name = _("Clone VDI Project")

    finalize_button_name = _("Clone Project")

    default_steps = (CreateProjectInfo,
                     CloneProjectDefault,
                     UpdateProjectDefaultNetwork,
                     proj_workflows.UpdateProjectMembers,
                     CreateProjectQuota)

    # pylint: disable=super-init-not-called
    def __init__(self, request=None, context_seed=None, entry_point=None,
                 *args, **kwargs):
        if PROJECT_GROUP_ENABLED:
            self.default_steps = (CreateProjectInfo,
                                  CloneProjectDefault,
                                  UpdateProjectDefaultNetwork,
                                  proj_workflows.UpdateProjectMembers,
                                  proj_workflows.UpdateProjectGroups,
                                  CreateProjectQuota)

        workflows.Workflow.__init__(self, request=request,
                                    context_seed=context_seed,
                                    entry_point=entry_point,
                                    *args,
                                    **kwargs)

    def handle(self, request, data):
        data['project_id'] = None
        return super(CloneVDIProject, self).handle(request, data)


class UpdateVDIProjectNoQuota(UpdateVDIProject):

    if PROJECT_GROUP_ENABLED:
        default_steps = (proj_workflows.UpdateProjectInfo,
                         UpdateProjectDefault,
                         UpdateProjectDefaultNetwork,
                         UpdateVDIProjectMembersStep,
                         proj_workflows.UpdateProjectGroups)
    else:
        default_steps = (proj_workflows.UpdateProjectInfo,
                         UpdateProjectDefault,
                         UpdateProjectDefaultNetwork,
                         UpdateVDIProjectMembersStep)


class MultiUpdateVDIProjectNoQuota(UpdateVDIProject):

    if PROJECT_GROUP_ENABLED:
        default_steps = (MultiUpdateProjectInfo,
                         MultiUpdateProjectDefault,
                         MultiUpdateProjectDefaultNetwork,
                         UpdateVDIProjectMembersStep,
                         proj_workflows.UpdateProjectGroups)
    else:
        default_steps = (MultiUpdateProjectInfo,
                         MultiUpdateProjectDefault,
                         MultiUpdateProjectDefaultNetwork,
                         UpdateVDIProjectMembersStep)
