# coding: utf-8
# pylint: disable=attribute-defined-outside-init

import logging

from django.utils.translation import ugettext_lazy as _

from horizon import exceptions, tabs

from openstack_dashboard import api

from tionix_dashboard.clients.exceptions import UrlNotAvailableError
from tionix_dashboard.clients.tenants import TNXTenantClient
from tionix_dashboard.dashboards.tionix.vdi.helpers import (
    get_tenants_kwargs, get_vdi_instances)
from tionix_dashboard.dashboards.tionix.vdi.instances import (
    tables as instances_tables
)
from tionix_dashboard.dashboards.tionix.vdi.projects import (
    tables as projects_tables
)
from tionix_dashboard.dashboards.utils import (is_domain_supported)
from tionix_dashboard.helpers.views import (
    ServerFilterTabMixin, DTVTnxPaginationMixin)

logger = logging.getLogger(__name__)


class VDIInstancesTab(ServerFilterTabMixin,
                      DTVTnxPaginationMixin, tabs.TableTab):
    """Tab for vdi instances"""
    table_classes = (instances_tables.VDIInstancesTable,)
    name = _('Instances')
    slug = 'instances_tab'
    template_name = 'horizon/common/_detail_table.html'
    preload = False

    def get_vdi_instances_data(self):
        instances = []
        offset, limit = self.get_offset_limit()
        search_opts = self.get_filters({})
        try:
            self.table.total_count = len(get_vdi_instances(
                self.request, search_opts, tenants_load=False))
        except Exception as err:
            logger.error(unicode(err))
            exceptions.handle(
                self.request, _('Unable to retrieve instance list.'))
        else:
            search_opts.update(sort_dirs=('asc',), sort_keys=('created_at',))
            instances = get_vdi_instances(self.request, search_opts)
            range_end = offset + limit
            instances = instances[offset:range_end]
            total = len(instances)

            if offset == 0:
                self._prev = False

            if (offset + total) == self.table.total_count:
                self._more = False

            self.table.first_marker, self.table.last_marker = (
                0, self.calc_last_marker(self.table.total_count, limit)
            )

        return instances


class VDIProjectsTab(ServerFilterTabMixin, tabs.TableTab):
    """Tab for vdi projects"""
    table_classes = (projects_tables.VDIProjectsTable,)
    name = _('Projects')
    slug = 'projects_tab'
    template_name = 'horizon/common/_detail_table.html'
    preload = False

    # pylint: disable=protected-access
    def get_vdi_projects_data(self):
        tenants, self._more = [], False

        marker = self.request.GET.get(self.table._meta.pagination_param, None)

        kwargs = {
            'paginate': True,
            'filters': self.get_filters()
        }

        kwargs.update(get_tenants_kwargs(self.request))
        try:
            tenants, _more = TNXTenantClient.get_paginator(
                self.request,
                marker,
                is_vdi=True,
                **kwargs)
        except UrlNotAvailableError:
            raise
        except Exception as err:
            logger.error(unicode(err))
            exceptions.handle(self.request,
                              _("Unable to retrieve project list."))

        if is_domain_supported():
            domain_lookup = api.keystone.domain_lookup(self.request)
            if domain_lookup:
                for tenant in tenants:
                    tenant.domain_name = domain_lookup.get(tenant.domain_id)

        return tenants


class VDITabs(tabs.TabGroup):
    """Tab group for vdi"""
    slug = 'vdi_tab_group'
    tabs = (VDIInstancesTab, VDIProjectsTab)
    sticky = True
