# coding: utf-8
# pylint: disable=no-member

import logging

from django import shortcuts
from django.core.urlresolvers import reverse_lazy
from django.utils.translation import ugettext_lazy as _

from horizon import messages, tables
from horizon.utils import functions

from openstack_dashboard import api

logger = logging.getLogger(__name__)


class MultiLinkAction(tables.LinkAction):
    """
    Link action for multiple objects
    """

    req_param_name = 'id'

    def get_link_url(self, datum=None):
        return "javascript:void(0);"

    @property
    def attr_url(self):
        return reverse_lazy(self.url)

    def get_default_attrs(self):
        """
        Returns a list of the default HTML attributes for the action
        """
        attrs = super(MultiLinkAction, self).get_default_attrs()
        attrs.update({'data-batch-action': 'true',
                      'url': self.attr_url,
                      'data-multi-action': 'true',
                      'data-req-param-name': self.req_param_name})
        return attrs


class TnxFilterAction(tables.FilterAction):
    """
    Filters by filter_field from table
    """

    field_get_data = None

    def filter(self, table, data, filter_string):
        filter_field = table.get_filter_field()
        if filter_field and not self.is_api_filter(filter_field):
            q = filter_string.lower()
            get_data = None
            # pylint: disable=unsupported-membership-test
            # pylint: disable=unsubscriptable-object
            if self.field_get_data and filter_field in self.field_get_data:
                get_data = self.field_get_data[filter_field]
            else:
                column = table.columns.get(filter_field)
                if column:
                    get_data = column.get_data

            def comp(obj):
                obj_data = (
                    get_data(obj)
                    if get_data else getattr(obj, filter_field, '')
                ) or ''
                if not isinstance(obj_data, basestring):
                    obj_data = unicode(obj_data)
                return q in obj_data.lower()

            data = filter(comp, data)

        return data


class TnxUpdateAction(tables.UpdateAction):
    """
    Base action for update cells strips trailing and leading
    whitespaces
    """

    def allowed(self, request, datum, cell):
        """
        check permission
        """
        return api.keystone.keystone_can_edit_project()

    def action(self, request, datum, obj_id, cell_name, new_cell_value):
        if isinstance(new_cell_value, basestring):
            new_cell_value = new_cell_value.strip()

        return super(TnxUpdateAction, self).action(
            request, datum, obj_id, cell_name, new_cell_value
        )


class TnxBatchActionMixin(object):
    """Tionix Batch Action with quotes in messages"""

    classes = []

    def set_danger_btn_class(self, flag=False):
        """Add or remove btn-danger class"""

        if flag and "btn-danger" not in self.classes:
            self.classes += ("btn-danger",)
        elif not flag and "btn-danger" in self.classes:
            self.classes = [c for c in self.classes if c != "btn-danger"]

    def handle(self, table, request, obj_ids):
        action_success = []
        action_failure = []
        action_not_allowed = []
        for datum_id in obj_ids:
            datum = table.get_object_by_id(datum_id)
            datum_display = table.get_object_display(datum) or datum_id
            if not table._filter_action(self, request, datum):
                action_not_allowed.append(datum_display)
                msg = u'Permission denied to %s: "%s"' % (
                    self._get_action_name(past=True).lower(), datum_display
                )
                logger.warning(msg)
                continue
            try:
                self.action(request, datum_id)
                # Call update to invoke changes if needed
                self.update(request, datum)
                action_success.append(datum_display)
                self.success_ids.append(datum_id)
                logger.info(u'%s: "%s"', self._get_action_name(past=True),
                            datum_display)
            except Exception as ex:
                # Handle the exception but silence it since we'll display
                # an aggregate error message later. Otherwise we'd get
                # multiple error messages displayed to the user.
                action_failure.append(datum_display)
                action_description = (
                    self._get_action_name(past=True).lower(), datum_display)
                logger.warning(
                    u'Action %(action)s Failed for %(reason)s',
                    {'action': action_description, 'reason': ex}
                )

        # Begin with success message class, downgrade to info if problems.
        success_message_level = messages.success
        if action_not_allowed:
            msg = _('You are not allowed to %(action)s: "%(objs)s"')
            params = {"action":
                      self._get_action_name(action_not_allowed).lower(),
                      "objs": functions.lazy_join('", "', action_not_allowed)}
            messages.error(request, msg % params)
            success_message_level = messages.info
        if action_failure:
            msg = _('Unable to %(action)s: "%(objs)s"')
            params = {"action": self._get_action_name(action_failure).lower(),
                      "objs": functions.lazy_join('", "', action_failure)}
            messages.error(request, msg % params)
            success_message_level = messages.info
        if action_success:
            msg = '%(action)s: "%(objs)s"'
            params = {"action":
                      self._get_action_name(action_success, past=True),
                      "objs": functions.lazy_join('", "', action_success)}
            success_message_level(request, msg % params)

        return shortcuts.redirect(self.get_success_url(request))
