# coding: utf-8
# pylint: disable=no-member
# pylint: disable=protected-access

import json
import logging
import six

from django.template.defaultfilters import truncatechars
from django.utils.translation import ugettext_lazy as _
from django.utils import http

from horizon import exceptions
from horizon import tables
from horizon.utils import memoized
from horizon.utils.functions import get_page_size

from tionix_dashboard.helpers.utils import get_table_filter_action

logger = logging.getLogger(__name__)


class DTTnxPaginationMixin(object):
    """
    Data table with offset/limit pagination
    """

    def _get_prev_marker(self):
        return self.request.GET.get(
            self._meta.prev_pagination_param, '')

    def _get_marker(self):
        return self.request.GET.get(
            self._meta.pagination_param, '')

    def get_prev_marker(self):
        """
        Returns the identifier for the first object in the current data set
        for APIs that use offset/limit-based paging.
        """
        page_size = get_page_size(self.request)
        previous_marker = self._get_prev_marker() or self._get_marker()

        prev_marker = (int(previous_marker) - page_size
                       if previous_marker.isdigit() else 0)

        return http.urlquote_plus(prev_marker)

    def get_marker(self):
        """
        Returns the identifier for the last object in the current data set
        for APIs that use offset/limit-based paging.
        """
        page_size = get_page_size(self.request)
        previous_marker = self._get_prev_marker() or self._get_marker()

        marker = (int(previous_marker) + page_size
                  if previous_marker.isdigit() else page_size)

        return http.urlquote_plus(marker)

    def get_checked_ids(self):
        object_ids = self.request.session.get('object_ids', [])
        return json.dumps([int(i) for i in object_ids if i.isdigit()])

    def get_object_by_id(self, obj_id):
        try:
            obj = super(DTTnxPaginationMixin, self).get_object_by_id(obj_id)
        except exceptions.Http302:
            obj = obj_id
        return obj


class DataTableFirstLastPagination(object):
    """
    Table with first,last pages in paginator
    """

    first_marker = None
    last_marker = None
    total_count = None

    def __init__(self, request, data=None, needs_form_wrapper=None, **kwargs):
        assert isinstance(self, tables.DataTable)
        super(DataTableFirstLastPagination, self).__init__(
            request, data=data, needs_form_wrapper=needs_form_wrapper,
            **kwargs)

        _cols = [col.attrs.update(**{'data-column': name})
                 for name, col in self.columns.iteritems()]

    def has_first_data(self):
        return self.has_prev_data() and self.first_marker is not None

    def has_last_data(self):
        return self.has_more_data() and self.last_marker is not None

    def get_first_marker(self):
        return http.urlquote_plus(self.first_marker)

    def get_last_marker(self):
        return http.urlquote_plus(self.last_marker)

    def get_first_pagination_string(self):
        pagination_string = '='.join([self._meta.prev_pagination_param,
                                      self.get_first_marker()])
        return self.join_extra_pagination_string(pagination_string)

    def get_last_pagination_string(self):
        pagination_string = '='.join([self._meta.pagination_param,
                                      self.get_last_marker()])
        return self.join_extra_pagination_string(pagination_string)

    def get_prev_pagination_string(self):
        pagination_string = super(DataTableFirstLastPagination,
                                  self).get_prev_pagination_string()
        return self.join_extra_pagination_string(pagination_string)

    def get_pagination_string(self):
        pagination_string = super(DataTableFirstLastPagination,
                                  self).get_pagination_string()
        return self.join_extra_pagination_string(pagination_string)

    def has_total_count(self):
        return self.total_count is not None

    def get_total_count(self):
        return self.total_count

    def get_sort_dir(self):
        return self.request.session.get('sort_dir')

    def get_sort_key(self):
        return self.request.session.get('sort_key')

    @memoized.memoized_method
    def get_extra_pagination_string(self):
        return ''

    def join_extra_pagination_string(self, pagination_string):
        extra_string = self.get_extra_pagination_string()
        if extra_string:
            pagination_string = '&'.join([pagination_string,
                                          extra_string])
        return pagination_string

    class Meta(object):
        template = '_data_table.html'


class ColoredRowMixin(object):

    STATUS_CLASSES = None

    def get_row_status_class(self, status):
        status_class = ''
        if self.STATUS_CLASSES:
            status_class = self.STATUS_CLASSES.get(status, '')
        return status_class


class InstancesFilterPaginationMixin(object):
    def get_filter_string(self):
        """
        Get the filter string value. For 'server' type filters this is
        saved in the session so that it gets persisted across table loads.
        For other filter types this is obtained from the POST dict.
        """
        filter_action = get_table_filter_action(self)
        filter_string = ''
        if filter_action:
            param_name = filter_action.get_param_name()
            if filter_action.filter_type == 'server':
                filter_string = (self.request.POST.get(param_name, '') or
                                 self.request.session.get(param_name, ''))
            else:
                filter_string = self.request.POST.get(param_name, '')
        return filter_string

    def get_filter_field(self):
        """
        Get the filter field value used for 'server' type filters. This
        is the value from the filter action's list of filter choices.
        """
        filter_action = get_table_filter_action(self)
        filter_field = ''
        if filter_action:
            param_name = '%s_field' % filter_action.get_param_name()
            filter_field = (self.request.POST.get(param_name, '') or
                            self.request.session.get(param_name, ''))
        return filter_field


class ServerFilterTabTableMixin(object):

    def get_full_url(self):
        tab_group = self.kwargs['tab_group']
        tab = tab_group.get_selected_tab()
        return '{0}?{1}'.format(self.request.get_full_path().split('?')[0],
                                tab.get_query_string())


class TnxColumn(tables.Column):
    """
    Table column with translation of values that were not found in
    display_choices
    """

    display_choices_dict = None

    def __init__(self, *args, **kwargs):
        super(TnxColumn, self).__init__(*args, **kwargs)
        if self.display_choices:
            self.display_choices_dict = {
                value.lower(): display
                for value, display in self.display_choices
            }

    def get_data(self, datum):
        """Returns the final display data for this column from the given
        inputs.

        The return value will be either the attribute specified for this column
        or the return value of the attr:`~horizon.tables.Column.transform`
        method for this column.
        """
        datum_id = self.table.get_object_id(datum)

        if datum_id in self.table._data_cache[self]:
            return self.table._data_cache[self][datum_id]

        data = self.get_raw_data(datum)
        display_value = None

        if self.display_choices_dict:
            data = data or ''
            data = self.display_choices_dict.get(data.lower(),
                                                 _(data.capitalize()))
        if not display_value:
            for filter_func in self.filters:
                try:
                    data = filter_func(data)
                except Exception as err:
                    logger.error(unicode(err))
                    msg = ("Filter '%(filter)s' failed with data "
                           "'%(data)s' on column '%(col_name)s'")
                    args = {'filter': filter_func.__name__,
                            'data': data,
                            'col_name': six.text_type(self.verbose_name)}
                    logger.warning(msg, args)

        if data and self.truncate:
            data = truncatechars(data, self.truncate)

        self.table._data_cache[self][datum_id] = data

        return self.table._data_cache[self][datum_id]
