# coding: utf-8

import re
from collections import OrderedDict

from django import shortcuts

from horizon.utils.functions import get_page_size

from tionix_dashboard.helpers.utils import get_table_filter_action


class FilterStringModifierMixin(object):
    """
    Modifiers filter string before filtering
    """
    escape_fields = [u'name', ]
    escape_enabled = True
    is_last_page = False

    def get_filters(self, filters=None, include_not_api=False):
        filters = filters or {}
        filter_action = get_table_filter_action(self.table)
        if filter_action:
            filter_field = self.table.get_filter_field()
            if filter_action.is_api_filter(filter_field) or include_not_api:
                filter_string = self.table.get_filter_string()
                drop_marker = self.update_actual_filter(
                    filter_field, filter_string)
                search_action = self.request.POST.get('search_action', [])
                if drop_marker and 'pagination' not in search_action:
                    filters['marker'] = None
                if (getattr(filter_action, 'escape_enabled', True) and
                        filter_field in self.escape_fields):
                    filter_string = re.escape(filter_string)
                if filter_field and filter_string:
                    filter_string_modifier = getattr(filter_action,
                                                     'filter_string_modifiers',
                                                     None)
                    if filter_string_modifier:
                        modifier = filter_action.filter_string_modifiers.get(
                            filter_field
                        )
                        if modifier is not None:
                            filter_string = modifier(filter_string)
                    filters[filter_field] = filter_string
        return filters

    def update_actual_filter(self, filter_field, filter_string):
        actual_filter_field = getattr(self.table._meta, 'filter_field', '')
        actual_filter_string = getattr(self.table._meta, 'filter_string', '')
        self.table._meta.filter_field = filter_field
        self.table._meta.filter_string = filter_string
        if (actual_filter_field != filter_field or
                actual_filter_string != filter_string):
            self.is_last_page = False
            return True


# pylint: disable=expression-not-assigned
class ServerSortMultiSelect(object):
    object_ids_param = 'object_ids'
    sort_dir_param = 'sort_dir'
    sort_key_param = 'sort_key'
    redirect_param_name = 'is_redirect'

    params_tuple = (
        object_ids_param,
        sort_dir_param,
        sort_key_param
    )

    def handler_server_params(self, request):
        if (request.method == 'POST' or
                self.redirect_param_name in request.session):
            session_dict = {i: request.POST.get(i)
                            for i in self.params_tuple[1:]
                            if i in request.POST}
            object_ids = request.POST.getlist(self.object_ids_param)
            if object_ids:
                session_dict.update(**{self.object_ids_param: object_ids})

            request.session.update(session_dict)

        else:
            [request.session.pop(i)
             for i in self.params_tuple if i in request.session]

        if self.redirect_param_name in request.session:
            request.session.pop(self.redirect_param_name)

    # pylint: disable=no-member
    def get(self, request, *args, **kwargs):
        self.handler_server_params(request)

        _super = super(ServerSortMultiSelect, self)
        if hasattr(_super, 'get'):
            return _super.get(request, *args, **kwargs)


class DTVTnxPaginationMixin(ServerSortMultiSelect):
    """
    Data table view mixin for offset/limit pagination
    """
    _more = True
    _prev = True

    def has_prev_data(self, table):
        return self._prev

    def has_more_data(self, table):
        return self._more

    def get_offset_limit(self):

        limit = get_page_size(self.request)

        marker = self.request.GET.get(
            self.table._meta.prev_pagination_param, '')

        if not marker:
            marker = self.request.GET.get(
                self.table._meta.pagination_param, '')

        offset = int(marker) if marker.isdigit() else 0

        return offset, limit

    @staticmethod
    def calc_last_marker(total, page_size):
        total_pages = total / page_size
        if total % page_size == 0:
            total_pages -= 1
        return total_pages * page_size

    def post(self, request, *args, **kwargs):
        # If the server side table filter changed then go back to the first
        # page of data. Otherwise GET and POST handling are the same.
        # FIXME: why TnxInstancesTab not have handle_server_filter
        if (hasattr(self, 'handle_server_filter') and
                self.handle_server_filter(request)):
            self.handler_server_params(request)
            request.session[self.redirect_param_name] = True
            return shortcuts.redirect(self.get_table().get_absolute_url())
        return self.get(request, *args, **kwargs)


class ServerFilterTabMixin(FilterStringModifierMixin):
    """
    Server filter for tables in tab
    """

    table_classes = None

    def __init__(self, tab_group, request):
        super(ServerFilterTabMixin, self).__init__(tab_group, request)
        if not self.table_classes:
            class_name = self.__class__.__name__
            raise NotImplementedError("You must define a table_class "
                                      "attribute on %s" % class_name)
        # Instantiate our table classes but don't assign data yet
        kwargs = tab_group.kwargs
        kwargs.update(tab_group=tab_group)
        table_instances = [(table._meta.name, table(request, **kwargs))
                           for table in self.table_classes]
        self._tables = OrderedDict(table_instances)
        self._table_data_loaded = False

    @property
    def table(self):
        return self._tables.values()[0]

    def post(self, request, *args, **kwargs):
        # If the server side table filter changed then go back to the first
        # page of data. Otherwise GET and POST handling are the same.
        if self.handle_server_filter(request):
            return shortcuts.redirect(self.table.get_absolute_url())
        super(ServerFilterTabMixin, self).post(request, *args, **kwargs)

    def get_server_filter_info(self, request):
        filter_action = get_table_filter_action(self.table)
        if filter_action is None or filter_action.filter_type != 'server':
            return None
        param_name = filter_action.get_param_name()
        filter_string = request.POST.get(param_name)
        filter_string_session = request.session.get(param_name, "")
        changed = (filter_string is not None and
                   filter_string != filter_string_session)
        if filter_string is None:
            filter_string = filter_string_session
        filter_field_param = param_name + '_field'
        filter_field = request.POST.get(filter_field_param)
        filter_field_session = request.session.get(filter_field_param)
        if filter_field is None and filter_field_session is not None:
            filter_field = filter_field_session
        filter_info = {
            'action': filter_action,
            'value_param': param_name,
            'value': filter_string,
            'field_param': filter_field_param,
            'field': filter_field,
            'changed': changed
        }
        return filter_info

    def handle_server_filter(self, request):
        """
        Update the table server filter information in the session and
        determine if the filter has been changed.
        """
        filter_info = self.get_server_filter_info(request)
        if filter_info is None:
            return False
        request.session[filter_info['value_param']] = filter_info['value']
        if filter_info['field_param']:
            request.session[filter_info['field_param']] = filter_info['field']
        return filter_info['changed']

    def update_server_filter_action(self):
        """
        Update the table server side filter action based on the current
        filter. The filter info may be stored in the session and this will
        restore it.
        """
        filter_info = self.get_server_filter_info(self.request)
        if filter_info is not None:
            action = filter_info['action']
            setattr(action, 'filter_string', filter_info['value'])
            if filter_info['field_param']:
                setattr(action, 'filter_field', filter_info['field'])

    def get_context_data(self, request, **kwargs):
        self.update_server_filter_action()
        return super(ServerFilterTabMixin, self).get_context_data(request,
                                                                  **kwargs)


class ServerFilterViewMixin(object):
    """
    Server filter for tabbedviews
    """
    def post(self, request, *args, **kwargs):
        if 'search_action' in request.POST:
            if 'tab' in request.GET:
                targetslug = request.GET['tab'].split('__')[1]
            else:
                targetslug = None
            tabs = self.get_tabs(self.request, **self.kwargs).get_tabs()
            for tab in tabs:
                if targetslug is None or tab.slug == targetslug:
                    tab.post(request, *args, **kwargs)
                    break

        return super(ServerFilterViewMixin, self).post(request,
                                                       *args, **kwargs)


class ServerPaginationMixin(object):
    """
    Pagination in instance view
    """
    _prev = False
    _more = False
    instances = []
    search_opts = {}
    sort_dir = 'desc'
    is_last_page = False

    def has_more_data(self, table):
        return self._more

    def has_prev_data(self, table):
        return self._prev

    def marker_init(self):
        prev_marker = self.request.GET.get(
            self.table._meta.prev_pagination_param, None)

        first_marker = last_marker = 0
        marker = None
        if prev_marker is not None:
            if prev_marker != str(last_marker):
                marker = prev_marker
                self.sort_dir = 'asc'
        else:
            marker = self.request.GET.get(
                self.table._meta.pagination_param, None)
            if marker == str(last_marker):
                marker = None
                self.sort_dir = 'asc'
                self.is_last_page = True

        self.search_opts = self.get_filters({'marker': marker,
                                             'paginate': True})

        self.table.first_marker = first_marker
        self.table.last_marker = last_marker

    def post(self, request, *args, **kwargs):
        # redirect if search field reset
        # escape redirect if pagination use
        if 'search_action' in request.POST:
            search_action = request.POST['search_action']
            if (search_action == 'restart' or
                    (search_action == 'button' and
                     self.handle_server_filter(request))):
                return shortcuts.redirect(self.get_table().get_absolute_url())

        return self.get(request, *args, **kwargs)


class RefererRedirectMixin(object):
    """View redirect to http_referer from request"""

    def get_redirect_url(self):
        return self.request.META['HTTP_REFERER']

    def get_success_url(self):
        return self.get_redirect_url()
