# coding: utf-8

from collections import defaultdict
import logging.config
import os
import pkg_resources
import horizon
import openstack_auth

from django.core.urlresolvers import reverse_lazy

from tionix_dashboard.settings_handler import (
    update_module_settings, update_global_from_settings)

TNX_DASH_BASE_DIR = os.path.dirname(__file__)
TIONIX_DOCUMENTATION_LINK = 'http://docs.tionix.ru/'
TIONIX_DOCUMENTATION_DASH_SECTION = ('/tionix_modules/tionix-dash-docs/'
                                     'tionix_dashboard/functional.html')


# pylint: disable=no-member
def get_help_url():
    help_url = TIONIX_DOCUMENTATION_LINK
    dash_version = None

    try:
        dash_distribution = pkg_resources.get_distribution('tionix-dashboard')
        dash_version = dash_distribution.version
    except Exception:
        pass

    if dash_version is not None:
        dash_version = dash_version.split('.dev')[0]

        if not dash_version.endswith('0'):
            dash_version = '.'.join(dash_version.split('.')[:2])

        help_url = help_url + dash_version + TIONIX_DOCUMENTATION_DASH_SECTION

    return help_url

# Enable Tionix theme
try:
    from tionix_dashboard_theme import *
except ImportError:
    pass

LOG_LEVEL = 'INFO'

ADD_INSTALLED_APPS = [
    'tionix_dashboard',
    'tionix_dashboard.dashboards.project.project_images',
    'tionix_dashboard.dashboards.project.project_instances',
    'tionix_dashboard.dashboards.project.network_topology',
    'tionix_dashboard.dashboards.project.project_networks',
    'tionix_dashboard.dashboards.project.project_volumes',
    'tionix_dashboard.dashboards.admin.aggregates',
    'tionix_dashboard.dashboards.admin.flavors',
    'tionix_dashboard.dashboards.admin.hypervisors',
    'tionix_dashboard.dashboards.admin.instances',
    'tionix_dashboard.dashboards.admin.volumes',
    'tionix_dashboard.dashboards.admin.images',
    'tionix_dashboard.dashboards.admin.networks',
    'tionix_dashboard.dashboards.identity.domains',
    'tionix_dashboard.dashboards.identity.projects',
    'tionix_dashboard.dashboards.identity.users',
    'tionix_dashboard.dashboards.settings.user',
    'tionix_dashboard.dashboards.tionix.balancing',
    'tionix_dashboard.dashboards.tionix.licenses',
    'tionix_dashboard.dashboards.tionix.power_devices',
    'tionix_dashboard.dashboards.tionix.scheduler',
    'tionix_dashboard.dashboards.tionix.metrics',
    'tionix_dashboard.dashboards.tionix.vdi',
    'tionix_dashboard.dashboards.tionix.frames',
    'tionix_dashboard.management.commands.collectstatic',
]

# Time out for checking licenses
MAX_LICENSE_TIME_OUT = 60

OPENSTACK_AUTH_BASE_DIR = os.path.dirname(openstack_auth.__file__)

HORIZON_BASE_DIR = os.path.dirname(horizon.__file__)

LOCALE_PATHS = [
    os.path.join(TNX_DASH_BASE_DIR, 'locale'),
    os.path.join(OPENSTACK_AUTH_BASE_DIR, 'locale'),
    os.path.join(HORIZON_BASE_DIR, 'locale'),
]

try:
    import tionix_dashboard_theme
except ImportError:
    pass
else:

    TIONIX_DASHBOARD_THEME_BASE_DIR = os.path.dirname(
        tionix_dashboard_theme.__file__
    )

    LOCALE_PATHS.append(os.path.join(TIONIX_DASHBOARD_THEME_BASE_DIR,
                                     'locale'))

try:
    import openstack_dashboard
except ImportError:
    pass
else:
    from openstack_dashboard.settings import MIDDLEWARE_CLASSES, HORIZON_CONFIG

    OS_BASE_DIR = os.path.dirname(openstack_dashboard.__file__)

    LOCALE_PATHS.append(os.path.join(OS_BASE_DIR, 'locale'))

    MIDDLEWARE_CLASSES += (
        'django_user_agents.middleware.UserAgentMiddleware',
        'tionix_dashboard.dashboards.middleware.TionixMiddleware',
        'django_tools.middlewares.ThreadLocal.ThreadLocalMiddleware',
    )
    HORIZON_CONFIG.update(help_url=get_help_url())

try:
    from openstack_dashboard.local.local_settings import LOGGING
except ImportError:
    pass
else:
    LOGGING['loggers'].update({
        'tionix_dashboard': {
            'handlers': ['console'],
            'level': 'DEBUG',
            'propagate': False,
        },
        'urllib3.connectionpool': {
            'handlers': ['console'],
            'level': 'ERROR',
            'propagate': False,
        },
    })

TEMPLATE_DIRS = (
    os.path.join(OPENSTACK_AUTH_BASE_DIR, 'templates'),
    os.path.join(TNX_DASH_BASE_DIR, 'templates'),
)

SITE_BRANDING_LINK = reverse_lazy('horizon:tionix:licenses:index')

KEYSTONE = {
    'auth_version': '3.0',
    'auth_url': 'http://localhost:5000',
    'auth_user': 'admin',
    'auth_password': 'admin',
    'auth_tenant': 'admin',
    'compute_service_name': 'compute',
    'volume_service_name': 'volumev2',
    'network_service_name': 'network',
    'image_service_name': 'image',
    'identity_service_name': 'identity',
    'user_domain_name': 'default',
    'project_domain_name': 'default'
}

NOVA_VERSION = 2.25
NEUTRON_VERSION = 2
GLANCE_VERSION = 1

# Timeout for log and console rest api
LOG_CONSOLE_TIMEOUT = 5

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.mysql',
        'NAME': 'tionix_dash',
        'USER': 'tionix',
        'PASSWORD': 'password',
        'HOST': 'localhost',
        'PORT': '3306',
    }
}

CHECK_UPDATES_TIMEOUT = 3600
MOD_VERSIONS_CACHE_KEY = 'tionix_modules_versions'

if os.environ.get('DJANGO_SETTINGS_MODULE') == 'openstack_dashboard.settings':
    logging.config.dictConfig(LOGGING)
    logging.getLogger().setLevel(LOG_LEVEL)

    base_config_path = '/etc/tionix/tionix.yaml'
    module_config_path = '/etc/tionix/dashboard.yaml'

    module_settings = defaultdict(dict)
    gl = globals()

    update_module_settings(
        module_settings, gl, module_config_path, base_config_path)

    if module_settings['LOG_LEVEL']:
        logging.getLogger().setLevel(module_settings['LOG_LEVEL'])

    update_global_from_settings(module_settings, gl)

    logging.getLogger().setLevel(LOG_LEVEL)
