# coding: utf-8

import logging
import os
import pkgutil
from collections import defaultdict
from urlparse import urlparse

import yaml
from django.db import backends
from keystoneclient import discover

logger = logging.getLogger(__name__)

ENGINE_PREFIX = 'django.db.backends.'
duplicate_keys = set()

immutable_variables = ['NOVA_VERSION']
unused_variables = ['CINDER_VERSION', 'RABBIT_QUEUES',
                    'ceilometer_service_name']
pass_keys = ['auth_password', 'PASSWORD', 'password']

available_log_levels = ('CRITICAL', 'ERROR', 'WARN', 'WARNING', 'INFO',
                        'DEBUG', 'NOTSET')


def get_available_db_engines():
    return [
        ENGINE_PREFIX + name
        for _importer, name, ispkg in pkgutil.iter_modules(backends.__path__)
        if ispkg and name != 'dummy']


def duplicates_detect_constructor(loader, node, deep=False):
    """Detect duplicate keys.
       Function have no effect when using yaml.safe_load()
       Doesn't work correctly when the key names in different
       sections the same
    """
    entry = {}
    for key_node, value_node in node.value:
        key = loader.construct_object(key_node, deep=deep)
        value = loader.construct_object(value_node, deep=deep)

        if key in entry:
            duplicate_keys.add(key)

        entry[key] = value
    return loader.construct_mapping(node, deep)


yaml.add_constructor(yaml.resolver.BaseResolver.DEFAULT_MAPPING_TAG,
                     duplicates_detect_constructor)


def read_settings(settings):
    duplicate_keys.clear()

    with open(settings) as fh:
        base_config = yaml.load(fh)

        if duplicate_keys:
            logger.error(
                u'Duplicate options found in file {0}: {1}. '.format(
                    settings, list(duplicate_keys)))

    duplicate_keys.clear()

    return base_config


def merge_dicts(base_dict, update_dict):
    for key, value in update_dict.iteritems():
        base = base_dict.get(key)
        if isinstance(base, dict) and isinstance(value, dict):
            base_dict[key] = merge_dicts(base, value)
        else:
            base_dict[key] = value

    return base_dict


def update_module_settings(result_settings, gl, module_config_path,
                           base_config_path):
    path = defaultdict(dict)

    if os.path.exists(base_config_path):
        path = read_settings(base_config_path)
    if os.path.exists(module_config_path):
        path = merge_dicts(path, read_settings(module_config_path))

    for key, value in path.iteritems():
        update_key(result_settings, key, value, gl, log_changes=False)


def update_global_from_settings(module_settings, gl):
    """ Update globals from module settings """
    for key, value in module_settings.iteritems():
        update_key(gl, key, value, log_changes=True)


def update_key(config, key, value, gl=None, parents='', log_changes=False):
    """ Recursively update values of nested dict """
    base_config = gl or config
    if key in base_config:
        if isinstance(value, dict):
            for k, v in value.iteritems():
                glob = base_config[key] if gl else None
                formatted_key = '{0}{1}.'.format(parents, key)
                update_key(config[key], k, v, gl=glob,
                           parents=formatted_key, log_changes=log_changes)
        else:
            config[key] = get_valid_value(
                base_config, key, value, parents, log_changes=log_changes)
    elif key == 'DB':
        if 'default' not in config['DATABASES']:
            config['DATABASES']['default'] = dict()
        for k, v in value.iteritems():
            update_key(
                config['DATABASES']['default'], k, v,
                gl['DATABASES']['default'],
                parents='DATABASES.default.',
                log_changes=log_changes
            )
    elif key in unused_variables:
        pass
    else:
        logger.error(u'{0}{1} is not found in settings.'.format(
            parents, key))


def get_valid_value(gl, key, value, parent='', log_changes=False):
    default = gl[key]
    if key in immutable_variables:
        value = default
    elif duplicate_keys and key in duplicate_keys:
        logger.error(
            u'Value for duplicate option "{0}{1}" '
            u'set to default: {2} '.format(parent, key, default))
        value = default
    else:
        if key in validators:
            value = validators[key](key, value, default)
        if default != value and log_changes:
            showed_default = default
            showed_value = value
            if key in pass_keys:
                showed_default = '*' * len(default)
                showed_value = '*' * len(value)
            logger.debug(u'Changed value of "{0}{1}" from "{2}"'
                         u' to "{3}"'.format(parent, key,
                                             showed_default,
                                             showed_value))
    return value


def get_valid_port(key, value=None, default_port=None):
    val = unicode(value)
    if not val.isdigit():
        logger.error(
            u'Invalid port: {0}. '
            u'Port must consist only of numbers. '
            u'Port set to default: {1}'.format(value, default_port))

        value = default_port
    return value


def get_valid_db_engine(key, value, default_db_engine):
    processed_value = value.lower()
    available_engines = get_available_db_engines()

    if not processed_value.startswith(ENGINE_PREFIX):
        processed_value = ENGINE_PREFIX + processed_value

    if processed_value not in available_engines:
        logger.error(
            u'Invalid DB engine: {0}. '
            u'Supporting DB engines: "{1}". '
            u'DB engine set to: {2}'.format(
                value, available_engines, default_db_engine))

        processed_value = default_db_engine
    return processed_value


def get_valid_keystone_auth_version(key, value, default_auth_version):
    valid = True
    valid_keystone_versions = discover._CLIENT_VERSIONS.keys()
    val = unicode(value)
    try:
        val = float(val.replace('v', '', 1))
    except ValueError:
        valid = False
    else:
        if val not in valid_keystone_versions:
            valid = False

    if not valid:
        logger.error(
            u'Invalid keystone auth version: {0}. '
            u'Keystone auth version set to default: {1}'.format(
                value, default_auth_version))

        value = default_auth_version

    return value


def get_valid_keystone_auth_url(key, value, default_auth_url):
    parsed_url = urlparse(value)
    invalid_port = False

    try:
        parsed_url.port
    except ValueError:
        invalid_port = True

    if not parsed_url.scheme or not parsed_url.hostname or invalid_port:
        logger.error(
            u'Invalid keystone auth url: {0}. '
            u'Url format for example {1}. '
            u'Keystone auth url set to default: {1}'.format(
                value, default_auth_url))

        value = default_auth_url
    else:
        url = u'{scheme}://{addr}'.format(
            scheme=parsed_url.scheme,
            addr=parsed_url.netloc,
        )
        if url != value:
            logger.debug(
                u"Normalized keystone_auth_url. "
                u"Changed '{0}' to '{1}'".format(value, url)
            )
            value = url
    return value


def get_valid_neutron_api_version(key, value, default_neutron_version):

    available_major_versions = ['2']

    val = unicode(value)
    major_version = val.replace('v', '').split('.')[0]
    if available_major_versions and (
            major_version not in available_major_versions):
        logger.error(
            u'Invalid neutron version: {0}. '
            u'Available major versions are {1}. '
            u'Cinder version set to default: {2}'.format(
                value, available_major_versions, default_neutron_version))

        value = default_neutron_version
    return value


def get_valid_log_level(key, value, default_log_level):
    processed_value = value.upper()

    if processed_value not in available_log_levels:
        processed_value = default_log_level

        logger.error(
            u'Invalid log level: {0}. '
            u'Log level must be one of {1}. '
            u'Log level set to default: {2}'.format(
                value, available_log_levels, default_log_level))

    return processed_value


def get_valid_int(key, value, default):
    value = unicode(value)
    if not (value.isdigit() and int(value) > 0):
        logger.error(
            u'Invalid {0}: {1}. '
            u'{0} must consist only of numbers. '
            u'{0} set to default: {2}'.format(key, value, default))

        value = default
    return int(value)


def get_valid_bool(key, value, default):
    if value not in (True, False, 1, 0):
        logger.error(
            u'Invalid {0}: {1}. '
            u'{0} must be boolean type. '
            u'{0} set to default: {2}'.format(key, value, default))

        value = default
    return value


def get_valid_str(key, value, default):
    return unicode(value)


validators = {
    'LOG_LEVEL': get_valid_log_level,
    'NEUTRON_VERSION': get_valid_neutron_api_version,
    'auth_url': get_valid_keystone_auth_url,
    'auth_version': get_valid_keystone_auth_version,
    'ENGINE': get_valid_db_engine,
    'PORT': get_valid_port,
    'LOG_CONSOLE_TIMEOUT': get_valid_int,
    'auth_password': get_valid_str,
    'password': get_valid_str,
    'PASSWORD': get_valid_str,
}
